@@ -22,9 +22,13 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "ad1848.h"
-#include "initval.h"
+#include "../include/driver.h"
+#include "../include/ad1848.h"
+#include "../include/initval.h"
+
+MODULE_DESCRIPTION("\
+Driver: AD1848/AD1847/CS4248\n\
+");
 
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
@@ -32,20 +36,18 @@ int snd_port[SND_CARDS] = SND_DEFAULT_PORT;	/* PnP setup */
 int snd_irq[SND_CARDS] = SND_DEFAULT_IRQ;	/* 5,7,9,11,12,15 */
 int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
-#ifdef MODULE_PARM
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for AD1848 soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for AD1848 soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for AD1848 driver.");
+MODULE_PARM_DESC(snd_port, "Port # for AD1848 driver. [PORT12]");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for AD1848 driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for AD1848 driver. [IRQ]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for AD1848 driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for AD1848 driver. [DMA]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for AD1848 driver.");
-#endif
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for AD1848 driver. [DMA_SIZE]");
 
 static struct snd_card_ad1848 {
 	snd_irq_t *irqptr;
@@ -67,19 +69,17 @@ static void snd_card_ad1848_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static snd_pcm_t *snd_card_ad1848_detect(snd_card_t * card,
-					  unsigned short port,
-					  snd_irq_t * irqptr,
-					  snd_dma_t * dma1ptr)
+static snd_pcm_t * __init snd_card_ad1848_detect(snd_card_t * card,
+						 unsigned short port,
+						 snd_irq_t * irqptr,
+						 snd_dma_t * dma1ptr)
 {
 	snd_pcm_t *pcm;
 
 	if (snd_register_ioport(card, port, 4, "AD1848", NULL) < 0)
 		return NULL;
 
-	pcm = snd_ad1848_new_device(card, port, irqptr, dma1ptr, AD1848_HW_DETECT);
-
-	if (!pcm) {
+	if (snd_ad1848_new_pcm(card, 0, port, irqptr, dma1ptr, AD1848_HW_DETECT, &pcm) < 0) {
 		snd_unregister_ioports(card);
 		return NULL;
 	}
@@ -102,8 +102,9 @@ static void snd_card_ad1848_interrupt(int irq, void *dev_id, struct pt_regs *reg
 		snd_ad1848_interrupt(acard->pcm, status);
 }
 
-static int snd_card_ad1848_resources(int dev, struct snd_card_ad1848 *acard,
-				      snd_card_t * card)
+static int __init snd_card_ad1848_resources(int dev,
+					    struct snd_card_ad1848 *acard,
+					    snd_card_t * card)
 {
 	static int possible_irqs[] = {-1};
 	static int possible_dmas[] = {-1};
@@ -124,17 +125,16 @@ static int snd_card_ad1848_resources(int dev, struct snd_card_ad1848 *acard,
 	return 0;
 }
 
-static int snd_card_ad1848_probe(int dev, struct snd_card_ad1848 *acard)
+static int __init snd_card_ad1848_probe(int dev, struct snd_card_ad1848 *acard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
-	snd_pcm1_t *pcm1;
 	snd_kmixer_t *mixer = NULL;
 	ad1848_t *codec;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 		     snd_card_ad1848_use_inc, snd_card_ad1848_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 
 	card->type = SND_CARD_TYPE_AD1848;
@@ -145,25 +145,16 @@ static int snd_card_ad1848_probe(int dev, struct snd_card_ad1848 *acard)
 	pcm = snd_card_ad1848_detect(card, snd_port[dev],
 				     acard->irqptr, acard->dma1ptr);
 
-	if (!pcm) {
+	if (pcm == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (ad1848_t *) pcm1->private_data;
+	codec = snd_magic_cast(ad1848_t, pcm->private_data, -ENXIO);
 	acard->codec = codec;
 	acard->pcm_status_reg = codec->port + 2;
 
-	mixer = snd_ad1848_new_mixer(pcm, 0);
-	if (!mixer)
+	if (snd_ad1848_new_mixer(pcm, 0, &mixer) < 0)
 		goto __nodev;
-	if (snd_mixer_register(mixer, 0) < 0)
-		goto __nodev;
-	if (snd_pcm_register(pcm, 0)) {
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
-	}
 	strcpy(card->abbreviation, "AD1848");
 	strcpy(card->shortname, pcm->name);
 
@@ -179,41 +170,33 @@ static int snd_card_ad1848_probe(int dev, struct snd_card_ad1848 *acard)
 		acard->mixer = mixer;
 		return 0;
 	}
-	snd_mixer_unregister(mixer);
-	snd_pcm_unregister(pcm);
-	snd_card_free(card);
-	return -ENOMEM;
 
       __nodev:
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm)
-		snd_pcm_free(pcm);
 	snd_card_free(card);
-
 	return -ENXIO;
 }
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int alsa_card_ad1848_init(void)
+#endif
 {
 	int dev, cards;
 	struct snd_card_ad1848 *acard;
 
-#ifndef LINUX_2_1
-	register_symtab(NULL);
-#endif
-
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_card_ad1848 *)
-		    snd_malloc(sizeof(struct snd_card_ad1848));
-		if (!acard)
+		    snd_kcalloc(sizeof(struct snd_card_ad1848), GFP_KERNEL);
+		if (acard == NULL)
 			continue;
-		memset(acard, 0, sizeof(struct snd_card_ad1848));
 		if (snd_card_ad1848_probe(dev, acard) < 0) {
-			snd_free(acard, sizeof(struct snd_card_ad1848));
+			snd_kfree(acard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk("AD1848 soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_card_ad1848_cards[dev] = acard;
@@ -221,13 +204,17 @@ int init_module(void)
 	}
 
 	if (!cards) {
+#ifdef MODULE
 		snd_printk("AD1848 soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __init cleanup_module(void)
 {
 	int idx;
 	struct snd_card_ad1848 *acard;
@@ -236,14 +223,9 @@ void cleanup_module(void)
 		acard = snd_card_ad1848_cards[idx];
 		if (acard) {
 			snd_card_unregister(acard->card);
-
-			if (acard->mixer)
-				snd_mixer_unregister(acard->mixer);
-			if (acard->pcm) {
-				snd_pcm_unregister(acard->pcm);
-			}
-			snd_card_free(acard->card);
-			snd_free(acard, sizeof(struct snd_card_ad1848));
+			snd_kfree(acard);
 		}
 	}
 }
+
+#endif
