@@ -1,7 +1,7 @@
 
 /*
     card-als100.c - driver for Avance Logic ALS100 based soundcards.
-    Copyright (C) 1999 by Massimo Piccioni <piccio@caronte.csr.unibo.it>
+    Copyright (C) 1999-2000 by Massimo Piccioni <dafastidio@libero.it>
 
     Thanks to Pierfrancesco 'qM2' Passerini.
 
@@ -20,6 +20,7 @@
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */
 
+#define __SND_OSS_COMPAT__
 #define SND_MAIN_OBJECT_FILE
 #include "../include/driver.h"
 #include "../include/initval.h"
@@ -27,6 +28,19 @@
 #include "../include/opl3.h"
 #include "../include/sb.h"
 
+MODULE_DESCRIPTION("\
+Driver: Avance Logic ALS1X0\n\
+Driver: 3D Melody MF1000n\
+Card: Avance Logic ALS100\n\
+Card: Avance Logic ALS110\n\
+Card: Avance Logic ALS120\n\
+Card: Digimate 3D Sound\n\
+ISAPNP: ALS0001=@@@0001,@X@0001,@H@0001\n\
+ISAPNP: ALS0110=@@@1001,@X@1001,@H@1001\n\
+ISAPNP: ALS0120=@@@2001,@X@2001,@H@2001\n\
+ISAPNP: RTL3000=@@@2001,@X@2001,@H@2001\n\
+");
+
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;		/* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;		/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;		/* PnP setup */
@@ -45,23 +59,23 @@ MODULE_PARM_DESC(snd_index, "Index value for als100 based soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for als100 based soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for als100 driver.");
+MODULE_PARM_DESC(snd_port, "Port # for als100 driver. [PORT12]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for als100 driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for als100 driver. [PORT12]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for als100 driver.");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for als100 driver. [PORT12]");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for als100 driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for als100 driver. [IRQ]");
 MODULE_PARM(snd_mpu_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for als100 driver.");
+MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for als100 driver. [IRQ]");
 MODULE_PARM(snd_dma8, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for als100 driver.");
+MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for als100 driver. [DMA8]");
 MODULE_PARM(snd_dma16, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for als100 driver.");
+MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for als100 driver. [DMA16]");
 MODULE_PARM(snd_dma8_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for als100 driver.");
+MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for als100 driver. [DMA8_SIZE]");
 MODULE_PARM(snd_dma16_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for als100 driver.");
+MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for als100 driver. [DMA16_SIZE]");
 #endif	/* MODULE_PARM */
 
 static struct snd_card_als100 {
@@ -74,26 +88,43 @@ static struct snd_card_als100 {
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *rmidi;
 	snd_hwdep_t *synth;
-#ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *logdev;
-	struct isapnp_logdev *logdevmpu;
-	struct isapnp_logdev *logdevopl;
-#endif	/* CONFIG_ISAPNP */
+#ifdef __ISAPNP__
+	struct isapnp_dev *dev;
+	struct isapnp_dev *devmpu;
+	struct isapnp_dev *devopl;
+#endif	/* __ISAPNP__ */
 } *snd_card_als100_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 static unsigned int snd_card_als100_pnpids[] = {
+	/* ALS100 - PRO16PNP */
 	(ISAPNP_VENDOR('A','L','S')<<16)|ISAPNP_DEVICE(0x0001),    /* device */
 	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x0001),  /* audio */
 	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x0001),  /* mpu401 */
 	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x0001),  /* opl */
+	/* ALS110 - MF1000 - Digimate 3D Sound */
+	(ISAPNP_VENDOR('A','L','S')<<16)|ISAPNP_DEVICE(0x0110),    /* device */
+	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x1001),  /* audio */
+	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x1001),  /* mpu401 */
+	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x1001),  /* opl */
+	/* ALS120 */
+	(ISAPNP_VENDOR('A','L','S')<<16)|ISAPNP_DEVICE(0x0120),    /* device */
+	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* audio */
+	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* mpu401 */
+	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* opl */
+	/* RTL3000 */
+	(ISAPNP_VENDOR('R','T','L')<<16)|ISAPNP_DEVICE(0x3000),    /* device */
+	(ISAPNP_VENDOR('@','@','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* audio */
+	(ISAPNP_VENDOR('@','X','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* mpu401 */
+	(ISAPNP_VENDOR('@','H','@')<<16)|ISAPNP_FUNCTION(0x2001),  /* opl */
 	0       /* end */
 };
-#endif	/* CONFIG_ISAPNP */
+#endif	/* __ISAPNP__ */
 
 #define DRIVER_NAME	"snd-card-als100"
 
-#define snd_als100_printk(args...)	snd_printk(DRIVER_NAME": " ##args)
+#define snd_als100_printk(args...)	snd_printk(__FILE__": " ##args)
+
 
 static void snd_card_als100_use_inc(snd_card_t *card)
 {
@@ -106,7 +137,7 @@ static void snd_card_als100_use_dec(snd_card_t *card)
 }
 
 static void snd_card_als100_interrupt(int irq, void *dev_id,
-							struct pt_regs *regs)
+				      struct pt_regs *regs)
 {
 	struct snd_card_als100 *acard;
 	snd_pcm_t *pcm;
@@ -120,11 +151,12 @@ static void snd_card_als100_interrupt(int irq, void *dev_id,
 	if (!(pcm = acard->pcm))
 		return;
 
-	if (!(sb16dsp = (sbdsp_t *) pcm->private_data))
+	sb16dsp = snd_magic_cast(sbdsp_t, pcm->private_data, );
+	if (sb16dsp == NULL)
 		return;
 
 	spin_lock_irqsave(&sb16dsp->mixer.lock, flags);
-	status = snd_sb16mixer_read(&sb16dsp->mixer, 0x82);
+	status = snd_sb16mixer_read(&sb16dsp->mixer, SB_DSP4_IRQSTATUS);
 	spin_unlock_irqrestore(&sb16dsp->mixer.lock, flags);
 
 	if (status & 0x03)
@@ -132,7 +164,7 @@ static void snd_card_als100_interrupt(int irq, void *dev_id,
 }
 
 static void snd_card_als100_mpu_interrupt(int irq, void *dev_id,
-							struct pt_regs *regs)
+					  struct pt_regs *regs)
 {
 	struct snd_card_als100 *acard;
 
@@ -142,8 +174,9 @@ static void snd_card_als100_mpu_interrupt(int irq, void *dev_id,
 	snd_mpu401_uart_interrupt(acard->rmidi);
 }
 
-static int snd_card_als100_resources(int dev, struct snd_card_als100 *acard,
-							snd_card_t *card)
+static int __init snd_card_als100_resources(int dev,
+					    struct snd_card_als100 *acard,
+					    snd_card_t *card)
 {
 	int error;
 	static int no_alternatives[] = {-1};
@@ -185,121 +218,114 @@ static int snd_card_als100_resources(int dev, struct snd_card_als100 *acard,
 	return 0;
 }
 
-#ifdef CONFIG_ISAPNP
-static int snd_card_als100_isapnp(int dev, struct snd_card_als100 *acard)
+#ifdef __ISAPNP__
+static int __init snd_card_als100_isapnp(int dev, struct snd_card_als100 *acard)
 {
-        int i;
+	static int idx = 0;
+	static struct isapnp_card *card = NULL;
         unsigned int tmp;
 	struct isapnp_dev *pdev;
-	struct isapnp_config cfg;
 
-      	pdev = NULL;
-	for (i = 0; (tmp = snd_card_als100_pnpids[i]); i+=4)
-		if ((pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev)))
+      __again:
+	while ((tmp = snd_card_als100_pnpids[idx]) != 0) {
+		if ((card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card)))
 			break;
-	if (!pdev)
+		idx += 4;
+	}
+	if (card == NULL)
 		return -ENODEV;
 
-	tmp = snd_card_als100_pnpids[i+1];
-	acard->logdev = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->logdev)
-		return -ENXIO;
+	tmp = snd_card_als100_pnpids[idx+1];
+	acard->dev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (acard->dev == NULL)
+		goto __again;
+	if (acard->dev->active)
+		goto __again;
 
-	tmp = snd_card_als100_pnpids[i+2];
-	acard->logdevmpu = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	tmp = snd_card_als100_pnpids[i+3];
-	acard->logdevopl = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
+	tmp = snd_card_als100_pnpids[idx+2];
+	acard->devmpu = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	tmp = snd_card_als100_pnpids[idx+3];
+	acard->devopl = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
 
-	if (isapnp_cfg_begin(acard->logdev->dev->csn, acard->logdev->number))
+	pdev = acard->dev;
+	if (pdev->prepare(pdev)<0)
 		return -EAGAIN;
-	if (isapnp_config_init(&cfg, acard->logdev)) {
-		isapnp_cfg_end();
-		return -EAGAIN;
-	}
 
 	if (snd_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_port[dev], 16);
 	if (snd_dma8[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma8[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma8[dev],
+			1);
 	if (snd_dma16[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma16[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma16[dev],
+			1);
 	if (snd_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq[dev], 1);
 
-	if (isapnp_configure(&cfg)) {
+	if (pdev->activate(pdev)<0) {
 		snd_als100_printk("AUDIO isapnp configure failure\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
 
-	snd_port[dev] = cfg.port[0];
-	snd_dma8[dev] = cfg.dma[1];
-	snd_dma16[dev] = cfg.dma[0];
-	snd_irq[dev] = cfg.irq[0];
-        isapnp_activate(acard->logdev->number);
+	snd_port[dev] = pdev->resource[0].start;
+	snd_dma8[dev] = pdev->dma_resource[1].start;
+	snd_dma16[dev] = pdev->dma_resource[0].start;
+	snd_irq[dev] = pdev->irq_resource[0].start;
 
-	if (!acard->logdevmpu || isapnp_config_init(&cfg, acard->logdevmpu)) {
-		isapnp_cfg_end();
+	pdev = acard->devmpu;
+	if (pdev == NULL || pdev->prepare(pdev)<0) {
+		snd_mpu_port[dev] = -1;
 		return 0;
 	}
 
 	if (snd_mpu_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_mpu_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_mpu_port[dev],
+			2);
 	if (snd_mpu_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_mpu_irq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_mpu_irq[dev],
+			1);
 
-	if (isapnp_configure(&cfg)) {
+	if (pdev->activate(pdev)<0) {
 		snd_als100_printk("MPU-401 isapnp configure failure\n");
 		snd_mpu_port[dev] = -1;
-		isapnp_deactivate(acard->logdevmpu->number);
-		acard->logdevmpu = NULL;
-	}
-	else {
-		snd_mpu_port[dev] = cfg.port[0];
-		snd_mpu_irq[dev] = cfg.irq[0];
-		isapnp_activate(acard->logdevmpu->number);
+		acard->devmpu = NULL;
+	} else {
+		snd_mpu_port[dev] = pdev->resource[0].start;
+		snd_mpu_irq[dev] = pdev->irq_resource[0].start;
 	}
 
-	if (!acard->logdevopl || isapnp_config_init(&cfg, acard->logdevopl)) {
-		isapnp_cfg_end();
+	pdev = acard->devopl;
+	if (pdev == NULL || pdev->prepare(pdev)<0) {
+		snd_fm_port[dev] = -1;
 		return 0;
 	}
 
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_fm_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_fm_port[dev], 4);
 
-	if (isapnp_configure(&cfg)) {
+	if (pdev->activate(pdev)<0) {
 		snd_als100_printk("OPL isapnp configure failure\n");
 		snd_fm_port[dev] = -1;
-		isapnp_deactivate(acard->logdevopl->number);
-		acard->logdevopl = NULL;
-	}
-	else {
-		snd_fm_port[dev] = cfg.port[0];
-		isapnp_activate(acard->logdevopl->number);
+		acard->devopl = NULL;
+	} else {
+		snd_fm_port[dev] = pdev->resource[0].start;
 	}
 
-	isapnp_cfg_end();
 	return 0;
 }
 
 static void snd_card_als100_deactivate(struct snd_card_als100 *acard)
 {
-	if (!acard->logdev)
-		return;
-
-	if (isapnp_cfg_begin(acard->logdev->dev->csn, acard->logdev->number)<0)
-		return;
-	isapnp_deactivate(acard->logdev->number);
-	if (acard->logdevmpu)
-		isapnp_deactivate(acard->logdevmpu->number);
-	if (acard->logdevopl)
-		isapnp_deactivate(acard->logdevopl->number);
-	isapnp_cfg_end();
+	if (acard->dev)
+		acard->dev->deactivate(acard->dev);
+	if (acard->devmpu)
+		acard->devmpu->deactivate(acard->devmpu);
+	if (acard->devopl)
+		acard->devopl->deactivate(acard->devopl);
 }
-#endif	/* CONFIG_ISAPNP */
+#endif	/* __ISAPNP__ */
 
-static int snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
+static int __init snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
 {
 	int error;
 	sbdsp_t *codec;
@@ -310,69 +336,48 @@ static int snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
 	snd_hwdep_t *synth;
 
 	if (!(card = snd_card_new(snd_index[dev], snd_id[dev],
-			    snd_card_als100_use_inc, snd_card_als100_use_dec)))
+			snd_card_als100_use_inc, snd_card_als100_use_dec)))
 		return -ENOMEM;
-	card->type = SND_CARD_TYPE_SB_16;
+	card->type = SND_CARD_TYPE_ALS100;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	if ((error = snd_card_als100_isapnp(dev, acard)))
 		goto __on_error;
 #else
 	snd_als100_printk("you have to enable PnP support ...\n");
 	return -ENOSYS;
-#endif	/* CONFIG_ISAPNP */
+#endif	/* __ISAPNP__ */
 
 	if ((error = snd_card_als100_resources(dev, acard, card)))
 		goto __on_error;
 
-	if (!(pcm = snd_sb16dsp_new_device(card, snd_port[dev], acard->irqptr,
-				acard->dma8ptr, acard->dma16ptr, SB_HW_ALS100)))
+	if (snd_sb16dsp_new_pcm(card, 0, snd_port[dev], acard->irqptr,
+		        acard->dma8ptr, acard->dma16ptr,
+		        SB_HW_ALS100, &pcm) < 0)
 		goto __on_error;
 
-	codec = (sbdsp_t *) pcm->private_data;
-	if (!(mixer = snd_sb16dsp_new_mixer(card, &codec->mixer,
-							codec->hardware, 0))) {
-		snd_pcm_free(pcm);
+	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);
+	if (snd_sb16dsp_new_mixer(codec, 0, pcm, &mixer) < 0)
 		goto __on_error;
-	}
-	if ((error = snd_mixer_register(mixer, 0 ))) {
-		snd_mixer_free(mixer);
-		snd_pcm_free(pcm);
-		goto __on_error;
-	}
 
 	rmidi = NULL;
 	if (snd_mpu_port[dev] > 0) {
-		if (!(rmidi = snd_mpu401_uart_new_device(card,
+		if (snd_mpu401_uart_new(card, 0,
 				MPU401_HW_ALS100,
-				snd_mpu_port[dev], snd_mpu_irq[dev])))
+				snd_mpu_port[dev],
+				snd_mpu_irq[dev],
+				&rmidi) < 0)
 			snd_als100_printk("no MPU-401device at 0x%x ?\n",
 				snd_mpu_port[dev]);
-		else if (snd_rawmidi_register(rmidi, 0)) {
-			snd_rawmidi_free(rmidi);
-			rmidi = NULL;
-		}
 	}
 
 	synth = NULL;
 	if (snd_fm_port[dev] > 0) {
-		if (!(synth = snd_opl3_new_device(card,
-				snd_fm_port[dev], snd_fm_port[dev] + 2,
-				OPL3_HW_AUTO, 0)))
+		if (snd_opl3_new(card, 0,
+				 snd_fm_port[dev], snd_fm_port[dev] + 2,
+				 OPL3_HW_AUTO, 0, &synth) < 0)
 			snd_als100_printk("no OPL device at 0x%x-0x%x ?\n",
 				snd_fm_port[dev], snd_fm_port[dev] + 2);
-		else if (snd_hwdep_register(synth, 0)) {
-			snd_hwdep_free(synth);
-			synth = NULL;
-		}
-	}
-
-	if ((error = snd_pcm_register(pcm, 0))) {
-		snd_hwdep_unregister(synth);
-		snd_rawmidi_unregister(rmidi);
-		snd_mixer_unregister(mixer);
-		snd_pcm_free(pcm);
-		goto __on_error;
 	}
 
 	strcpy(card->abbreviation, "ALS100");
@@ -389,66 +394,64 @@ static int snd_card_als100_probe(int dev, struct snd_card_als100 *acard)
 		return 0;
 	}
 
-	snd_hwdep_unregister(synth);
-	snd_rawmidi_unregister(rmidi);
-	snd_mixer_unregister(mixer);
-	snd_pcm_free(pcm);
-
       __on_error:
 	snd_card_free(card);
 	return error ? error : -ENXIO;
 }
 
-static void snd_card_als100_free(int dev)
+#ifdef MODULE
+
+static void __exit snd_card_als100_free(int dev)
 {
 	struct snd_card_als100 *acard;
 
 	if ((acard = snd_card_als100_cards[dev])) {
 		snd_card_als100_cards[dev] = NULL;
 		snd_card_unregister(acard->card);
-		if (acard->synth)
-			snd_hwdep_unregister(acard->synth);
-		if (acard->rmidi)
-			snd_rawmidi_unregister(acard->rmidi);
-		if (acard->mixer)
-			snd_mixer_unregister(acard->mixer);
-		if (acard->pcm)
-			snd_pcm_unregister(acard->pcm);
-		snd_card_free(acard->card);
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 		snd_card_als100_deactivate(acard);
-#endif	/* CONFIG_ISAPNP */
+#endif	/* __ISAPNP__ */
 		snd_kfree(acard);
 	}
 }
 
-int init_module(void)
+#endif
+
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_card_als100_init(void)
+#endif
 {
 	int cards, dev;
 	struct snd_card_als100 *acard;
 
 	for (dev = cards = 0; dev < SND_CARDS; dev++) {
 		acard = (struct snd_card_als100 *) snd_kcalloc(
-				sizeof(struct snd_card_als100), GFP_KERNEL);
-		if (!acard)
+			sizeof(struct snd_card_als100), GFP_KERNEL);
+		if (acard == NULL)
 			break;
 
 		if (snd_card_als100_probe(dev, acard) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_card_als100_deactivate(acard);
-#endif	/* CONFIG_ISAPNP */
+#endif	/* __ISAPNP__ */
 			snd_kfree(acard);
 			continue;
 		}
 		snd_card_als100_cards[cards++] = acard;
 	}
 
+#ifdef MODULE
 	if (!cards)
-		snd_printk("no ALS100 based soundcards found\n");
+		snd_als100_printk("no ALS100 based soundcards found\n");
+#endif
 	return cards ? 0 : -ENODEV;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int dev;
 
@@ -456,3 +459,4 @@ void cleanup_module(void)
 		snd_card_als100_free(dev);
 }
 
+#endif
