@@ -1,5 +1,5 @@
 /*
- *  Driver for generic CS4231 chips
+ *  Generic driver for CS4231 chips
  *  Copyright (c) by Jaroslav Kysela <perex@suse.cz>
  *  Originally the CS4232/CS4232A driver, modified for use on CS4231 by
  *  Tugrul Galatali <galatalt@stuy.edu>
@@ -21,10 +21,14 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "cs4231.h"
-#include "mpu401.h"
-#include "initval.h"
+#include "../include/driver.h"
+#include "../include/cs4231.h"
+#include "../include/mpu401.h"
+#include "../include/initval.h"
+
+MODULE_DESCRIPTION("\
+Driver: Generic CS4231\n\
+");
 
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
@@ -36,28 +40,26 @@ int snd_dma1[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma1_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma2[SND_CARDS] = SND_DEFAULT_DMA;	/* 0,1,3,5,6,7 */
 int snd_dma2_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
-#ifdef MODULE_PARM
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for CS4231 soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for CS4231 soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for CS4231 driver.");
+MODULE_PARM_DESC(snd_port, "Port # for CS4231 driver. [PORT12]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for CS4231 driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for CS4231 driver. [PORT12]");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for CS4231 driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for CS4231 driver. [IRQ]");
 MODULE_PARM(snd_mpu_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for CS4231 driver.");
+MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for CS4231 driver. [IRQ]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for CS4231 driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for CS4231 driver. [DMA]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for CS4231 driver.");
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for CS4231 driver. [DMA_SIZE]");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA2 # for CS4231 driver.");
+MODULE_PARM_DESC(snd_dma2, "DMA2 # for CS4231 driver. [DMA]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for CS4231 driver.");
-#endif
+MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for CS4231 driver. [DMA_SIZE]");
 
 static struct snd_card_cs4231 {
 	snd_irq_t *irqptr;
@@ -82,12 +84,12 @@ static void snd_card_cs4231_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static snd_pcm_t *snd_card_cs4231_detect(snd_card_t * card,
-					 unsigned short port,
-					 unsigned short mpu_port,
-					 snd_irq_t *irqptr,
-					 snd_dma_t *dma1ptr,
-					 snd_dma_t *dma2ptr)
+static snd_pcm_t * __init snd_card_cs4231_detect(snd_card_t * card,
+						 unsigned short port,
+						 unsigned short mpu_port,
+						 snd_irq_t *irqptr,
+						 snd_dma_t *dma1ptr,
+						 snd_dma_t *dma2ptr)
 {
 	snd_pcm_t *pcm;
 
@@ -100,9 +102,8 @@ static snd_pcm_t *snd_card_cs4231_detect(snd_card_t * card,
 			return NULL;
 		}
 	}
-	pcm = snd_cs4231_new_device(card, port, irqptr,
-				    dma1ptr, dma2ptr, CS4231_HW_DETECT, 0);
-	if (!pcm) {
+	if (snd_cs4231_new_pcm(card, 0, port, irqptr,
+			       dma1ptr, dma2ptr, CS4231_HW_DETECT, 0, &pcm) < 0) {
 		snd_unregister_ioports(card);
 		return NULL;
 	}
@@ -132,8 +133,9 @@ static void snd_card_cs4231_midi_interrupt(int irq, void *dev_id, struct pt_regs
 	snd_mpu401_uart_interrupt(acard->rmidi);
 }
 
-static int snd_card_cs4231_resources(int dev, struct snd_card_cs4231 *acard,
-				     snd_card_t * card)
+static int __init snd_card_cs4231_resources(int dev,
+					    struct snd_card_cs4231 *acard,
+					    snd_card_t * card)
 {
 	static int possible_irqs[] = {-1};
 	static int possible_dmas[] = {-1};
@@ -175,18 +177,17 @@ static int snd_card_cs4231_resources(int dev, struct snd_card_cs4231 *acard,
 	return 0;
 }
 
-static int snd_card_cs4231_probe(int dev, struct snd_card_cs4231 *acard)
+static int __init snd_card_cs4231_probe(int dev, struct snd_card_cs4231 *acard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
-	snd_pcm1_t *pcm1;
 	snd_kmixer_t *mixer = NULL;
 	snd_rawmidi_t *rmidi = NULL;
 	cs4231_t *codec;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 		       snd_card_cs4231_use_inc, snd_card_cs4231_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 	card->type = SND_CARD_TYPE_CS4231;
 	if (snd_card_cs4231_resources(dev, acard, card) < 0) {
@@ -200,41 +201,23 @@ static int snd_card_cs4231_probe(int dev, struct snd_card_cs4231 *acard)
 				     acard->irqptr, acard->dma1ptr,
 				     acard->dma2ptr == NULL ?
 				     	acard->dma1ptr : acard->dma2ptr);
-	if (!pcm) {
+	if (pcm == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	codec = (cs4231_t *) pcm1->private_data;
+	codec = snd_magic_cast(cs4231_t, pcm->private_data, -ENXIO);
 	acard->codec = codec;
 	acard->pcm_status_reg = codec->port + 2;
 
-	mixer = snd_cs4231_new_mixer(pcm, 0);
-	if (!mixer)
+	if (snd_cs4231_new_mixer(pcm, 0, &mixer) < 0)
 		goto __nodev;
 
 	if (acard->mpuirqptr) {
-		rmidi = snd_mpu401_uart_new_device(card, MPU401_HW_CS4232,
-						   snd_mpu_port[dev],
-						   acard->mpuirqptr->irq);
-		if (!rmidi)
-			goto __nodev;
-		if (snd_rawmidi_register(rmidi, 0) < 0)
-			goto __nodev;
-	}
-	if (snd_mixer_register(mixer, 0) < 0) {
-		if (rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		goto __nodev;
-	}
-	if (snd_pcm_register(pcm, 0)) {
-		if (rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
+		if (snd_mpu401_uart_new(card, 0, MPU401_HW_CS4232,
+					snd_mpu_port[dev],
+					acard->mpuirqptr->irq,
+					&rmidi) < 0)
+			snd_printk("CS4231: MPU401 not detected\n");
 	}
 	strcpy(card->abbreviation, "CS4231");
 	strcpy(card->shortname, pcm->name);
@@ -253,54 +236,50 @@ static int snd_card_cs4231_probe(int dev, struct snd_card_cs4231 *acard)
 		acard->rmidi = rmidi;
 		return 0;
 	}
-	snd_mixer_unregister(mixer);
-	snd_pcm_unregister(pcm);
-	snd_card_free(card);
-	return -ENOMEM;
 
       __nodev:
-	if (rmidi)
-		snd_rawmidi_free(rmidi);
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm)
-		snd_pcm_free(pcm);
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_card_cs4231_init(void)
+#endif
 {
 	int dev, cards;
 	struct snd_card_cs4231 *acard;
 
-#ifndef LINUX_2_1
-	register_symtab(NULL);
-#endif
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_card_cs4231 *)
-				snd_malloc(sizeof(struct snd_card_cs4231));
-		if (!acard)
+				snd_kcalloc(sizeof(struct snd_card_cs4231), GFP_KERNEL);
+		if (acard == NULL)
 			continue;
-		memset(acard, 0, sizeof(struct snd_card_cs4231));
 		if (snd_card_cs4231_probe(dev, acard) < 0) {
-			snd_free(acard, sizeof(struct snd_card_cs4231));
+			snd_kfree(acard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk("CS4231 soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_card_cs4231_cards[dev] = acard;
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk("CS4231 soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int idx;
 	struct snd_card_cs4231 *acard;
@@ -309,15 +288,9 @@ void cleanup_module(void)
 		acard = snd_card_cs4231_cards[idx];
 		if (acard) {
 			snd_card_unregister(acard->card);
-			if (acard->rmidi)
-				snd_rawmidi_unregister(acard->rmidi);
-			if (acard->mixer)
-				snd_mixer_unregister(acard->mixer);
-			if (acard->pcm) {
-				snd_pcm_unregister(acard->pcm);
-			}
-			snd_card_free(acard->card);
-			snd_free(acard, sizeof(struct snd_card_cs4231));
+			snd_kfree(acard);
 		}
 	}
 }
+
+#endif
