@@ -26,6 +26,51 @@
 #include "../include/opl3.h"
 #include "../include/initval.h"
 
+#ifdef CS4232
+MODULE_DESCRIPTION("\
+Driver: Cirrus Logic CS4232\n\
+Card: Generic CS4232\n\
+Card: Turtle Beach TBS-2000\n\
+Card: Turtle Beach Tropez Plus\n\
+Card: AudioSystem EWS64\n\
+ISAPNP: CSC7532=CSC0000,CSC0010,CSCb006\n\
+ISAPNP: CSC7632=CSC0000,CSC0010,PNPb006\n\
+ISAPNP: CSCa836=CSCa800,CSCa810,CSCa8a3\n\
+");
+#else
+MODULE_DESCRIPTION("\
+Driver: Cirrus Logic CS4236+\n\
+Card: Generic CS4235\n\
+Card: Generic CS4236B\n\
+Card: Generic CS4237B\n\
+Card: Generic CS4238B\n\
+Card: Generic CS4239\n\
+Card: Genius Sound Maker 3DJ\n\
+Card: HP 6330 Onboard sound\n\
+Card: ACER AW35/Pro\n\
+Card: ACER AW37\n\
+Card: ACER AW37/Pro\n\
+Card: EliteGroup P5TX-LA motherboard\n\
+Card: Dell Optiplex GX1\n\
+Card: Turtle Beach Malibu\n\
+Card: Maxi Sound 16 PnP\n\
+Card: Gallant SC-70P\n\
+ISAPNP: CSC0437=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC0b35=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC1425=CSC0100,CSC0110\n\
+ISAPNP: CSC1525=CSC0100,CSC0110,CSC0103\n\
+ISAPNP: CSC4236=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC4237=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC6835=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC7537=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC9836=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSC9837=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSCd925=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSCd937=CSC0000,CSC0010,CSC0003\n\
+ISAPNP: CSCe825=CSC0100,CSC0110\n\
+");
+#endif
+
 #if defined(CS4232)
 #define IDENT "CS4232"
 #else
@@ -53,29 +98,29 @@ MODULE_PARM_DESC(snd_index, "Index value for " IDENT " soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for " IDENT " soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_port, "Port # for " IDENT " driver. [PORT12]");
 MODULE_PARM(snd_cport, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_cport, "Control port # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_cport, "Control port # for " IDENT " driver. [PORT12]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for " IDENT " driver. [PORT12]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for " IDENT " driver. [PORT12]");
 MODULE_PARM(snd_sb_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_sb_port, "SB port # for " IDENT " driver (optional).");
+MODULE_PARM_DESC(snd_sb_port, "SB port # for " IDENT " driver (optional). [PORT12]");
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for " IDENT " driver. [IRQ]");
 MODULE_PARM(snd_mpu_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_mpu_irq, "MPU-401 IRQ # for " IDENT " driver. [IRQ]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for " IDENT " driver. [DMA]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for " IDENT " driver. [DMA_SIZE]");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA2 # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_dma2, "DMA2 # for " IDENT " driver. [DMA]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for " IDENT " driver.");
+MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for " IDENT " driver. [DMA_SIZE]");
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "Enable/disable ISA PnP for " IDENT " driver.");
+MODULE_PARM_DESC(snd_isapnp, "Enable/disable ISA PnP for " IDENT " driver. [BOOL]");
 
 static struct snd_card_cs4236 {
 	snd_irq_t *irqptr;
@@ -89,14 +134,14 @@ static struct snd_card_cs4236 {
 	snd_hwdep_t *synth;
 	cs4231_t *codec;
 	unsigned short pcm_status_reg;
-#ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *wss;
-	struct isapnp_logdev *ctrl;
-	struct isapnp_logdev *mpu;
+#ifdef __ISAPNP__
+	struct isapnp_dev *wss;
+	struct isapnp_dev *ctrl;
+	struct isapnp_dev *mpu;
 #endif
 } *snd_card_cs4236_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 #ifdef CS4232
 static unsigned int snd_card_pnpids[] = {
 	/* Turtle Beach TBS-2000 (CS4232) */
@@ -109,6 +154,11 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('P','N','P')<<16)|ISAPNP_FUNCTION(0xb006), /* MPU */
+	/* AudioSystem EWS64 (CS4232) */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xa836),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa800), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa810), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0xa8a3), /* MPU */
 	/* --- */
 	0	/* end */
 };
@@ -125,6 +175,11 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */	
+	/* CS4235 on mainboard without MPU */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1425),	  /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
+	0,							  /* MPU */
 	/* HP 6330 Onboard sound */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x1525),	  /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
@@ -155,16 +210,26 @@ static unsigned int snd_card_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
-	/* AW37/Pro - CS4235 */
+	/* Gallant SC-70P */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0x9837),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* ACER AW37/Pro - CS4235 */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xd925),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
-	/* AW35/Pro - CS4237B */
+	/* ACER AW35/Pro - CS4237B */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xd937),   /* DEVICE */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0010), /* CTRL */
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0003), /* MPU */
+	/* CS4235 without MPU401 */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_DEVICE(0xe825),   /* DEVICE */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0100), /* WSS */
+	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0110), /* CTRL */
+	0,							  /* MPU */
 	/* --- */
 	0	/* end */
 };
@@ -181,15 +246,15 @@ static void snd_card_cs4236_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static snd_pcm_t *snd_card_cs4236_detect(snd_card_t * card,
-					 unsigned short port,
-					 unsigned short cport,
-					 unsigned short mpu_port,
-					 unsigned short fm_port,
-					 unsigned short sb_port,
-					 snd_irq_t *irqptr,
-					 snd_dma_t *dma1ptr,
-					 snd_dma_t *dma2ptr)
+static snd_pcm_t * __init snd_card_cs4236_detect(snd_card_t * card,
+						 unsigned short port,
+						 unsigned short cport,
+						 unsigned short mpu_port,
+						 unsigned short fm_port,
+						 unsigned short sb_port,
+						 snd_irq_t *irqptr,
+						 snd_dma_t *dma1ptr,
+						 snd_dma_t *dma2ptr)
 {
 	snd_pcm_t *pcm;
 
@@ -221,15 +286,14 @@ static snd_pcm_t *snd_card_cs4236_detect(snd_card_t * card,
 		}
 	}
 #ifdef CS4236
-	pcm = snd_cs4236_new_device(card, port, cport,
-				    irqptr, dma1ptr, dma2ptr,
-				    CS4231_HW_DETECT3, 0);
+	if (snd_cs4236_new_pcm(card, 0, port, cport,
+			       irqptr, dma1ptr, dma2ptr,
+			       CS4231_HW_DETECT3, 0, &pcm) < 0) {
 #else
-	pcm = snd_cs4231_new_device(card, port,
-				    irqptr, dma1ptr, dma2ptr,
-				    CS4231_HW_DETECT, 0);
+	if (snd_cs4231_new_pcm(card, 0, port,
+			       irqptr, dma1ptr, dma2ptr,
+			       CS4231_HW_DETECT, 0, &pcm) < 0) {
 #endif
-	if (!pcm) {
 		snd_unregister_ioports(card);
 		return NULL;
 	}
@@ -259,8 +323,9 @@ static void snd_card_cs4236_midi_interrupt(int irq, void *dev_id, struct pt_regs
 	snd_mpu401_uart_interrupt(acard->rmidi);
 }
 
-static int snd_card_cs4236_resources(int dev, struct snd_card_cs4236 *acard,
-				     snd_card_t * card)
+static int __init snd_card_cs4236_resources(int dev,
+					    struct snd_card_cs4236 *acard,
+					    snd_card_t * card)
 {
 	static int possible_irqs[] = {-1};
 	static int possible_dmas[] = {-1};
@@ -302,156 +367,133 @@ static int snd_card_cs4236_resources(int dev, struct snd_card_cs4236 *acard,
 	return 0;
 }
 
-#ifdef CONFIG_ISAPNP
-static int snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
+#ifdef __ISAPNP__
+static int __init snd_card_cs4236_isapnp(int dev, struct snd_card_cs4236 *acard)
 {
-	int idx;
-	unsigned int tmp;
+	static int idx = 0;
+	static struct isapnp_card *card = NULL;
 	struct isapnp_dev *pdev;
-	struct isapnp_config cfg;
+	unsigned int tmp;
 	
-	/* hmm. looks crazy, but solves most trouble */
-	idx = 0;
       __again:
-      	pdev = NULL;
+      	acard->wss = acard->ctrl = acard->mpu = NULL;
 	while ((tmp = snd_card_pnpids[idx]) != 0) {
-		pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev);
-		if (pdev)
+		card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card);
+		if (card)
 			break;
 		idx += 4;
 	}
-	if (!pdev) {
+	if (card == NULL) {
 		snd_printdd("isapnp failed for " IDENT "\n");
 		return -ENODEV;
 	}
 	tmp = snd_card_pnpids[idx+1];
-	acard->wss = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->wss) {
+	acard->wss = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (acard->wss == NULL) {
 		snd_printdd("isapnp failed for " IDENT " (WSS) - %i\n", idx);
-		idx += 5;
 		goto __again;		/* maybe we have another config */
+	} else if (acard->wss->active) {
+		goto __again;
 	}
 	tmp = snd_card_pnpids[idx+2];
-	acard->ctrl = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->ctrl) {
+	acard->ctrl = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (acard->ctrl == NULL) {
 		snd_printdd("isapnp failed for " IDENT " (CTRL) - %i\n", idx);
-		idx += 5;
 		goto __again;		/* maybe we have another config */
+	} else if (acard->ctrl->active) {
+		goto __again;
 	}
-	if (snd_mpu_port[dev] > 0) {
-		tmp = snd_card_pnpids[idx+3];
-		acard->mpu = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-		if (!acard->mpu) {
+	if (snd_mpu_port[dev] > 0 && (tmp = snd_card_pnpids[idx+3]) != 0) {
+		acard->mpu = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+		if (acard->mpu == NULL) {
 			snd_printdd("isapnp failed for " IDENT " (MPU) - %i\n", idx);
-			idx += 5;
 			goto __again;		/* maybe we have another config */
+		} else if (acard->mpu->active) {
+			goto __again;
 		}
+	} else {
+		snd_mpu_port[dev] = -1;
 	}
-	/* begin PnP initialization steps */
-	if (isapnp_cfg_begin(acard->wss->dev->csn, acard->wss->number)<0)
-		return -EAGAIN;
 	/* WSS initialization */
-	if (isapnp_config_init(&cfg, acard->wss)<0) {
-		isapnp_cfg_end();
+	pdev = acard->wss;
+	if (pdev->prepare(pdev) < 0)
 		return -EAGAIN;
-	}
 	if (snd_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_port[dev], 4);
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		cfg.port[1] = snd_fm_port[dev];
+		isapnp_resource_change(&pdev->resource[1], snd_fm_port[dev], 4);
 	if (snd_sb_port[dev] != SND_AUTO_PORT)
-		cfg.port[2] = snd_sb_port[dev];
+		isapnp_resource_change(&pdev->resource[2], snd_sb_port[dev], 16);
 	if (snd_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq[dev];
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq[dev], 1);
 	if (snd_dma1[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma1[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma1[dev], 1);
 	if (snd_dma2[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma2[dev] < 0 ? 254 : snd_dma2[dev];
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma2[dev] < 0 ? 4 : snd_dma2[dev], 1);
+	if (pdev->activate(pdev)<0) {
 		snd_printk(IDENT " isapnp configure failed (out of resources?)\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
-	snd_printdd("isapnp WSS: wss port=0x%x, fm port=0x%x, sb port=0x%x\n", cfg.port[0], cfg.port[1], cfg.port[2]);
-	snd_printdd("isapnp WSS: irq=%i, dma1=%i, dma2=%i\n", cfg.irq[0], cfg.dma[0], cfg.dma[1]);
-	snd_port[dev] = cfg.port[0];
-	snd_fm_port[dev] = cfg.port[1];
-	snd_sb_port[dev] = cfg.port[2];
-	snd_irq[dev] = cfg.irq[0];
-	snd_dma1[dev] = cfg.dma[0];
-	snd_dma2[dev] = cfg.dma[1] == 254 ? -1 : cfg.dma[1];
+	snd_port[dev] = pdev->resource[0].start;
+	snd_fm_port[dev] = pdev->resource[1].start;
+	snd_sb_port[dev] = pdev->resource[2].start;
+	snd_irq[dev] = pdev->irq_resource[0].start;
+	snd_dma1[dev] = pdev->dma_resource[0].start;
+	snd_dma2[dev] = pdev->dma_resource[1].start == 4 ? -1 : pdev->dma_resource[1].start;
+	snd_printdd("isapnp WSS: wss port=0x%lx, fm port=0x%lx, sb port=0x%lx\n",
+			snd_port[dev], snd_fm_port[dev], snd_sb_port[dev]);
+	snd_printdd("isapnp WSS: irq=%i, dma1=%i, dma2=%i\n",
+			snd_irq[dev], snd_dma1[dev], snd_dma2[dev]);
 	/* CTRL initialization */
-	if (isapnp_config_init(&cfg, acard->ctrl)<0) {
-		isapnp_cfg_end();
+	pdev = acard->ctrl;
+	if (pdev->prepare(pdev) < 0) {
+		acard->wss->deactivate(acard->wss);
 		return -EAGAIN;
 	}
 	if (snd_cport[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_cport[dev];
-	cfg.port_disable[0] = snd_port[dev];
-	cfg.port_disable_size[0] = 4;
-	cfg.port_disable[1] = snd_fm_port[dev];
-	cfg.port_disable_size[1] = 4;
-	cfg.port_disable[2] = snd_sb_port[dev];
-	cfg.port_disable_size[2] = 16;
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->resource[0], snd_cport[dev], 8);
+	if (pdev->activate(pdev)<0) {
 		snd_printk(IDENT " isapnp configure failed (out of resources?)\n");
-		isapnp_cfg_end();
+		acard->wss->deactivate(acard->wss);
 		return -EBUSY;
 	}
-	snd_cport[dev] = cfg.port[0];
+	snd_cport[dev] = pdev->resource[0].start;
 	snd_printdd("isapnp CTRL: control port=0x%x\n", snd_cport[dev]);
 	/* MPU initialization */
 	if (snd_mpu_port[dev] > 0) {
-		if (isapnp_config_init(&cfg, acard->mpu)<0) {
-			isapnp_cfg_end();
+		pdev = acard->mpu;
+		if (pdev->prepare(pdev) < 0) {
+			acard->wss->deactivate(acard->wss);
+			acard->ctrl->deactivate(acard->ctrl);
 			return -EAGAIN;
 		}
 		if (snd_mpu_port[dev] != SND_AUTO_PORT)
-			cfg.port[0] = snd_mpu_port[dev];
+			isapnp_resource_change(&pdev->resource[0], snd_mpu_port[dev], 2);
 		if (snd_mpu_irq[dev] != SND_AUTO_IRQ)
-			cfg.irq[0] = snd_mpu_irq[dev];
-		cfg.port_disable[0] = snd_port[dev];
-		cfg.port_disable_size[0] = 4;
-		cfg.port_disable[1] = snd_fm_port[dev];
-		cfg.port_disable_size[1] = 4;
-		cfg.port_disable[2] = snd_sb_port[dev];
-		cfg.port_disable_size[2] = 16;
-		cfg.port_disable[3] = snd_cport[dev];
-		cfg.port_disable_size[3] = 8;
-		cfg.irq_disable[0] = snd_irq[dev];
-		if (isapnp_configure(&cfg)<0) {
+			isapnp_resource_change(&pdev->irq_resource[0], snd_mpu_irq[dev], 1);
+		if (pdev->activate(pdev)<0) {
 			snd_mpu_port[dev] = SND_AUTO_PORT;
 		} else {
-			snd_mpu_port[dev] = cfg.port[0];
-			snd_mpu_irq[dev] = cfg.irq[0];
+			snd_mpu_port[dev] = pdev->resource[0].start;
+			snd_mpu_irq[dev] = pdev->irq_resource[0].start;
 		}
 		snd_printdd("isapnp MPU: port=0x%x, irq=%i\n", snd_mpu_port[dev], snd_mpu_irq[dev]);
 	}
-	/* enable all valid devices at one time */
-	isapnp_activate(acard->wss->number);
-	isapnp_activate(acard->ctrl->number);
-	if (snd_mpu_port[dev] != SND_AUTO_PORT)
-		isapnp_activate(acard->mpu->number);
-	isapnp_cfg_end();
 	return 0;
 }
 
 static void snd_card_cs4236_deactivate(struct snd_card_cs4236 *acard)
 {
-	if (!acard->wss)
-		return;
-	if (isapnp_cfg_begin(acard->wss->dev->csn, acard->wss->number)<0)
-		return;
-	isapnp_deactivate(acard->wss->number);
+	if (acard->wss)
+		acard->wss->deactivate(acard->wss);
 	if (acard->ctrl)
-		isapnp_deactivate(acard->ctrl->number);
+		acard->wss->deactivate(acard->ctrl);
 	if (acard->mpu)
-		isapnp_deactivate(acard->mpu->number);
-	isapnp_cfg_end();
+		acard->wss->deactivate(acard->mpu);
 }
 #endif
 
-static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
+static int __init snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 {
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
@@ -460,7 +502,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 	snd_hwdep_t *synth = NULL;
 	cs4231_t *codec;
 
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if (snd_port[dev] == SND_AUTO_PORT ||
 	    snd_cport[dev] == SND_AUTO_PORT) {
 		snd_printk("probing for " IDENT " is not supported\n");
@@ -469,7 +511,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 #endif
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 		       snd_card_cs4236_use_inc, snd_card_cs4236_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 #ifdef CS4236
 	card->type = SND_CARD_TYPE_CS4236;
@@ -477,7 +519,7 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 #ifdef CS4232
 	card->type = SND_CARD_TYPE_CS4232;
 #endif
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	if (snd_isapnp[dev]) {
 		if (snd_card_cs4236_isapnp(dev, acard)<0) {
 			snd_printk("isapnp detection failed and probing for " IDENT " is not supported\n");
@@ -510,61 +552,33 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 				     acard->irqptr, acard->dma1ptr,
 				     acard->dma2ptr == NULL ?
 				     	acard->dma1ptr : acard->dma2ptr);
-	if (!pcm) {
+	if (pcm == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
 	}
-	codec = (cs4231_t *) pcm->private_data;
+	codec = snd_magic_cast(cs4231_t, pcm->private_data, -ENXIO);
 	acard->codec = codec;
 	acard->pcm_status_reg = codec->port + 2;
 
 #ifdef CS4236
-	mixer = snd_cs4236_new_mixer(pcm, 0);
-#else
-	mixer = snd_cs4231_new_mixer(pcm, 0);
-#endif
-	if (!mixer)
+	if (snd_cs4236_new_mixer(pcm, 0, &mixer) < 0)
 		goto __nodev;
+#else
+	if (snd_cs4231_new_mixer(pcm, 0, &mixer) < 0)
+		goto __nodev;
+#endif
 
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		synth = snd_opl3_new_device(card, snd_fm_port[dev],
-					    snd_fm_port[dev] + 2,
-					    OPL3_HW_OPL3_CS, -1);
+		if (snd_opl3_new(card, 0, snd_fm_port[dev],
+				 snd_fm_port[dev] + 2,
+				 OPL3_HW_OPL3_CS, -1, &synth) < 0)
+			snd_printk(IDENT ": OPL3 not detected\n");
 
 	if (acard->mpuirqptr) {
-		rmidi = snd_mpu401_uart_new_device(card, MPU401_HW_CS4232,
-						   snd_mpu_port[dev],
-						   acard->mpuirqptr->irq);
-		if (!rmidi)
-			goto __nodev;
-		if (snd_rawmidi_register(rmidi, 0) < 0)
-			goto __nodev;
-	}
-	if (synth && snd_hwdep_register(synth, 0) < 0) {
-		if (rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		goto __nodev;
-	}
-	if (snd_mixer_register(mixer, 0) < 0) {
-		if (synth)
-			snd_hwdep_unregister(synth);
-		synth = NULL;
-		if (rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		goto __nodev;
-	}
-	if (snd_pcm_register(pcm, 0)) {
-		if (synth)
-			snd_hwdep_unregister(synth);
-		synth = NULL;
-		if (rmidi)
-			snd_rawmidi_unregister(rmidi);
-		rmidi = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
+		if (snd_mpu401_uart_new(card, 0, MPU401_HW_CS4232,
+					snd_mpu_port[dev],
+					acard->mpuirqptr->irq, &rmidi) < 0)
+			snd_printk(IDENT ": MPU401 not detected\n");
 	}
 #ifdef CS4236
 	strcpy(card->abbreviation, "CS4236");
@@ -589,26 +603,22 @@ static int snd_card_cs4236_probe(int dev, struct snd_card_cs4236 *acard)
 		acard->synth = synth;
 		return 0;
 	}
-	snd_hwdep_unregister(synth);
-	snd_mixer_unregister(mixer);
-	snd_pcm_unregister(pcm);
-	snd_card_free(card);
-	return -ENOMEM;
 
       __nodev:
-	if (synth)
-		snd_hwdep_free(synth);
-	if (rmidi)
-		snd_rawmidi_free(rmidi);
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm)
-		snd_pcm_free(pcm);
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+#ifdef CS4232
+int __init alsa_card_cs4232_init(void)
+#endif
+#ifdef CS4236
+int __init alsa_card_cs4236_init(void)
+#endif
+#endif
 {
 	int dev, cards;
 	struct snd_card_cs4236 *acard;
@@ -616,30 +626,36 @@ int init_module(void)
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_card_cs4236 *)
 				snd_kcalloc(sizeof(struct snd_card_cs4236), GFP_KERNEL);
-		if (!acard)
+		if (acard == NULL)
 			continue;
 		memset(acard, 0, sizeof(struct snd_card_cs4236));
 		if (snd_card_cs4236_probe(dev, acard) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_card_cs4236_deactivate(acard);
 #endif
 			snd_kfree(acard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk(IDENT " soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_card_cs4236_cards[dev] = acard;
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk(IDENT " soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int idx;
 	struct snd_card_cs4236 *acard;
@@ -648,20 +664,12 @@ void cleanup_module(void)
 		acard = snd_card_cs4236_cards[idx];
 		if (acard) {
 			snd_card_unregister(acard->card);
-			if (acard->synth)
-				snd_hwdep_unregister(acard->synth);
-			if (acard->rmidi)
-				snd_rawmidi_unregister(acard->rmidi);
-			if (acard->mixer)
-				snd_mixer_unregister(acard->mixer);
-			if (acard->pcm) {
-				snd_pcm_unregister(acard->pcm);
-			}
-			snd_card_free(acard->card);
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_card_cs4236_deactivate(acard);
 #endif
 			snd_kfree(acard);
 		}
 	}
 }
+
+#endif
