@@ -32,6 +32,28 @@
 #endif
 #include "../include/initval.h"
 
+#ifndef SND_STB
+MODULE_DESCRIPTION("\
+Driver: AMD InterWave\n\
+Card: AMD InterWave generic\n\
+Card: Gravis UltraSound Plug & Play\n\
+Card: STB SoundRage32\n\
+Card: MED3210\n\
+Card: Dynasonix Pro\n\
+Card: Panasonic PCA761AW\n\
+ISAPNP: GRV0001=GRV0000\n\
+ISAPNP: STB011a=STB0010\n\
+ISAPNP: DXP3201=DXP0010\n\
+ISAPNP: CDC1111=CDC1112\n\
+ISAPNP: ADV55ff=ADV0010\n\
+");
+#else
+MODULE_DESCRIPTION("\
+Driver: AMD InterWave STB with TEA6330T\n\
+ISAPNP: ADV550a=ADV0010,ADV0015\n\
+");
+#endif
+
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	/* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	/* ID for this card */
 int snd_port[SND_CARDS] = SND_DEFAULT_PORT;	/* 0x210,0x220,0x230,0x240,0x250,0x260 */
@@ -53,27 +75,29 @@ MODULE_PARM_DESC(snd_index, "Index value for InterWave soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for InterWave soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for InterWave driver.");
+MODULE_PARM_DESC(snd_port, "Port # for InterWave driver. [list=0x210,0x220,0x230,0x240,0x250,0x260]");
+#ifdef SND_STB
 MODULE_PARM(snd_port_tc, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port_tc, "Tone control (TEA6330T - i2c bus) port # for InterWave driver.");
+MODULE_PARM_DESC(snd_port_tc, "Tone control (TEA6330T - i2c bus) port # for InterWave driver. [list=0x350,0x360,0x370,0x380]");
+#endif
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for InterWave driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for InterWave driver. [list=3,5,9,11,12,15]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for InterWave driver.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for InterWave driver. [list=0,1,3,5,6,7]");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA2 # for InterWave driver.");
+MODULE_PARM_DESC(snd_dma2, "DMA2 # for InterWave driver. [list=0,1,3,5,6,7]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "DMA1 size in kB for InterWave driver.");
+MODULE_PARM_DESC(snd_dma1_size, "DMA1 size in kB for InterWave driver. [DMA_SIZE]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "DMA2 size in kB for InterWave driver.");
+MODULE_PARM_DESC(snd_dma2_size, "DMA2 size in kB for InterWave driver. [DMA_SIZE]");
 MODULE_PARM(snd_joystick_dac, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_joystick_dac, "Joystick DAC level 0.59V-4.52V or 0.389V-2.98V for InterWave driver.");
+MODULE_PARM_DESC(snd_joystick_dac, "Joystick DAC level 0.59V-4.52V or 0.389V-2.98V for InterWave driver. [range=0-31]");
 MODULE_PARM(snd_midi, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_midi, "MIDI UART enable for InterWave driver.");
+MODULE_PARM_DESC(snd_midi, "MIDI UART enable for InterWave driver. [BOOL]");
 MODULE_PARM(snd_pcm_voices, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_pcm_voices, "Reserved PCM voices for InterWave driver.");
+MODULE_PARM_DESC(snd_pcm_voices, "Reserved PCM voices for InterWave driver. [range=2-16]");
 MODULE_PARM(snd_effect, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_effect, "Effects enable for InterWave driver.");
+MODULE_PARM_DESC(snd_effect, "Effects enable for InterWave driver. [BOOL]");
 
 struct snd_interwave {
 	snd_irq_t *irqptr;
@@ -90,17 +114,17 @@ struct snd_interwave {
 #endif
 	unsigned short gus_status_reg;
 	unsigned short pcm_status_reg;
-#ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *logdev;
+#ifdef __ISAPNP__
+	struct isapnp_dev *dev;
 #ifdef SND_STB
-	struct isapnp_logdev *logdevtc;
+	struct isapnp_dev *devtc;
 #endif
 #endif
 };
 
 static struct snd_interwave *snd_interwave_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 static unsigned int snd_interwave_pnpids[] = {
 #ifndef SND_STB
 	/* Gravis UltraSound Plug & Play */
@@ -154,7 +178,7 @@ static void snd_interwave_i2c_setlines(struct snd_i2c_bus *bus,
 	printk("i2c_setlines - 0x%x <- %i,%i\n", port, ctrl, data);
 #endif
 	outb((data << 1) | ctrl, port);
-	snd_delay(1);
+	udelay(10);
 }
 
 static int snd_interwave_i2c_getdataline(struct snd_i2c_bus *bus)
@@ -170,8 +194,8 @@ static int snd_interwave_i2c_getdataline(struct snd_i2c_bus *bus)
 	return res;
 }
 
-static int snd_interwave_detect_stb(struct snd_interwave *iwcard,
-				    snd_gus_card_t * gus, int dev)
+static int __init snd_interwave_detect_stb(struct snd_interwave *iwcard,
+					   snd_gus_card_t * gus, int dev)
 {
 	unsigned short port;
 	struct snd_i2c_bus *bus;
@@ -199,7 +223,7 @@ static int snd_interwave_detect_stb(struct snd_interwave *iwcard,
 	}
 	sprintf(name, "InterWave-%i", dev);
 	bus = snd_i2c_bus_new(SND_I2C_BUSID_INTERWAVE, name);
-	if (!bus)
+	if (bus == NULL)
 		goto __nodev;
 	bus->data = (void *) ((unsigned long) port);
 	bus->lock = SPIN_LOCK_UNLOCKED;
@@ -235,9 +259,9 @@ static int snd_interwave_detect_stb(struct snd_interwave *iwcard,
 }
 #endif
 
-static int snd_interwave_detect(struct snd_interwave *iwcard,
-				snd_gus_card_t * gus,
-				unsigned short port, int dev)
+static int __init snd_interwave_detect(struct snd_interwave *iwcard,
+				       snd_gus_card_t * gus,
+				       unsigned short port, int dev)
 {
 	unsigned long flags;
 	unsigned char rev1, rev2;
@@ -270,9 +294,9 @@ static int snd_interwave_detect(struct snd_interwave *iwcard,
 	if ((snd_gf1_i_look8(gus, SND_GF1_GB_RESET) & 0x07) != 0)
 		goto __nodev;
 #endif
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_i_write8(gus, SND_GF1_GB_RESET, 1);	/* release reset */
-	snd_delay(16);
+	udelay(160);
 #ifdef CONFIG_SND_DEBUG_DETECT
 	{
 		int d;
@@ -319,7 +343,7 @@ static void snd_interwave_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	struct snd_interwave *iwcard = (struct snd_interwave *) dev_id;
 	int loop, max = 100;
 
-	if (!iwcard || !iwcard->gus || !iwcard->pcm)
+	if (iwcard == NULL || iwcard->gus == NULL || iwcard->pcm == NULL)
 		return;
 
 	do {
@@ -335,8 +359,8 @@ static void snd_interwave_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	} while (loop && --max > 0);
 }
 
-static int snd_interwave_resources(int dev, struct snd_interwave *iwcard,
-				   snd_card_t * card)
+static int __init snd_interwave_resources(int dev, struct snd_interwave *iwcard,
+					  snd_card_t * card)
 {
 	static int possible_irqs[] = {5, 11, 12, 9, 7, 15, 3, -1};
 	static int possible_dmas[] = {0, 1, 3, 5, 6, 7, -1};
@@ -367,15 +391,15 @@ static int snd_interwave_resources(int dev, struct snd_interwave *iwcard,
 	return 0;
 }
 
-static void snd_interwave_reset(snd_gus_card_t * gus)
+static void __init snd_interwave_reset(snd_gus_card_t * gus)
 {
 	snd_gf1_write8(gus, SND_GF1_GB_RESET, 0x00);
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_write8(gus, SND_GF1_GB_RESET, 0x01);
-	snd_delay(16);
+	udelay(160);
 }
 
-static void snd_interwave_bank_sizes(snd_gus_card_t * gus, int *sizes)
+static void __init snd_interwave_bank_sizes(snd_gus_card_t * gus, int *sizes)
 {
 	int idx;
 	unsigned int local;
@@ -424,7 +448,7 @@ struct rom_hdr {
 	/* 511 */ unsigned char csum;
 };
 
-static void snd_interwave_detect_memory(snd_gus_card_t * gus)
+static void __init snd_interwave_detect_memory(snd_gus_card_t * gus)
 {
 	static unsigned int lmc[13] =
 	{
@@ -532,7 +556,7 @@ static void snd_interwave_detect_memory(snd_gus_card_t * gus)
 		snd_interwave_reset(gus);
 }
 
-static void snd_interwave_init(int dev, snd_gus_card_t * gus)
+static void __init snd_interwave_init(int dev, snd_gus_card_t * gus)
 {
 	unsigned long flags;
 
@@ -554,50 +578,54 @@ static void snd_interwave_init(int dev, snd_gus_card_t * gus)
 
 }
 
-static int snd_interwave_master_volume(void *private_data, int w_flag, int *voices)
+static int snd_interwave_master_volume(snd_kmixer_element_t *element,
+				       int w_flag, int *voices)
 {
-	return snd_cs4231_mixer_stereo_volume(private_data,
+	return snd_cs4231_mixer_stereo_volume(element,
 					      w_flag, voices,
 					      31, 1, 0,
 					      CS4231_LINE_LEFT_OUTPUT,
 					      CS4231_LINE_RIGHT_OUTPUT);
 }
 
-static int snd_interwave_master_switch(void *private_data, int w_flag, unsigned int *bitmap)
+static int snd_interwave_master_switch(snd_kmixer_element_t *element,
+				       int w_flag, unsigned int *bitmap)
 {
-	return snd_cs4231_mixer_stereo_switch(private_data,
+	return snd_cs4231_mixer_stereo_switch(element,
 					      w_flag, bitmap,
 					      7, 1,
 					      CS4231_LINE_LEFT_OUTPUT,
 					      CS4231_LINE_RIGHT_OUTPUT);
 }
 
-static int snd_interwave_mic_volume(void *private_data, int w_flag, int *voices)
+static int snd_interwave_mic_volume(snd_kmixer_element_t *element,
+				    int w_flag, int *voices)
 {
-	return snd_cs4231_mixer_stereo_volume(private_data,
+	return snd_cs4231_mixer_stereo_volume(element,
 					      w_flag, voices,
 					      31, 1, 0,
 					      CS4231_LEFT_MIC_INPUT,
 					      CS4231_RIGHT_MIC_INPUT);
 }
 
-static int snd_interwave_mic_switch(void *private_data, int w_flag, unsigned int *bitmap)
+static int snd_interwave_mic_switch(snd_kmixer_element_t *element,
+				    int w_flag, unsigned int *bitmap)
 {
-	return snd_cs4231_mixer_stereo_switch(private_data,
+	return snd_cs4231_mixer_stereo_switch(element,
 					      w_flag, bitmap,
 					      7, 1,
 					      CS4231_LEFT_MIC_INPUT,
 					      CS4231_RIGHT_MIC_INPUT);
 }
 
-int snd_interwave_mixer_group_master(void *private_data,
-				     snd_kmixer_file_t * file,
-				     int w_flag,
-				     snd_mixer_group_t * ugroup)
+static int snd_interwave_mixer_group_master(snd_kmixer_group_t * group,
+					    snd_kmixer_file_t * file,
+					    int w_flag,
+					    snd_mixer_group_t * ugroup)
 {
-	cs4231_t *codec = (cs4231_t *)private_data;
+	cs4231_t *codec = snd_magic_cast(cs4231_t, group->private_data, -ENXIO);
 
-	return snd_cs4231_mixer_group_ctrl1(codec, file, w_flag, ugroup,
+	return snd_cs4231_mixer_group_ctrl1(group, file, w_flag, ugroup,
 					    snd_interwave_master_volume,
 					    codec->me_vol_master,
 					    31,
@@ -606,14 +634,14 @@ int snd_interwave_mixer_group_master(void *private_data,
 					    NULL);
 }
 
-int snd_interwave_mixer_group_mic(void *private_data,
+int snd_interwave_mixer_group_mic(snd_kmixer_group_t * group,
 				  snd_kmixer_file_t * file,
 				  int w_flag,
 				  snd_mixer_group_t * ugroup)
 {
-	cs4231_t *codec = (cs4231_t *)private_data;
+	cs4231_t *codec = snd_magic_cast(cs4231_t, group->private_data, -ENXIO);
 
-	return snd_cs4231_mixer_group_ctrl1(codec, file, w_flag, ugroup,
+	return snd_cs4231_mixer_group_ctrl1(group, file, w_flag, ugroup,
 					    snd_interwave_mic_volume,
 					    codec->me_vol_mic,
 					    31,
@@ -622,7 +650,7 @@ int snd_interwave_mixer_group_mic(void *private_data,
 					    codec->me_mux_mic);
 }
 
-static int snd_interwave_mixer(snd_kmixer_t * mixer, cs4231_t * codec)
+static int __init snd_interwave_mixer(snd_kmixer_t * mixer, cs4231_t * codec)
 {
 	snd_kmixer_group_t *group;
 	struct snd_stru_mixer_lib_volume1 *volume1;
@@ -659,7 +687,7 @@ static int snd_interwave_mixer(snd_kmixer_t * mixer, cs4231_t * codec)
 	if ((oelement2 = snd_mixer_element_find(mixer, "MIC Volume", 0, SND_MIXER_ETYPE_VOLUME1)) == NULL)
 		goto __error;
 #endif
-	volume1 = (struct snd_stru_mixer_lib_volume1 *)codec->me_vol_mic->private_data;
+	volume1 = (struct snd_stru_mixer_lib_volume1 *)snd_mixer_ext_element_private_data(codec->me_vol_mic);
 	volume1->ranges[0].max = volume1->ranges[1].max = 31;
 	volume1->ranges[0].min_dB = volume1->ranges[1].min_dB = -3450;
 	volume1->ranges[0].max_dB = volume1->ranges[1].max_dB = 1200;
@@ -701,131 +729,122 @@ static int snd_interwave_mixer(snd_kmixer_t * mixer, cs4231_t * codec)
       	return 1;
 }
 
-#ifdef CONFIG_ISAPNP
-static int snd_interwave_isapnp(int dev, struct snd_interwave *iwcard)
+#ifdef __ISAPNP__
+static int __init snd_interwave_isapnp(int dev, struct snd_interwave *iwcard)
 {
-        int idx, step;
-        unsigned int tmp;
+        static int idx = 0;
+	static struct isapnp_card *card = NULL;
 	struct isapnp_dev *pdev;
-	struct isapnp_logdev *logdev;
-	struct isapnp_config cfg;
-
 #ifndef SND_STB
-	step = 2;
+	int step = 2;
 #else
-	step = 3;
+	int step = 3;
 #endif	
-	idx = 0;
+        unsigned int tmp;
+
       __again:
-      	pdev = NULL;
+	iwcard->dev = NULL;
+#ifdef SND_STB
+	iwcard->devtc = NULL;
+#endif
 	while ((tmp = snd_interwave_pnpids[idx]) != 0) {
-		pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev);
-		if (pdev)
+		card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card);
+		if (card)
 			break;
 		idx += step;
 	}
-	if (!pdev) {
+	if (card == NULL) {
 		snd_printdd("isapnp failed for InterWave\n");
 		return -ENODEV;
 	}
 	tmp = snd_interwave_pnpids[idx+1];
-	iwcard->logdev = logdev = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!iwcard->logdev) {
+	iwcard->dev = pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (pdev == NULL) {
                 snd_printdd("isapnp failed for InterWave (Codec & Synth) - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
+        } else if (pdev->active) {
+        	goto __again;
         }
 #ifdef SND_STB
 	tmp = snd_interwave_pnpids[idx+2];
-	iwcard->logdevtc = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!iwcard->logdevtc) {
+	iwcard->devtc = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (iwcard->devtc == NULL) {
                 snd_printdd("isapnp failed for InterWave (Tone Control) - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
+        } else if (pdev->active) {
+        	goto __again;
         }
 #endif
-	if (isapnp_cfg_begin(logdev->dev->csn, logdev->number))
-		return -EAGAIN;
 	/* Synth & Codec initialization */
-	if (isapnp_config_init(&cfg, logdev)) {
-		isapnp_cfg_end();
+	if (pdev->prepare(pdev)<0)
 		return -EAGAIN;
-	}
 	if (snd_port[dev] != SND_AUTO_PORT) {
-		cfg.port[0] = snd_port[dev];
-		cfg.port[1] = snd_port[dev] + 0x100;
-		cfg.port[2] = snd_port[dev] + 0x10c;
+		isapnp_resource_change(&pdev->resource[0], snd_port[dev], 16);
+		isapnp_resource_change(&pdev->resource[1], snd_port[dev] + 0x100, 12);
+		isapnp_resource_change(&pdev->resource[2], snd_port[dev] + 0x10c, 4);
 	}
 	if (snd_dma1[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma1[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma1[dev], 1);
 	if (snd_dma2[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma2[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma2[dev], 1);
+	if (snd_dma2[dev] < 0)
+		isapnp_resource_change(&pdev->dma_resource[1], 4, 1);
 	if (snd_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq[dev];
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq[dev], 1);
+	if (pdev->activate(pdev)<0) {
 		snd_printk("InterWave isapnp configure failure (out of resources?)\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
-	snd_printdd("isapnp IW: sb port=0x%x, gf1 port=0x%x, codec port=0x%x\n", cfg.port[0], cfg.port[1], cfg.port[2]);
-	snd_printdd("isapnp IW: dma1=%i, dma2=%i, irq=%i\n", cfg.dma[0], cfg.dma[1], cfg.irq[0]);
-	if (cfg.port[0] + 0x100 != cfg.port[1] ||
-	    cfg.port[0] + 0x10c != cfg.port[2]) {
+	if (pdev->resource[0].start + 0x100 != pdev->resource[1].start ||
+	    pdev->resource[0].start + 0x10c != pdev->resource[2].start) {
 		snd_printk("InterWave isapnp configure failure (wrong ports)\n");
-		isapnp_cfg_end();
+		pdev->deactivate(pdev);
 		return -ENOENT;
 	}
-	snd_port[dev] = cfg.port[0];
-	snd_dma1[dev] = cfg.dma[0];
+	snd_port[dev] = pdev->resource[0].start;
+	snd_dma1[dev] = pdev->dma_resource[0].start;
 	if (snd_dma2[dev] >= 0)
-		snd_dma2[dev] = cfg.dma[1];
-	snd_irq[dev] = cfg.irq[0];
+		snd_dma2[dev] = pdev->dma_resource[1].start;
+	snd_irq[dev] = pdev->irq_resource[0].start;
+	snd_printdd("isapnp IW: sb port=0x%x, gf1 port=0x%x, codec port=0x%x\n",
+				pdev->resource[0].start,
+				pdev->resource[1].start,
+				pdev->resource[2].start);
+	snd_printdd("isapnp IW: dma1=%i, dma2=%i, irq=%i\n", snd_dma1[dev], snd_dma2[dev], snd_irq[dev]);
 #ifdef SND_STB
 	/* Tone Control initialization */
-	if (isapnp_config_init(&cfg, iwcard->logdevtc)) {
-		isapnp_cfg_end();
+	pdev = iwcard->devtc;
+	if (pdev->prepare(pdev)<0) {
+		iwcard->dev->deactivate(iwcard->dev);
 		return -EAGAIN;
 	}
 	if (snd_port_tc[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_port_tc[dev];
-	cfg.port_disable[0] = snd_port[dev];
-	cfg.port_disable_size[0] = 16;
-	cfg.port_disable[1] = snd_port[dev] + 0x100;
-	cfg.port_disable_size[1] = 16;
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->resource[0], snd_port_tc[dev], 1);
+	if (pdev->activate(pdev)<0) {
 		snd_printk("InterWave Tone Control isapnp configure failure (out of resources?)\n");
-		isapnp_cfg_end();
+		iwcard->dev->deactivate(iwcard->dev);
 		return -EBUSY;
 	}
-	snd_port_tc[dev] = cfg.port[0];
-	snd_printdd("isapnp IW: tone control port=0x%x\n", cfg.port[0]);
+	snd_port_tc[dev] = pdev->resource[0].start;
+	snd_printdd("isapnp IW: tone control port=0x%x\n", snd_port_tc[dev]);
 #endif
-        isapnp_activate(iwcard->logdev->number);
-#ifdef SND_STB
-	isapnp_activate(iwcard->logdevtc->number);
-#endif
-	isapnp_cfg_end();
 	return 0;
 }
 
 static void snd_interwave_deactivate(struct snd_interwave *iwcard)
 {
-	if (!iwcard->logdev)
-		return;
-	if (isapnp_cfg_begin(iwcard->logdev->dev->csn, iwcard->logdev->number)<0)
-		return;
-	isapnp_deactivate(iwcard->logdev->number);
+	if (iwcard->dev)
+		iwcard->dev->deactivate(iwcard->dev);
 #ifdef SND_STB
-	if (iwcard->logdevtc)
-		isapnp_deactivate(iwcard->logdevtc->number);
+	if (iwcard->devtc)
+		iwcard->devtc->deactivate(iwcard->devtc);
 #endif
-	isapnp_cfg_end();
 }
-#endif /* CONFIG_ISAPNP */
+#endif /* __ISAPNP__ */
 
-static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
+static int __init snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 {
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	static int possible_ports[] =
 			{0x210, 0x220, 0x230, 0x240, 0x250, 0x260, -1};
 	int *ports = possible_ports;
@@ -837,13 +856,14 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 	snd_kmixer_t *mixer = NULL;
 	snd_rawmidi_t *midi_uart = NULL;
 	char *str;
+	int err;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			    snd_interwave_use_inc, snd_interwave_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 	card->static_data = iwcard;
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	if (snd_interwave_isapnp(dev, iwcard)) {
 		snd_card_free(card);
 		return -ENODEV;
@@ -859,11 +879,11 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 			       iwcard->dma1ptr,
 			       iwcard->dma2ptr,
 			       0, 32, snd_pcm_voices[dev], snd_effect[dev]);
-	if (!gus) {
+	if (gus == NULL) {
 		snd_card_free(card);
 		return -ENOMEM;
 	}
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if (snd_port[dev] == SND_AUTO_PORT) {
 		for (ports = possible_ports; *ports >= 0; ports++) {
 			if (!snd_interwave_detect(iwcard, gus, *ports, dev))
@@ -879,7 +899,7 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 			snd_card_free(card);
 			return -ENODEV;
 		}
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	}
 #endif
 	iwcard->gus_status_reg = gus->gf1.reg_irqstat;
@@ -890,29 +910,29 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 		snd_card_free(card);
 		return -EINVAL;
 	}
-	pcm = snd_cs4231_new_device(card,
-				    gus->gf1.port + 0x10c,
-				    iwcard->irqptr,
-				    iwcard->dma2ptr == NULL ?
+	err = snd_cs4231_new_pcm(card, 0,
+				 gus->gf1.port + 0x10c,
+				 iwcard->irqptr,
+				 iwcard->dma2ptr == NULL ?
 				        iwcard->dma1ptr : iwcard->dma2ptr,
-				    iwcard->dma1ptr,
-				    CS4231_HW_INTERWAVE,
-				    2);
-	if (!pcm)
+				 iwcard->dma1ptr,
+				 CS4231_HW_INTERWAVE,
+				 2, &pcm);
+	if (err < 0)
 		goto __nodev;
 	sprintf(pcm->name + strlen(pcm->name), " rev %c", gus->revision + 'A');
 	strcat(pcm->name, " (codec)");
-	mixer = snd_cs4231_new_mixer(pcm, 0);
-	if (!mixer)
+	err = snd_cs4231_new_mixer(pcm, 0, &mixer);
+	if (err < 0)
 		goto __nodev;
 	if (snd_pcm_voices[dev] > 0) {
 		snd_kmixer_element_t *element;
 		
 		element = snd_mixer_element_find(mixer, SND_MIXER_IN_AUX, 0, SND_MIXER_ETYPE_INPUT);
-		if (!element)
+		if (element == NULL)
 			goto __nodev;
-		pcm1 = snd_gf1_pcm_new_device(gus, mixer, element, 1);
-		if (!pcm1)
+		err = snd_gf1_pcm_new(gus, mixer, element, 1, &pcm1);
+		if (err < 0)
 			goto __nodev;
 	}
 	if (snd_interwave_mixer(mixer, (cs4231_t *)(pcm->private_data)) < 0)
@@ -941,50 +961,14 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 #endif
 
 	gus->uart_enable = snd_midi[dev];
-	midi_uart = snd_gf1_rawmidi_new_device(gus);
-	if (!midi_uart)
+	err = snd_gf1_rawmidi_new(gus, 0, &midi_uart);
+	if (err < 0)
 		goto __nodev;
 
 #ifdef CONFIG_SND_SEQUENCER
 	if (snd_gus_attach_synthesizer(gus) < 0)
 		goto __nodev;
 #endif
-	if (snd_rawmidi_register(midi_uart, 0) < 0) {
-#ifdef CONFIG_SND_SEQUENCER
-		snd_gus_detach_synthesizer(gus);
-#endif
-		goto __nodev;
-	}
-	if (snd_mixer_register(mixer, 0) < 0) {
-#ifdef CONFIG_SND_SEQUENCER
-		snd_gus_detach_synthesizer(gus);
-#endif
-		snd_rawmidi_unregister(midi_uart);
-		midi_uart = NULL;
-		goto __nodev;
-	}
-	if (snd_pcm_register(pcm, 0) < 0) {
-#ifdef CONFIG_SND_SEQUENCER
-		snd_gus_detach_synthesizer(gus);
-#endif
-		snd_rawmidi_unregister(midi_uart);
-		midi_uart = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		goto __nodev;
-	}
-	if (pcm1 && snd_pcm_register(pcm1, 1) < 0) {
-#ifdef CONFIG_SND_SEQUENCER
-		snd_gus_detach_synthesizer(gus);
-#endif
-		snd_rawmidi_unregister(midi_uart);
-		midi_uart = NULL;
-		snd_mixer_unregister(mixer);
-		mixer = NULL;
-		snd_pcm_unregister(pcm);
-		pcm = NULL;
-		goto __nodev;
-	}
 #if 0
 	snd_cs4231_debug((cs4231_t *) pcm->private_data);
 #endif
@@ -1016,27 +1000,8 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 		iwcard->midi_uart = midi_uart;
 		return 0;
 	}
-	snd_gf1_stop(gus);
-#ifdef CONFIG_SND_SEQUENCER
-	snd_gus_detach_synthesizer(gus);
-#endif
-	snd_pcm_unregister(pcm1);
-	pcm1 = NULL;
-	snd_pcm_unregister(pcm);
-	pcm = NULL;
-	snd_mixer_unregister(mixer);
-	mixer = NULL;
 
       __nodev:
-	snd_gus_init_dma_irq(gus, 0);
-	if (midi_uart)
-		snd_rawmidi_free(midi_uart);
-	if (mixer)
-		snd_mixer_free(mixer);
-	if (pcm1)
-		snd_pcm_free(pcm1);
-	if (pcm)
-		snd_pcm_free(pcm);
 	snd_card_free(card);
 #ifdef SND_STB
 	if (iwcard->i2c) {
@@ -1047,80 +1012,79 @@ static int snd_interwave_probe(int dev, struct snd_interwave *iwcard)
 	return -ENXIO;
 }
 
-static int snd_interwave_free(int dev)
+#ifdef MODULE
+
+static int __exit snd_interwave_free(int dev)
 {
 	struct snd_interwave *iwcard;
-	snd_pcm_t *pcm;
 
 	iwcard = snd_interwave_cards[dev];
 	snd_interwave_cards[dev] = NULL;
 	if (iwcard) {
 		snd_card_unregister(iwcard->card);
-		if (iwcard->gus) {
-			snd_gf1_stop(iwcard->gus);
-			snd_gus_init_dma_irq(iwcard->gus, 0);
-		}
-#ifdef CONFIG_SND_SEQUENCER
-		snd_gus_detach_synthesizer(iwcard->gus);
-#endif
-		if (iwcard->midi_uart)
-			snd_rawmidi_unregister(iwcard->midi_uart);
-		if (iwcard->mixer)
-			snd_mixer_unregister(iwcard->mixer);
-		if (iwcard->pcm1) {
-			snd_pcm_unregister(iwcard->pcm1);
-		}
-		if (iwcard->pcm) {
-			pcm = iwcard->pcm;
-			iwcard->pcm = NULL;	/* turn off interrupts */
-			snd_pcm_unregister(pcm);
-		}
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 		snd_interwave_deactivate(iwcard);
 #endif
 #ifdef SND_STB
 		if (iwcard->i2c)
 			snd_i2c_bus_free(iwcard->i2c);
 #endif
-		snd_card_free(iwcard->card);
 		snd_kfree(iwcard);
 	}
 	return 0;
 }
 
-int init_module(void)
+#endif
+
+#ifdef MODULE
+int __init init_module(void)
+#else
+#ifndef SND_STB
+int __init alsa_card_interwave_init(void)
+#else
+int __init alsa_card_interwave_stb_init(void)
+#endif
+#endif
 {
 	int dev, cards;
 	struct snd_interwave *iwcard;
 
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		iwcard = (struct snd_interwave *) snd_kcalloc(sizeof(struct snd_interwave), GFP_KERNEL);
-		if (!iwcard)
+		if (iwcard == NULL)
 			continue;
 		if (snd_interwave_probe(dev, iwcard) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_interwave_deactivate(iwcard);
 #endif
 			snd_kfree(iwcard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk("InterWave soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_interwave_cards[dev] = iwcard;
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk("InterWave soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int dev;
 
 	for (dev = 0; dev < SND_CARDS; dev++)
 		snd_interwave_free(dev);
 }
+
+#endif
