@@ -22,16 +22,48 @@
 #define SND_MAIN_OBJECT_FILE
 #include "../include/driver.h"
 #include "../include/sb.h"
+#include "../include/sb16_csp.h"
 #include "../include/mpu401.h"
 #include "../include/initval.h"
-#include "../include/sb_csp.h"
 #include "../include/opl3.h"
 #include "../include/emu8000.h"
 #include "../include/seq_device.h"
 
-#if 0
-#define SND_TEST_SB_OLD_MIDI	/* doesn't work for me (SB AWE 64) - Perex */
+#ifndef SND_SBAWE
+MODULE_DESCRIPTION("\
+Driver: Sound Blaster 16\n\
+ISAPNP: CTL0024=CTL0041\n\
+ISAPNP: CTL0026=CTL0031\n\
+ISAPNP: CTL0027=CTL0031\n\
+ISAPNP: CTL0029=CTL0031\n\
+ISAPNP: CTL002b=CTL0031\n\
+ISAPNP: CTL0051=CTL0001\n\
+ISAPNP: CTL0070=CTL0001\n\
+ISAPNP: CTL0080=CTL0041\n\
+ISAPNP: CTL00f0=CTL0043\n\
+");
+#else
+MODULE_DESCRIPTION("\
+Driver: Sound Blaster AWE\n\
+Card: Sound Blaster AWE 32\n\
+Card: Sound Blaster AWE 64\n\
+ISAPNP: CTL0039=CTL0031,CTL0021\n\
+ISAPNP: CTL0042=CTL0031,CTL0021\n\
+ISAPNP: CTL0043=CTL0031,CTL0021\n\
+ISAPNP: CTL0044=CTL0031,CTL0021\n\
+ISAPNP: CTL0048=CTL0031,CTL0021\n\
+ISAPNP: CTL0054=CTL0031,CTL0021\n\
+ISAPNP: CTL009c=CTL0041,CTL0021\n\
+ISAPNP: CTL009d=CTL0042,CTL0022\n\
+ISAPNP: CTL009e=CTL0044,CTL0023\n\
+ISAPNP: CTL00c1=CTL0042,CTL0022\n\
+ISAPNP: CTL00c3=CTL0045,CTL0022\n\
+ISAPNP: CTL00c5=CTL0045,CTL0022\n\
+ISAPNP: CTL00c7=CTL0045,CTL0022\n\
+ISAPNP: CTL00e4=CTL0045,CTL0022\n\
+");
 #endif
+
 #if 0
 #define SND_DEBUG_IRQ
 #endif
@@ -54,7 +86,10 @@ int snd_dma8_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_dma16[SND_CARDS] = SND_DEFAULT_DMA;	/* 5,6,7 */
 int snd_dma16_size[SND_CARDS] = SND_DEFAULT_DMA_SIZE;	/* 8,16,32,64,128 */
 int snd_mic_agc[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
-#ifdef CONFIG_ISAPNP
+#ifdef CONFIG_SND_SB16_CSP
+int snd_csp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 0};
+#endif
+#ifdef __ISAPNP__
 int snd_isapnp[SND_CARDS] = {[0 ... (SND_CARDS - 1)] = 1};
 #endif
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
@@ -62,30 +97,34 @@ MODULE_PARM_DESC(snd_index, "Index value for SoundBlaster 16 soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for SoundBlaster 16 soundcard.");
 MODULE_PARM(snd_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_port, "Port # for SB16 driver.");
+MODULE_PARM_DESC(snd_port, "Port # for SB16 driver. [list=0x220,0x240,0x260]");
 MODULE_PARM(snd_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for SB16 driver.");
+MODULE_PARM_DESC(snd_mpu_port, "MPU-401 port # for SB16 driver. [list=0x330,0x300]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port # for SB16 driver.");
+MODULE_PARM_DESC(snd_fm_port, "FM port # for SB16 driver. [list=0x388]");
 #ifdef SND_SBAWE_EMU8000
 MODULE_PARM(snd_awe_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_awe_port, "AWE port # for SB16 driver.");
+MODULE_PARM_DESC(snd_awe_port, "AWE port # for SB16 driver. [list=0x620]");
 #endif
 MODULE_PARM(snd_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_irq, "IRQ # for SB16 driver.");
+MODULE_PARM_DESC(snd_irq, "IRQ # for SB16 driver. [IRQ]");
 MODULE_PARM(snd_dma8, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for SB16 driver.");
+MODULE_PARM_DESC(snd_dma8, "8-bit DMA # for SB16 driver. [DMA8]");
 MODULE_PARM(snd_dma16, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for SB16 driver.");
+MODULE_PARM_DESC(snd_dma16, "16-bit DMA # for SB16 driver. [DMA16]");
 MODULE_PARM(snd_dma8_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for SB16 driver.");
+MODULE_PARM_DESC(snd_dma8_size, "Size of 8-bit DMA # for SB16 driver. [DMA8_SIZE]");
 MODULE_PARM(snd_dma16_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for SB16 driver.");
+MODULE_PARM_DESC(snd_dma16_size, "Size of 16-bit DMA # for SB16 driver. [DMA16_SIZE]");
 MODULE_PARM(snd_mic_agc, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_mic_agc, "Mic Auto-Gain-Control switch.");
-#ifdef CONFIG_ISAPNP
+MODULE_PARM_DESC(snd_mic_agc, "Mic Auto-Gain-Control switch. [BOOL]");
+#ifdef CONFIG_SND_SB16_CSP
+MODULE_PARM(snd_csp, "1-" __MODULE_STRING(SND_CARDS) "i");
+MODULE_PARM_DESC(snd_csp, "CSP chip support. [BOOL]");
+#endif
+#ifdef __ISAPNP__
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for specified soundcard.");
+MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for specified soundcard. [BOOL]");
 #endif
 
 static struct snd_sb16 {
@@ -96,22 +135,22 @@ static struct snd_sb16 {
 	snd_pcm_t *pcm;
 	snd_kmixer_t *mixer;
 	snd_rawmidi_t *rmidi;
-#ifdef CONFIG_SND_SEQUENCER
-	snd_seq_device_t *synth;
-#endif
+	snd_hwdep_t *synth;
 #ifdef SND_SBAWE_EMU8000
 	snd_seq_device_t *awe;
 #endif
+#ifdef CONFIG_SND_SB16_CSP
 	snd_hwdep_t *csp;
-#ifdef CONFIG_ISAPNP
-	struct isapnp_logdev *logdev;
+#endif
+#ifdef __ISAPNP__
+	struct isapnp_dev *dev;
 #ifdef SND_SBAWE_EMU8000
-	struct isapnp_logdev *logdevwt;
+	struct isapnp_dev *devwt;
 #endif
 #endif
 } *snd_sb16_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 static unsigned int snd_sb16_pnpids[] = {
 #ifndef SND_SBAWE
 	/* Sound Blaster 16 PnP */
@@ -136,6 +175,9 @@ static unsigned int snd_sb16_pnpids[] = {
 	/* Sound Blaster Vibra16C */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0070),	  /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0001), /* Audio */
+	/* Sound Blaster Vibra16CL -- added by ctm@ardi.com */
+	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0080),   /* DEVICE */
+	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0041), /* Audio */
 	/* Sound Blaster Vibra16X */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x00f0),   /* DEVICE */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0043), /* Audio */
@@ -170,6 +212,12 @@ static unsigned int snd_sb16_pnpids[] = {
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
 #ifdef SND_SBAWE_EMU8000
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
+#endif
+	/* Sound Blaster AWE 32 PnP */
+	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x0054),   /* DEVICE */
+	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0031), /* Audio */
+#ifdef SND_SBAWE_EMU8000
+	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_FUNCTION(0x0021), /* WaveTable */
 #endif
 	/* Sound Blaster AWE 32 PnP */
 	(ISAPNP_VENDOR('C','T','L')<<16)|ISAPNP_DEVICE(0x009c),   /* DEVICE */
@@ -235,16 +283,16 @@ static void snd_sb16_use_dec(snd_card_t * card)
 	MOD_DEC_USE_COUNT;
 }
 
-static snd_pcm_t *snd_sb16_detect(int dev,
-				  snd_card_t * card,
-				  unsigned short port,
-				  snd_irq_t * irqptr,
-				  snd_dma_t * dma8ptr,
-				  snd_dma_t * dma16ptr)
+static snd_pcm_t * __init snd_sb16_detect(int dev,
+					  snd_card_t * card,
+					  unsigned short port,
+					  snd_irq_t * irqptr,
+					  snd_dma_t * dma8ptr,
+					  snd_dma_t * dma16ptr)
 {
 	snd_pcm_t *pcm;
 	sbdsp_t *codec;
-	int mpu_flag = 0;
+	int mpu_flag = 0, err;
 
 	if (snd_register_ioport(card, port, 16, "Sound Blaster 16", NULL) < 0)
 		return NULL;
@@ -274,17 +322,17 @@ static snd_pcm_t *snd_sb16_detect(int dev,
 		snd_awe_port[dev] = -1;
 	}
 #endif
-	pcm = snd_sb16dsp_new_device(card, port, irqptr,
-				     dma8ptr, dma16ptr, SB_HW_16);
-	if (!pcm) {
+	err = snd_sb16dsp_new_pcm(card, 0, port, irqptr,
+				  dma8ptr, dma16ptr, SB_HW_16, &pcm);
+	if (err < 0) {
 		snd_unregister_ioports(card);
 		return NULL;
 	}
-	codec = (sbdsp_t *) pcm->private_data;
+	codec = snd_magic_cast(sbdsp_t, pcm->private_data, NULL);
 	if (codec->hardware != SB_HW_16) {
 		snd_printdd("SB 16 chip was not detected at 0x%x\n", port);
 	      __return1:
-		snd_pcm_free(pcm);
+		snd_device_free(card, pcm);
 		snd_unregister_ioports(card);
 		return NULL;
 	}
@@ -313,9 +361,9 @@ static void snd_sb16_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 	pcm = acard->pcm;
 	if (pcm == NULL)
 		return;
-	sbdsp = (sbdsp_t *) pcm->private_data;
+	sbdsp = snd_magic_cast(sbdsp_t, pcm->private_data, );
 	spin_lock_irqsave(&sbdsp->mixer.lock, flags);
-	status = snd_sb16mixer_read(&sbdsp->mixer, 0x82);
+	status = snd_sb16mixer_read(&sbdsp->mixer, SB_DSP4_IRQSTATUS);
 	spin_unlock_irqrestore(&sbdsp->mixer.lock, flags);
 #ifdef SND_DEBUG_IRQ
 	printk("SB16: status = 0x%x\n", status);
@@ -329,8 +377,8 @@ static void snd_sb16_interrupt(int irq, void *dev_id, struct pt_regs *regs)
 #endif
 }
 
-static int snd_sb16_resources(int dev, struct snd_sb16 *acard,
-			      snd_card_t * card)
+static int __init snd_sb16_resources(int dev, struct snd_sb16 *acard,
+				     snd_card_t * card)
 {
 	static int possible_irqs[] = {5, 9, 10, 7, -1};
 	static int possible_dmas8[] = {1, 3, 0, -1};
@@ -371,211 +419,155 @@ static int snd_sb16_resources(int dev, struct snd_sb16 *acard,
 	return 0;
 }
 
-#ifdef CONFIG_ISAPNP
-static int snd_sb16_isapnp(int dev, struct snd_sb16 *acard)
+#ifdef __ISAPNP__
+static int __init snd_sb16_isapnp(int dev, struct snd_sb16 *acard)
 {
-        int idx, step;
-        unsigned int tmp;
+	static int idx = 0;
+	static struct isapnp_card *card = NULL;
 	struct isapnp_dev *pdev;
-	struct isapnp_logdev *logdev;
-	struct isapnp_config cfg;
-
 #ifndef SND_SBAWE_EMU8000
-	step = 2;
+        int step = 2;
 #else
-	step = 3;
-#endif	
-	idx = 0;
+	int step = 3;
+#endif
+        unsigned int tmp;
+
       __again:
-      	pdev = NULL;
+      	acard->dev = NULL;
+#ifdef SND_SBAWE_EMU8000
+      	acard->devwt = NULL;
+#endif
 	while ((tmp = snd_sb16_pnpids[idx]) != 0) {
-		pdev = isapnp_find_device(tmp >> 16, tmp & 0xffff, dev);
-		if (pdev)
+		card = isapnp_find_card(tmp >> 16, tmp & 0xffff, card);
+		if (card)
 			break;
 		idx += step;
 	}
-	if (!pdev) {
+	if (card == NULL) {
 		snd_printdd("isapnp failed for SB16\n");
 		return -ENODEV;
 	}
 	tmp = snd_sb16_pnpids[idx+1];
-	acard->logdev = logdev = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->logdev) {
+	acard->dev = pdev = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (pdev == NULL) {
                 snd_printdd("isapnp failed for SB16 - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
+        } else {
+        	if (pdev->active)
+        		goto __again;
         }
 #ifdef SND_SBAWE_EMU8000
 	tmp = snd_sb16_pnpids[idx+2];
-	acard->logdevwt = isapnp_find_logdev(pdev, tmp >> 16, tmp & 0xffff, 0);
-	if (!acard->logdevwt) {
+	acard->devwt = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
+	if (acard->devwt == NULL) {
                 snd_printdd("isapnp failed for SB16 (WaveTable) - %i\n", idx);
-                idx += step;
                 goto __again;           /* maybe we have another config */
+        } else {
+        	if (acard->devwt->active)
+        		goto __again;
         }
 #endif
-	if (isapnp_cfg_begin(logdev->dev->csn, logdev->number))
-		return -EAGAIN;
 	/* Audio initialization */
-	if (isapnp_config_init(&cfg, logdev)) {
-		isapnp_cfg_end();
+	if (pdev->prepare(pdev) < 0)
 		return -EAGAIN;
-	}
 	if (snd_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_port[dev];
+		isapnp_resource_change(&pdev->resource[0], snd_port[dev], 16);
 	if (snd_mpu_port[dev] != SND_AUTO_PORT)
-		cfg.port[1] = snd_mpu_port[dev];
+		isapnp_resource_change(&pdev->resource[1], snd_mpu_port[dev], 2);
 	if (snd_fm_port[dev] != SND_AUTO_PORT)
-		cfg.port[2] = snd_fm_port[dev];
+		isapnp_resource_change(&pdev->resource[2], snd_mpu_port[dev], 4);
 	if (snd_dma8[dev] != SND_AUTO_DMA)
-		cfg.dma[0] = snd_dma8[dev];
+		isapnp_resource_change(&pdev->dma_resource[0], snd_dma8[dev], 1);
 	if (snd_dma16[dev] != SND_AUTO_DMA)
-		cfg.dma[1] = snd_dma16[dev];
+		isapnp_resource_change(&pdev->dma_resource[1], snd_dma16[dev], 1);
 	if (snd_irq[dev] != SND_AUTO_IRQ)
-		cfg.irq[0] = snd_irq[dev];
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->irq_resource[0], snd_irq[dev], 1);
+	if (pdev->activate(pdev) < 0) {
 		snd_printk("SB16 isapnp configure failure (out of resources?)\n");
-		isapnp_cfg_end();
 		return -EBUSY;
 	}
-	snd_printdd("isapnp SB16: port=0x%x, mpu port=0x%x, fm port=0x%x\n", cfg.port[0], cfg.port[1], cfg.port[2]);
-	snd_printdd("isapnp SB16: dma1=%i, dma2=%i, irq=%i\n", cfg.dma[0], cfg.dma[1], cfg.irq[0]);
-	snd_port[dev] = cfg.port[0];
-	snd_mpu_port[dev] = cfg.port[1];
-	snd_fm_port[dev] = cfg.port[2];
-	snd_dma8[dev] = cfg.dma[0];
-	snd_dma16[dev] = cfg.dma[1];
-	snd_irq[dev] = cfg.irq[0];
+	snd_port[dev] = pdev->resource[0].start;
+	snd_mpu_port[dev] = pdev->resource[1].start;
+	snd_fm_port[dev] = pdev->resource[2].start;
+	snd_dma8[dev] = pdev->dma_resource[0].start;
+	snd_dma16[dev] = pdev->dma_resource[1].start;
+	snd_irq[dev] = pdev->irq_resource[0].start;
+	snd_printdd("isapnp SB16: port=0x%lx, mpu port=0x%lx, fm port=0x%lx\n",
+			snd_port[dev], snd_mpu_port[dev], snd_fm_port[dev]);
+	snd_printdd("isapnp SB16: dma1=%i, dma2=%i, irq=%i\n",
+			snd_dma8[dev], snd_dma16[dev], snd_irq[dev]);
 #ifdef SND_SBAWE_EMU8000
 	/* WaveTable initialization */
-	if (isapnp_config_init(&cfg, acard->logdevwt)) {
-		isapnp_cfg_end();
+	pdev = acard->devwt;
+	if (pdev->prepare(pdev)<0) {
+		acard->dev->deactivate(acard->dev);
 		return -EAGAIN;
 	}
 	if (snd_awe_port[dev] != SND_AUTO_PORT)
-		cfg.port[0] = snd_awe_port[dev];
-	cfg.port_disable[0] = snd_port[dev];
-	cfg.port_disable_size[0] = 16;
-	cfg.port_disable[1] = snd_mpu_port[dev];
-	cfg.port_disable_size[1] = 2;
-	cfg.port_disable[2] = snd_fm_port[dev];
-	cfg.port_disable_size[2] = 4;
-	if (isapnp_configure(&cfg)<0) {
+		isapnp_resource_change(&pdev->resource[0], snd_awe_port[dev], 4);
+	if (pdev->activate(pdev)<0) {
 		snd_printk("SB WaveTable isapnp configure failure (out of resources?)\n");
-		isapnp_cfg_end();
+		acard->dev->deactivate(acard->dev);		
 		return -EBUSY;
 	}
-	snd_awe_port[dev] = cfg.port[0];
+	snd_awe_port[dev] = pdev->resource[0].start;
 	/* set other two ports which aren't listed in PnP EEPROM */
-	isapnp_cfg_set_word(ISAPNP_CFG_PORT + (1 << 1), cfg.port[0] + (0xa40 - 0x640));
-	isapnp_cfg_set_word(ISAPNP_CFG_PORT + (2 << 1), cfg.port[0] + (0xe40 - 0x640));
-	snd_printdd("isapnp SB16: wavetable port=0x%x\n", cfg.port[0]);
-#endif
-        isapnp_activate(acard->logdev->number);
-#ifdef SND_SBAWE_EMU8000
-	isapnp_activate(acard->logdevwt->number);
-#endif
+	/* FIXME: remove for 2.4!!! */
+	isapnp_cfg_begin(pdev->bus->number, pdev->devfn);
+	isapnp_write_word(ISAPNP_CFG_PORT + (1 << 1), snd_awe_port[dev] + (0xa40 - 0x640));
+	isapnp_write_word(ISAPNP_CFG_PORT + (2 << 1), snd_awe_port[dev] + (0xe40 - 0x640));
 	isapnp_cfg_end();
+	snd_printdd("isapnp SB16: wavetable port=0x%lx\n", pdev->resource[0].start);
+#endif
 	return 0;
 }
 
 static void snd_sb16_deactivate(struct snd_sb16 *acard)
 {
-	if (!acard->logdev)
-		return;
-	if (isapnp_cfg_begin(acard->logdev->dev->csn, acard->logdev->number)<0)
-		return;
-	isapnp_deactivate(acard->logdev->number);
+	if (acard->dev)
+		acard->dev->deactivate(acard->dev);
 #ifdef SND_SBAWE_EMU8000
-	if (acard->logdevwt)
-		isapnp_deactivate(acard->logdevwt->number);
+	if (acard->devwt)
+		acard->devwt->deactivate(acard->devwt);
 #endif
-	isapnp_cfg_end();
 }
-#endif /* CONFIG_ISAPNP */
+#endif /* __ISAPNP__ */
 
-#ifdef SND_SBAWE
-static int snd_sbawe3d_get_switch(snd_kmixer_t *mixer, snd_kswitch_t *kswitch, snd_switch_t *uswitch)
+static int __init snd_sb16_probe(int dev, struct snd_sb16 *acard)
 {
-	unsigned long flags;
-	sbmixer_t *sbmix;
-  
-	sbmix = (sbmixer_t *)kswitch->private_data;
-	uswitch->type = SND_SW_TYPE_BOOLEAN;
-	spin_lock_irqsave(&sbmix->lock, flags);
-	uswitch->value.enable =
-		snd_sb16mixer_read(sbmix, SB_DSP4_3D) & 0x01 ? 1 : 0;
-	/*printk("snd-sbawe: get switch %d\n", uswitch->value.enable);*/
-	spin_unlock_irqrestore(&sbmix->lock, flags);
-	return 0;
-}
-
-static int snd_sbawe3d_set_switch(snd_kmixer_t *mixer, snd_kswitch_t *kswitch, snd_switch_t *uswitch)
-{
-	unsigned long flags;
-	sbmixer_t *sbmix;
-	int change = 0;
-	unsigned char reg, val;
-	
-	sbmix = (sbmixer_t *)kswitch->private_data;
-	if (uswitch->type != SND_SW_TYPE_BOOLEAN)
-		return -EINVAL;
-	val = uswitch->value.enable ? 0x01 : 0x00;
-	spin_lock_irqsave(&sbmix->lock, flags);
-	reg = snd_sb16mixer_read(sbmix, SB_DSP4_3D);
-	change = (reg & 0x01) != val ? 1 : 0;
-	snd_sb16mixer_write(sbmix, SB_DSP4_3D, (reg & ~0x01) | val);
-	spin_unlock_irqrestore(&sbmix->lock, flags);
-	return change;
-}
-
-snd_kswitch_t snd_sbawe3d_switch = {
-	"3D Stereo Enhancement",
-	(snd_get_switch_t *)snd_sbawe3d_get_switch,
-	(snd_set_switch_t *)snd_sbawe3d_set_switch,
-	0,
-	NULL,
-	NULL
-};
-#endif
-
-static int snd_sb16_probe(int dev, struct snd_sb16 *acard)
-{
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	static int possible_ports[] = {0x220, 0x240, 0x260, -1};
 	int *ports = possible_ports;
 #endif
 	sbdsp_t *codec;
 	snd_card_t *card;
 	snd_pcm_t *pcm = NULL;
-	int pcm_registered = 0;
 	snd_kmixer_t *mixer = NULL;
-	int mixer_registered = 0;
 	snd_rawmidi_t *rmidi = NULL;
-	int rmidi_registered = 0;
-#ifdef CONFIG_SND_SEQUENCER
-	snd_seq_device_t *synth = NULL;
-#endif
+	snd_hwdep_t *synth = NULL;
 #ifdef SND_SBAWE_EMU8000
 	snd_seq_device_t *awe = NULL;
 #endif
+#ifdef CONFIG_SND_SB16_CSP
 	snd_hwdep_t *csp = NULL;
-	int csp_registered = 0;
+#endif
+	unsigned long flags;
+	int err;
 
 	card = snd_card_new(snd_index[dev], snd_id[dev],
 			    snd_sb16_use_inc, snd_sb16_use_dec);
-	if (!card)
+	if (card == NULL)
 		return -ENOMEM;
 	card->static_data = acard;
 	card->type = SND_CARD_TYPE_SB_16;	/* overriden in probe function */
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if (snd_sb16_resources(dev, acard, card) < 0) {
 		snd_card_free(card);
 		return -EBUSY;
 	}
 #endif
 	pcm = NULL;
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if (snd_port[dev] == SND_AUTO_PORT) {
 		for (ports = possible_ports; *ports >= 0; ports++) {
 			pcm = snd_sb16_detect(dev, card, *ports,
@@ -600,77 +592,74 @@ static int snd_sb16_probe(int dev, struct snd_sb16 *acard)
 				      acard->irqptr,
 				      acard->dma8ptr,
 				      acard->dma16ptr);
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	}
 #endif
-	if (!pcm) {
+	if (pcm == NULL) {
 		snd_card_free(card);
 		return -ENODEV;
 	}
+	codec = snd_magic_cast(sbdsp_t, pcm->private_data, -ENXIO);
 
-	codec = (sbdsp_t *) pcm->private_data;
-	mixer = snd_sb16dsp_new_mixer(card, &codec->mixer, codec->hardware, 0);
-	if (!mixer)
+	if (codec->mpu_port) {
+		err = snd_mpu401_uart_new(card, 0, MPU401_HW_SB,
+					  codec->mpu_port,
+					  acard->irqptr->irq, &rmidi);
+		if (err < 0)
+			goto __nodev;
+	}
+
+	snd_opl3_new(card, 0, codec->port, codec->port + 2,
+		     OPL3_HW_OPL3, -1, &synth);
+
+	err = snd_sb16dsp_new_mixer(codec, 0, pcm, &mixer);
+	if (err < 0)
 		goto __nodev;
 
-#ifdef SND_SBAWE
-	snd_mixer_switch_new(mixer, &snd_sbawe3d_switch, &codec->mixer);
-#endif
-
-	/* FIXME: this is not working yet.. Arg should be allocated */
-	/*synth = snd_opl3_new_device(card, codec->port, codec->port + 2,
-				    OPL3_HW_OPL3, -1);*/
-#ifdef CONFIG_SND_SEQUENCER
-	snd_seq_device_register(card, 0, NULL, SND_SEQ_DEV_OPL3, NULL, 0, &synth);
-#endif
-#ifdef SND_SBAWE_EMU8000
-	if (snd_awe_port[dev] > 0) {
-		snd_seq_device_register(card, 1, "EMU-8000", SND_SEQ_DEV_EMU8000, (void*)snd_awe_port[dev], 0, &awe);
-		if (!awe) {
-			snd_printk("sbawe: fatal error - EMU-8000 synthesizer not detected at 0x%x\n", snd_awe_port[dev]);
-			goto __nodev;
+#ifdef CONFIG_SND_SB16_CSP
+	/* CSP chip on SB16ASP/AWE32 */
+	if ((codec->hardware == SB_HW_16) && snd_csp[dev]) {
+		snd_sb_csp_t *p;
+		snd_sb_csp_new(codec, 2, &csp);
+		if (csp) {
+			p = snd_magic_cast(snd_sb_csp_t, csp->private_data, -ENXIO);
+			p->mixer = mixer;
+			if (mixer)
+				p->mixer_dest = codec->mixer.me_vol_pcm;
+			snd_sb_csp_register_callbacks((snd_sb_csp_callback_t **) &codec->csp_callbacks);
+			codec->csp_private_data = p;
+			codec->csp_acquired = 0;
+			codec->hardware = SB_HW_16CSP;
+		} else {
+			snd_printk("sb16: warning - CSP chip not detected on soundcard #%i\n", dev + 1);
 		}
 	}
 #endif
-#ifndef SND_TEST_SB_OLD_MIDI
-	if (codec->mpu_port) {
-		rmidi = snd_mpu401_uart_new_device(card, MPU401_HW_SB,
-						   codec->mpu_port,
-						   acard->irqptr->irq);
-		if (!rmidi)
+#ifdef SND_SBAWE_EMU8000
+	if (snd_awe_port[dev] > 0) {
+		emu8000_arg_t *awe_arg;
+		snd_seq_device_new(card, 1, SND_SEQ_DEV_ID_EMU8000,
+				   sizeof(emu8000_arg_t), &awe);
+		if (awe == NULL) {
+			snd_printk("sbawe: fatal error - EMU-8000 synthesizer not detected at 0x%x\n", snd_awe_port[dev]);
 			goto __nodev;
+		}
+		strcpy(awe->name, "EMU-8000");
+		awe_arg = SND_SEQ_DEVICE_ARGPTR(awe);
+		awe_arg->port = snd_awe_port[dev];
+		awe_arg->mixer = mixer;
+		awe_arg->mixer_index = 0;
+		if (mixer)
+			awe_arg->mixer_dest = codec->mixer.me_vol_synth;
 	}
-#else
-	rmidi = snd_sbdsp_midi_new_device(card, pcm);
-	if (!rmidi)
-		goto __nodev;
 #endif
 
-	if (rmidi) {
-		if (snd_rawmidi_register(rmidi, 0) < 0)
-			goto __nodev;
-		rmidi_registered = 1;
-	}
-	if (snd_mixer_register(mixer, 0) < 0)
-		goto __nodev;
-	mixer_registered = 1;
-
-	if (snd_pcm_register(pcm, 0))
-		goto __nodev;
-	pcm_registered = 1;
-
-	/* CSP chip on SB16ASP/AWE32 */
-	csp = snd_sb_csp_new_device(codec);
-	if (csp) {
-		if (snd_hwdep_register(csp, 2))
-			goto __nodev;
-		csp_registered = 1;
-	}
-
 	/* setup Mic AGC */
+	spin_lock_irqsave(&codec->mixer.lock, flags);
 	snd_sb16mixer_write(&codec->mixer, SB_DSP4_MIC_AGC,
 		(snd_sb16mixer_read(&codec->mixer, SB_DSP4_MIC_AGC) & 0x01) |
 		(snd_mic_agc[dev] ? 0x00 : 0x01));
+	spin_unlock_irqrestore(&codec->mixer.lock, flags);
 
 	strcpy(card->abbreviation, "SB16");
 	strcpy(card->shortname, "Sound Blaster 16");
@@ -689,10 +678,12 @@ static int snd_sb16_probe(int dev, struct snd_sb16 *acard)
 		acard->card = card;
 		acard->pcm = pcm;
 		acard->mixer = mixer;
+#ifdef CONFIG_SND_SB16_CSP
 		acard->csp = csp;
+#endif
 		acard->rmidi = rmidi;
 #ifdef CONFIG_SND_SEQUENCER
-		acard->synth = synth;
+		/*acard->synth = synth;*/
 #endif
 #ifdef SND_SBAWE_EMU8000
 		acard->awe = awe;
@@ -701,43 +692,13 @@ static int snd_sb16_probe(int dev, struct snd_sb16 *acard)
 	}
 
       __nodev:
-	if (csp) {
-		if (csp_registered)
-			snd_hwdep_unregister(csp);
-		else
-			snd_hwdep_free(csp);
-	}
-#ifdef CONFIG_SND_SEQUENCER
-	if (synth)
-		snd_seq_device_unregister(synth);
-#endif
-#ifdef SND_SBAWE_EMU8000
-	if (awe)
-		snd_seq_device_unregister(awe);
-#endif
-	if (rmidi) {
-		if (rmidi_registered)
-			snd_rawmidi_unregister(rmidi);
-		else
-			snd_rawmidi_free(rmidi);
-	}
-	if (mixer) {
-		if (mixer_registered)
-			snd_mixer_unregister(mixer);
-		else
-			snd_mixer_free(mixer);
-	}
-	if (pcm) {
-		if (pcm_registered)
-			snd_pcm_unregister(pcm);
-		else
-			snd_pcm_free(pcm);
-	}
 	snd_card_free(card);
 	return -ENXIO;
 }
 
-static int snd_sb16_free(int dev)
+#ifdef MODULE
+
+static int __exit snd_sb16_free(int dev)
 {
 	struct snd_sb16 *acard;
 
@@ -745,26 +706,7 @@ static int snd_sb16_free(int dev)
 	snd_sb16_cards[dev] = NULL;
 	if (acard) {
 		snd_card_unregister(acard->card);
-		if (acard->csp)
-			snd_hwdep_unregister(acard->csp);
-#ifdef CONFIG_SND_SEQUENCER
-		if (acard->synth)
-			snd_seq_device_unregister(acard->synth);
-#endif
-#ifdef SND_SBAWE_EMU8000
-		if (acard->awe)
-			snd_seq_device_unregister(acard->awe);
-#endif
-		if (acard->rmidi)
-			snd_rawmidi_unregister(acard->rmidi);
-		if (acard->mixer)
-			snd_mixer_unregister(acard->mixer);
-		if (acard->pcm) {
-			snd_pcm_unregister(acard->pcm);
-			acard->pcm = NULL;
-		}
-		snd_card_free(acard->card);
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 		snd_sb16_deactivate(acard);
 #endif
 		snd_kfree(acard);
@@ -772,39 +714,57 @@ static int snd_sb16_free(int dev)
 	return 0;
 }
 
-int init_module(void)
+#endif
+
+#ifdef MODULE
+int __init init_module(void)
+#else
+#ifdef SND_SBAWE_EMU8000
+int __init alsa_card_sbawe_init(void)
+#else
+int __init alsa_card_sb16_init(void)
+#endif
+#endif
 {
 	int dev, cards;
 	struct snd_sb16 *acard;
 
 	for (dev = cards = 0; dev < SND_CARDS && snd_port[dev] > 0; dev++) {
 		acard = (struct snd_sb16 *) snd_kcalloc(sizeof(struct snd_sb16), GFP_KERNEL);
-		if (!acard)
+		if (acard == NULL)
 			continue;
 		if (snd_sb16_probe(dev, acard) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 			snd_sb16_deactivate(acard);
 #endif
 			snd_kfree(acard);
 			if (snd_port[dev] == SND_AUTO_PORT)
 				break;
+#ifdef MODULE
 			snd_printk("Sound Blaster 16 soundcard #%i not found at 0x%x or device busy\n", dev + 1, snd_port[dev]);
+#endif
 			continue;
 		}
 		snd_sb16_cards[dev] = acard;
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk("Sound Blaster 16 soundcard #%i not found or device busy\n", dev + 1);
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int dev;
 
 	for (dev = 0; dev < SND_CARDS; dev++)
 		snd_sb16_free(dev);
 }
+
+#endif
