@@ -25,6 +25,14 @@
 #include "../include/opl3.h"
 #include "../include/snd_wavefront.h"
 
+MODULE_DESCRIPTION("\
+Driver: Turtle Beach Wavefront\n\
+Card: Turtle Beach Maui\n\
+Card: Turtle Beach Tropez\n\
+Card: Turtle Beach Tropez+\n\
+ISAPNP: CSC0000=CSC0010,CSCb006,CSC0004\n\
+");
+
 int snd_index[SND_CARDS] = SND_DEFAULT_IDX;	        /* Index 1-MAX */
 char *snd_id[SND_CARDS] = SND_DEFAULT_STR;	        /* ID for this card */
 int snd_cs4232_pcm_port[SND_CARDS] = SND_DEFAULT_PORT;	/* PnP setup */
@@ -44,48 +52,47 @@ int snd_isapnp_id = -1;         /* PnP; 0x7632 = Tropez+, 0x7532 = Tropez */
 #ifdef MODULE_PARM
 
 MODULE_AUTHOR("Paul Barton-Davis <pbd@op.net>");
-MODULE_DESCRIPTION("ALSA driver for Turtle Beach WaveFront soundcards");
 MODULE_PARM(snd_index, "1-" __MODULE_STRING(SND_CARDS) "i");
 MODULE_PARM_DESC(snd_index, "Index value for WaveFront soundcard.");
 MODULE_PARM(snd_id, "1-" __MODULE_STRING(SND_CARDS) "s");
 MODULE_PARM_DESC(snd_id, "ID string for WaveFront soundcard.");
 MODULE_PARM(snd_cs4232_pcm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_4232_port, "Port # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_4232_port, "Port # for CS4232 PCM interface. [PORT12]");
 MODULE_PARM(snd_cs4232_pcm_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_cs4232_pcm_irq, "IRQ # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_cs4232_pcm_irq, "IRQ # for CS4232 PCM interface. [list=5,7,9,11,12,15]");
 MODULE_PARM(snd_dma1, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1, "DMA1 # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_dma1, "DMA1 # for CS4232 PCM interface. [list=0,1,3,5,6,7]");
 MODULE_PARM(snd_dma1_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_dma1_size, "Size of DMA1 # for CS4232 PCM interface. [DMA_SIZE]");
 MODULE_PARM(snd_dma2, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2, "DMA2 # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_dma2, "DMA2 # for CS4232 PCM interface. [list=0,1,3,5,6,7]");
 MODULE_PARM(snd_dma2_size, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for CS4232 PCM interface.");
+MODULE_PARM_DESC(snd_dma2_size, "Size of DMA2 # for CS4232 PCM interface. [DMA_SIZE]");
 MODULE_PARM(snd_cs4232_mpu_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_cs4232_mpu_port, "port # for CS4232 MPU-401 interface.");
+MODULE_PARM_DESC(snd_cs4232_mpu_port, "port # for CS4232 MPU-401 interface. [PORT12]");
 MODULE_PARM(snd_cs4232_mpu_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_cs4232_mpu_irq, "IRQ # for CS4232 MPU-401 interface.");
+MODULE_PARM_DESC(snd_cs4232_mpu_irq, "IRQ # for CS4232 MPU-401 interface. [list=9,11,12,15]");
 MODULE_PARM(snd_ics2115_irq, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_ics2115_irq, "IRQ # for ICS2115.");
+MODULE_PARM_DESC(snd_ics2115_irq, "IRQ # for ICS2115. [list=9,11,12,15]");
 MODULE_PARM(snd_ics2115_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_ics2115_port, "Port # for ICS2115.");
+MODULE_PARM_DESC(snd_ics2115_port, "Port # for ICS2115. [PORT12]");
 MODULE_PARM(snd_fm_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_fm_port, "FM port #.");
+MODULE_PARM_DESC(snd_fm_port, "FM port #. [PORT12]");
 MODULE_PARM(snd_wss_port, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_wss_port, "Windows Sound System port #.");
+MODULE_PARM_DESC(snd_wss_port, "Windows Sound System port #. [PORT12]");
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 MODULE_PARM(snd_isapnp, "1-" __MODULE_STRING(SND_CARDS) "i");
-MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for WaveFront soundcards.");
+MODULE_PARM_DESC(snd_isapnp, "ISA PnP detection for WaveFront soundcards. [BOOL]");
 MODULE_PARM(snd_isapnp_id, "i");
-MODULE_PARM_DESC(snd_isapnp_id, "ISA PnP Device ID (0x7532 or 0x7632)");
+MODULE_PARM_DESC(snd_isapnp_id, "ISA PnP Device ID (0x7532 or 0x7632) [list=0x7532,0x7632]");
 #endif
 
 #endif MODULE_PARM
 
 static snd_wavefront_card_t *wavefront_cards[SND_CARDS] = SND_DEFAULT_PTR;
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 
 static unsigned int snd_wavefront_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0000), /* WSS */
@@ -94,7 +101,7 @@ static unsigned int snd_wavefront_pnpids[] = {
 	(ISAPNP_VENDOR('C','S','C')<<16)|ISAPNP_FUNCTION(0x0004), /* SYNTH */
 };
 
-static int 
+static int __init
 snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 {
 	static struct isapnp_card *card = NULL;
@@ -116,7 +123,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 
 	tmp = snd_wavefront_pnpids[0];
 	acard->wss = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
-	if (!acard->wss) {
+	if (acard->wss == NULL) {
 		snd_printk (LOGNAME "isapnp failed for WSS device\n");
 		return -ENODEV;
 	}
@@ -125,7 +132,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 
 	tmp = snd_wavefront_pnpids[1];
 	acard->ctrl = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
-	if (!acard->ctrl) {
+	if (acard->ctrl == NULL) {
 		snd_printk (LOGNAME "isapnp failed for CTRL device\n");
 		return -ENODEV;
 	}
@@ -134,7 +141,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 
 	tmp = snd_wavefront_pnpids[3];
 	acard->synth = isapnp_find_dev(card, tmp >> 16, tmp & 0xffff, NULL);
-	if (!acard->synth) {
+	if (acard->synth == NULL) {
 		snd_printk (LOGNAME "isapnp failed for Synth device\n");
 		return -ENODEV;
 	}
@@ -147,7 +154,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 		tmp = snd_wavefront_pnpids[2];
 		acard->mpu = isapnp_find_dev(card, tmp >> 16, 
 					     tmp & 0xffff, NULL);
-		if (!acard->mpu) {
+		if (acard->mpu == NULL) {
 			snd_printk ("isapnp failed for CS4232 MPU-401\n");
 			return -ENODEV;
 		}
@@ -184,7 +191,7 @@ snd_wavefront_isapnp (int dev, snd_wavefront_card_t *acard, int pnp_id)
 	snd_cs4232_pcm_port[dev] = pdev->resource[0].start;
 	snd_fm_port[dev] = pdev->resource[1].start;
 	snd_wss_port[dev] = pdev->resource[2].start;
-	snd_ics2115_port[dev] = pdev->resource[4].start;
+	snd_ics2115_port[dev] = pdev->resource[3].start;
 
 	snd_dma1[dev] = pdev->dma_resource[0].start;
 	snd_dma2[dev] = pdev->dma_resource[1].start == 4 ? -1 : pdev->dma_resource[1].start;
@@ -302,7 +309,7 @@ snd_wavefront_deactivate (snd_wavefront_card_t *acard)
 		acard->synth->deactivate(acard->synth);
 }
 
-#endif CONFIG_ISAPNP
+#endif __ISAPNP__
 
 static void snd_wavefront_use_inc(snd_card_t * card)
 {
@@ -359,9 +366,9 @@ static void snd_wavefront_ics2115_interrupt(int irq,
 	}
 }
 
-static int snd_wavefront_resources(int dev, 
-				   snd_wavefront_card_t *acard,
-				   snd_card_t * card)
+static int __init snd_wavefront_resources(int dev, 
+					  snd_wavefront_card_t *acard,
+					  snd_card_t * card)
 {
 	static int possible_irqs[] = {-1};
 	static int possible_dmas[] = {-1};
@@ -540,8 +547,9 @@ static int snd_wavefront_resources(int dev,
 	return -1;
 }
 
-snd_hwdep_t *
+snd_hwdep_t * __init
 snd_wavefront_new_synth (snd_card_t *card,
+			 int hw_dev,
 			 snd_wavefront_card_t *acard,
 			 unsigned short port)
 
@@ -556,7 +564,8 @@ snd_wavefront_new_synth (snd_card_t *card,
 		return NULL;
 	}
 
-	wavefront_synth = snd_hwdep_new_device (card, "WaveFront");
+	if (snd_hwdep_new(card, "WaveFront", hw_dev, &wavefront_synth) < 0)
+		return NULL;
 	strcpy (wavefront_synth->name, 
 		"WaveFront (ICS2115) wavetable synthesizer");
 	wavefront_synth->ops.open = snd_wavefront_synth_open;
@@ -567,8 +576,9 @@ snd_wavefront_new_synth (snd_card_t *card,
 	return wavefront_synth;
 }
 
-snd_hwdep_t *
+snd_hwdep_t * __init
 snd_wavefront_new_fx (snd_card_t *card,
+		      int hw_dev,
 		      snd_wavefront_card_t *acard,
 		      unsigned short port)
 
@@ -580,7 +590,8 @@ snd_wavefront_new_fx (snd_card_t *card,
 		return NULL;
 	}
 
-	fx_processor = snd_hwdep_new_device (card, "YSS225");
+	if (snd_hwdep_new (card, "YSS225", hw_dev, &fx_processor) < 0)
+		return NULL;
 	sprintf (fx_processor->name, "YSS225 FX Processor at %d", port);
 	fx_processor->ops.open = snd_wavefront_fx_open;
 	fx_processor->ops.release = snd_wavefront_fx_release;
@@ -592,8 +603,9 @@ snd_wavefront_new_fx (snd_card_t *card,
 static snd_wavefront_mpu_id internal_id = internal_mpu;
 static snd_wavefront_mpu_id external_id = external_mpu;
 
-snd_rawmidi_t *
+snd_rawmidi_t * __init
 snd_wavefront_new_midi (snd_card_t *card,
+			int midi_dev,
 			snd_wavefront_card_t *acard,
 			unsigned short port,
 			snd_wavefront_mpu_id mpu)
@@ -612,9 +624,8 @@ snd_wavefront_new_midi (snd_card_t *card,
 		}
 	}
 
-	if ((rmidi = snd_rawmidi_new_device (card, "WaveFront MIDI")) == NULL){
+	if (snd_rawmidi_new (card, "WaveFront MIDI", midi_dev, &rmidi) < 0)
 		return NULL;
-	}
 
 	if (mpu == internal_mpu) {
 		strcpy(rmidi->name, "WaveFront MIDI (Internal)");
@@ -624,9 +635,9 @@ snd_wavefront_new_midi (snd_card_t *card,
 		rmidi->private_data = &external_id;
 	}
 
-	memcpy (&rmidi->output.hw, &snd_wavefront_midi_output, 
+	memcpy (&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_wavefront_midi_output, 
 		sizeof (snd_wavefront_midi_output));
-	memcpy (&rmidi->input.hw, &snd_wavefront_midi_input, 
+	memcpy (&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_wavefront_midi_input, 
 	        sizeof (snd_wavefront_midi_input));
 
 	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT |
@@ -636,18 +647,7 @@ snd_wavefront_new_midi (snd_card_t *card,
 	return rmidi;
 }
 
-/* some bits so we can clean up if things fail midway */
-
-#define PCM_REGISTERED             0x1
-#define WAVEFRONT_REGISTERED       0x2
-#define MIXER_REGISTERED           0x4
-#define CS4232_MIDI_REGISTERED     0x8
-#define ICS2115_INTMIDI_REGISTERED 0x10
-#define ICS2115_EXTMIDI_REGISTERED 0x20
-#define OPL3_REGISTERED            0x40
-#define FX_REGISTERED              0x80
-
-static int 
+static int __init
 snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 {
 	snd_card_t *card;
@@ -660,8 +660,7 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 	snd_hwdep_t *wavefront_synth = NULL;
 	snd_hwdep_t *fx_processor = NULL;
 	cs4231_t *codec;
-	int err;
-	int registered_bits = 0;
+	int hw_dev = 0, midi_dev = 0;
 
 	if (snd_cs4232_mpu_port[dev] < 0) {
 		snd_cs4232_mpu_port[dev] = SND_AUTO_PORT;
@@ -674,23 +673,23 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 		snd_ics2115_port[dev] = SND_AUTO_PORT;
 	}
 
-#ifndef CONFIG_ISAPNP
+#ifndef __ISAPNP__
 	if (snd_cs4232_pcm_port[dev] == SND_AUTO_PORT ||
 	    snd_ics2115_port[dev] == SND_AUTO_PORT) {
 		snd_printk (LOGNAME "probing isn't supported\n");
 		return -ENODEV;
 	}
-#endif  CONFIG_ISAPNP
+#endif  __ISAPNP__
 	card = snd_card_new (snd_index[dev], 
 			     snd_id[dev],
 			     snd_wavefront_use_inc, 
 			     snd_wavefront_use_dec);
 
-	if (!card) {
+	if (card == NULL) {
 		return -ENOMEM;
 	}
 
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	if (snd_isapnp_id == -1) {
 		snd_printk (LOGNAME "option snd_isapnp_id not set\n");
 		return -EBUSY;
@@ -704,125 +703,84 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 			return -ENODEV;
 		}
 	}
-#endif CONFIG_ISAPNP
+#endif __ISAPNP__
 
 	card->private_data = acard;
 	card->private_free = NULL;
 
 	if (snd_wavefront_resources (dev, acard, card) < 0) {
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 		snd_wavefront_deactivate (acard);
-#endif CONFIG_ISAPNP
+#endif __ISAPNP__
 		snd_card_free (card);
 		return -EBUSY;
 	}
 
 	/* --------- PCM --------------- */
 
-	pcm = snd_cs4231_new_device (card, 
-				     snd_cs4232_pcm_port[dev], 
-				     acard->cs4232_pcm_irqptr,
-				     acard->dma1ptr, 
-				     acard->dma2ptr ? 
+	if (snd_cs4231_new_pcm (card, 0, 
+				snd_cs4232_pcm_port[dev], 
+				acard->cs4232_pcm_irqptr,
+				acard->dma1ptr, 
+				acard->dma2ptr ? 
 				     acard->dma2ptr : acard->dma1ptr,
-				     CS4231_HW_DETECT, 0);
-	
-	if (!pcm) {
+				CS4231_HW_DETECT, 0, &pcm) < 0) {
 		snd_printk (LOGNAME "can't allocate PCM device\n");
 		goto __nodev;
 	}
 
-	codec = (cs4231_t *) pcm->private_data;
+	codec = snd_magic_cast(cs4231_t, pcm->private_data, -ENXIO);
 	acard->codec = codec;
 	acard->pcm_status_reg = codec->port + 2;
 
-	if (snd_pcm_register (pcm, 0)) {
-		snd_printk (LOGNAME "can't register PCM device\n");
-		goto __nodev;
-	}
-	
-	registered_bits |= PCM_REGISTERED;
-
 	/* ------- ICS2115 Wavetable synth (w/MIDI) ------- */
 
-	wavefront_synth = snd_wavefront_new_synth
-		(card, acard, snd_ics2115_port[dev]);
+	wavefront_synth = snd_wavefront_new_synth (card, hw_dev, acard, snd_ics2115_port[dev]);
 
-	if (!wavefront_synth) {
+	if (wavefront_synth == NULL) {
 		snd_printk (LOGNAME "can't allocate WaveFront synth device\n");
 		goto __nodev;
 	}
 
 	strcpy (wavefront_synth->name, "ICS2115 Wavetable MIDI Synthesizer");
 	wavefront_synth->type = SND_HWDEP_TYPE_ICS2115;
-
-	if ((err = snd_hwdep_register (wavefront_synth, 0)) < 0) {
-		snd_printk (LOGNAME "can't register WaveFront synth "
-			    "device (%d)\n",
-			    -err);
-		goto __nodev;
-	}
-
-	registered_bits |= WAVEFRONT_REGISTERED;
+	hw_dev++;
 
 	/* --------- Mixer ------------ */
 
-	mixer = snd_cs4231_new_mixer(pcm, 0);
-
-	if (!mixer) {
+	if (snd_cs4231_new_mixer(pcm, 0, &mixer) < 0) {
 		snd_printk (LOGNAME "can't allocate mixer device\n");
 		goto __nodev;
 	}
 
-	if (snd_mixer_register(mixer, 0) < 0) {
-		snd_printk (LOGNAME "can't register mixer device\n");
-		goto __nodev;
-	}
-
-	registered_bits |= MIXER_REGISTERED;
-
 	/* ---------- OPL3 synth --------- */
 
 	if (snd_fm_port[dev] != SND_AUTO_PORT) {
-	        fm_synth = snd_opl3_new_device(card,
-					       snd_fm_port[dev],
-					       snd_fm_port[dev] + 2,
-					       OPL3_HW_OPL3_CS, -1);
-		if (!fm_synth) {
+	        if (snd_opl3_new(card, hw_dev,
+				 snd_fm_port[dev],
+				 snd_fm_port[dev] + 2,
+				 OPL3_HW_OPL3_CS, -1,
+				 &fm_synth) < 0) {
 			snd_printk (LOGNAME "can't allocate OPL3 synth\n");
 			goto __nodev;
 		}
 
 		strcpy (fm_synth->name, "Yamaha OPL3 4-Operator FM Synthesizer");
 		fm_synth->type = SND_HWDEP_TYPE_OPL3;
-
-		if (snd_hwdep_register(fm_synth, 1) < 0) {
-			snd_printk (LOGNAME "can't register OPL3 synth\n");
-			goto __nodev;
-		}
-
-		registered_bits |= OPL3_REGISTERED;
+		hw_dev++;
 	}
 
-
 	/* -------- CS4232 MPU-401 interface -------- */
 
 	if (acard->cs4232_mpu_irqptr) {
-		cs4232_rmidi = 
-			snd_mpu401_uart_new_device(card, MPU401_HW_CS4232,
-						   snd_cs4232_mpu_port[dev],
-						   acard->cs4232_mpu_irqptr->irq);
-		if (!cs4232_rmidi) {
+		if (snd_mpu401_uart_new(card, midi_dev, MPU401_HW_CS4232,
+					snd_cs4232_mpu_port[dev],
+					acard->cs4232_mpu_irqptr->irq,
+					&cs4232_rmidi) < 0) {
 			snd_printk (LOGNAME "can't allocate CS4232 MPU-401 device\n");
 			goto __nodev;
 		}
-
-		if (snd_rawmidi_register(cs4232_rmidi, 0) < 0) {
-			snd_printk (LOGNAME "can't register CS4232 MPU-401 device\n");
-			goto __nodev;
-		}
-
-		registered_bits |= CS4232_MIDI_REGISTERED;
+		midi_dev++;
 	}
 
 	/* ------ ICS2115 internal MIDI ------------ */
@@ -830,19 +788,15 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 	if (acard->ics2115_irqptr) {
 		ics2115_internal_rmidi = 
 			snd_wavefront_new_midi (card, 
+						midi_dev,
 						acard,
 						snd_ics2115_port[dev],
 						internal_mpu);
-		if (!ics2115_internal_rmidi) {
+		if (ics2115_internal_rmidi == NULL) {
 			snd_printk (LOGNAME "can't allocate ICS2115 internal MIDI device\n");
 			goto __nodev;
 		}
-
-		if (snd_rawmidi_register(ics2115_internal_rmidi, 0) < 0) {
-			snd_printk (LOGNAME "can't register ICS2115 internal MIDI  device\n");
-			goto __nodev;
-		}
-		registered_bits |= ICS2115_INTMIDI_REGISTERED;
+		midi_dev++;
 	}
 
 	/* ------ ICS2115 external MIDI ------------ */
@@ -850,42 +804,30 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 	if (acard->ics2115_irqptr) {
 		ics2115_external_rmidi = 
 			snd_wavefront_new_midi (card, 
+						midi_dev,
 						acard,
 						snd_ics2115_port[dev],
 						external_mpu);
-		if (!ics2115_external_rmidi) {
+		if (ics2115_external_rmidi == NULL) {
 			snd_printk (LOGNAME "can't allocate ICS2115 external MIDI  device\n");
 			goto __nodev;
 		}
-
-		if (snd_rawmidi_register(ics2115_external_rmidi, 1) < 0) {
-			snd_printk (LOGNAME "can't register ICS2115 external MIDI device\n");
-			goto __nodev;
-		}
-
-		registered_bits |= ICS2115_EXTMIDI_REGISTERED;
+		midi_dev++;
 	}
 
 	/* FX processor for Tropez+ */
 
 	if (acard->wavefront.has_fx) {
 		fx_processor = snd_wavefront_new_fx (card,
+						     hw_dev,
 						     acard,
 						     snd_ics2115_port[dev]);
-		if (!fx_processor) {
+		if (fx_processor == NULL) {
 			snd_printk (LOGNAME "can't allocate FX device\n");
 			goto __nodev;
 		}
 
-		if (snd_hwdep_register (fx_processor, 2) < 0) {
-			snd_printk (LOGNAME "can't register FX device\n");
-			goto __nodev;
-		}
-
-		registered_bits |= FX_REGISTERED;
-
-		strcpy (fx_processor->name, "YSS225 FX Processor");
-		fx_processor->type = SND_HWDEP_TYPE_YSS225;
+		hw_dev++;
 
 		strcpy(card->abbreviation, "Tropez+");
 		card->type = SND_CARD_TYPE_TROPEZPLUS;
@@ -938,64 +880,20 @@ snd_wavefront_probe (int dev, snd_wavefront_card_t *acard)
 		return 0;
 	}
 
-	snd_printk (LOGNAME "can't register card\n");
-
-	snd_rawmidi_unregister (cs4232_rmidi);
-	snd_rawmidi_unregister (ics2115_internal_rmidi);
-	snd_rawmidi_unregister (ics2115_external_rmidi);
-	snd_hwdep_unregister(fx_processor);
-	snd_hwdep_unregister(wavefront_synth);
-	snd_hwdep_unregister(fm_synth);
-	snd_mixer_unregister(mixer);
-	snd_pcm_unregister(pcm);
-	snd_card_free(card);
-	return -ENOMEM;
-
       __nodev:
-	if (registered_bits & CS4232_MIDI_REGISTERED)
-		snd_rawmidi_unregister (cs4232_rmidi);
-	if (registered_bits & ICS2115_INTMIDI_REGISTERED)
-		snd_rawmidi_unregister (ics2115_internal_rmidi);
-	if (registered_bits & ICS2115_EXTMIDI_REGISTERED)
-		snd_rawmidi_unregister (ics2115_external_rmidi);
-	if (registered_bits & FX_REGISTERED)
-		snd_hwdep_unregister(fx_processor);
-	if (registered_bits & WAVEFRONT_REGISTERED)
-		snd_hwdep_unregister(wavefront_synth);
-	if (registered_bits & OPL3_REGISTERED)
-		snd_hwdep_unregister(fm_synth);
-	if (registered_bits & MIXER_REGISTERED)
-		snd_mixer_unregister(mixer);
-	if (registered_bits & PCM_REGISTERED)
-		snd_pcm_unregister(pcm);
-
-	if (fx_processor)
-		snd_hwdep_free (fx_processor);
-	if (wavefront_synth)
-		snd_hwdep_free (wavefront_synth);
-	if (fm_synth)
-		snd_hwdep_free (fm_synth);
-	if (cs4232_rmidi)
-		snd_rawmidi_free (cs4232_rmidi);
-	if (ics2115_internal_rmidi)
-		snd_rawmidi_free (ics2115_internal_rmidi);
-	if (ics2115_external_rmidi)
-		snd_rawmidi_free (ics2115_external_rmidi);
-	if (mixer)
-		snd_mixer_free (mixer);
-	if (pcm)
-		snd_pcm_free (pcm);
-
-#ifdef CONFIG_ISAPNP
+#ifdef __ISAPNP__
 	snd_wavefront_deactivate (acard);
-#endif CONFIG_ISAPNP
+#endif __ISAPNP__
 	snd_card_free(card);
 	return -ENXIO;
 }	
 
 
-
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_card_wavefront_init(void)
+#endif
 {
 	int dev, cards;
 	snd_wavefront_card_t *acard;
@@ -1007,7 +905,7 @@ int init_module(void)
 		acard = (snd_wavefront_card_t *)
 			snd_kcalloc (sizeof (snd_wavefront_card_t), GFP_KERNEL);
 
-		if (!acard)
+		if (acard == NULL)
 			continue;
 
 		init_waitqueue_head(&acard->wavefront.interrupt_sleeper);
@@ -1024,13 +922,17 @@ int init_module(void)
 		cards++;
 	}
 	if (!cards) {
+#ifdef MODULE
 		snd_printk (LOGNAME "No cards found or devices busy\n");
+#endif
 		return -ENODEV;
 	}
 	return 0;
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
 	int idx;
 	snd_wavefront_card_t *acard;
@@ -1039,25 +941,9 @@ void cleanup_module(void)
 		acard = wavefront_cards[idx];
 		if (acard) {
 			snd_card_unregister (acard->card);
-			if (acard->fx_processor)
-				snd_hwdep_unregister (acard->fx_processor);
-			if (acard->wavefront_synth)
-				snd_hwdep_unregister(acard->wavefront_synth);
-			if (acard->fm_synth)
-				snd_hwdep_unregister(acard->fm_synth);
-			if (acard->cs4232_rmidi)
-				snd_rawmidi_unregister(acard->cs4232_rmidi);
-			if (acard->ics2115_internal_rmidi)
-				snd_rawmidi_unregister(acard->ics2115_internal_rmidi);
-			if (acard->ics2115_external_rmidi)
-				snd_rawmidi_unregister(acard->ics2115_external_rmidi);
-			if (acard->mixer)
-				snd_mixer_unregister(acard->mixer);
-			if (acard->pcm) {
-				snd_pcm_unregister(acard->pcm);
-			}
-			snd_card_free(acard->card);
 			snd_kfree (acard);
 		}
 	}
 }
+
+#endif
