@@ -25,7 +25,7 @@
  *
  */
 
-#include "pcm1.h"
+#include "pcm.h"
 #include "mixer.h"
 
 struct snd_stru_es18xx {
@@ -44,22 +44,35 @@ struct snd_stru_es18xx {
 	unsigned short audio2_vol;	/* volume level of audio2 */
 
 	unsigned short active;		/* active channel mask */
+	unsigned int p_a_dma_size;
+	unsigned int c_a_dma_size;
+	unsigned int p_b_dma_size;
 
+	snd_kmixer_element_t *mix_imux;
 	snd_kmixer_element_t *mix_mic;
 	snd_kmixer_element_t *mix_line;
 	snd_kmixer_element_t *mix_cd;
 	snd_kmixer_element_t *mix_iaccu;
 	snd_kmixer_element_t *mix_oaccu;
-	snd_kmixer_element_t *mix_igain;
+	snd_kmixer_element_t *mix_igain_v;
+	snd_kmixer_element_t *mix_opcm1_v, *mix_ipcm1_v;
+	snd_kmixer_element_t *mix_opcm2_v;
+	snd_kmixer_element_t *mix_omic_v, *mix_imic_v;
+	snd_kmixer_element_t *mix_oline_v, *mix_iline_v;
+	snd_kmixer_element_t *mix_ofm_v, *mix_ifm_v;
+	snd_kmixer_element_t *mix_omono_v, *mix_imono_v;
+	snd_kmixer_element_t *mix_ocd_v, *mix_icd_v;
+	snd_kmixer_element_t *mix_oaux_v, *mix_iaux_v;
+	snd_kmixer_element_t *mix_output_v, *mix_output_s;
+	snd_kmixer_element_t *mix_playback1, *mix_playback2;
+	snd_kmixer_element_t *mix_capture;
 
 	snd_card_t *card;
 	snd_pcm_t *pcm;
 	snd_pcm_subchn_t *playback_a_subchn;
-	snd_pcm1_subchn_t *playback_a_subchn1;
 	snd_pcm_subchn_t *capture_a_subchn;
-	snd_pcm1_subchn_t *capture_a_subchn1;
 	snd_pcm_subchn_t *playback_b_subchn;
-	snd_pcm1_subchn_t *playback_b_subchn1;
+	snd_kmixer_t *mixer;
 
 	spinlock_t reg_lock;
 	spinlock_t mixer_lock;
@@ -93,16 +106,16 @@ extern unsigned char snd_es18xx_mixer_read(es18xx_t * codec, unsigned char reg);
 
 extern void snd_es18xx_interrupt(es18xx_t * codec, unsigned char status);
 
-extern es18xx_t *snd_es18xx_new_device(snd_card_t * card,
-				       unsigned short port,
-				       unsigned short mpu_port,
-				       unsigned short fm_port,
-				       snd_irq_t * irqnum,
-				       snd_dma_t * dma1num,
-				       snd_dma_t * dma2num);
+extern int snd_es18xx_new_device(snd_card_t * card,
+				 unsigned short port,
+				 unsigned short mpu_port,
+				 unsigned short fm_port,
+				 snd_irq_t * irqnum,
+				 snd_dma_t * dma1num,
+				 snd_dma_t * dma2num,
+				 es18xx_t ** codec);
 extern int snd_es18xx_init(es18xx_t * codec, int enable);
-extern snd_pcm_t *snd_es18xx_pcm(es18xx_t * codec);
-extern snd_kmixer_t *snd_es18xx_mixer(es18xx_t * codec);
+extern int snd_es18xx_pcm(es18xx_t * codec, int device, snd_pcm_t ** rpcm);
+extern int snd_es18xx_mixer(es18xx_t * codec, int device, snd_pcm_t * pcm, snd_kmixer_t ** rmixer);
 
 #endif				/* __ES18xx_H */
-
