@@ -21,7 +21,11 @@
 #ifndef __SEQ_OSS_H
 #define __SEQ_OSS_H
 
+#ifndef ALSA_BUILD
+#include <linux/asequencer.h>
+#else
 #include "asequencer.h"
+#endif
 
 /*
  * type definitions
@@ -95,22 +99,6 @@ typedef struct snd_seq_oss_reg {
 } snd_seq_oss_reg_t;
 
 /* device id */
-#define SND_SEQ_DEV_OSS		"seq-oss"
-
-/*
- * registration of synth port:
- * returns a unique index if succeeded.
- * Arguments name, type, subtype and nvoices are used for OSS synth_info.
- * Callbacks oper must be given.  Private data pointer is passed only for
- * open callback.
- */
-/*int snd_seq_oss_synth_register(char *name, int type, int subtype, int nvoices,
-			      snd_seq_oss_callback_t *oper, void *private_data);*/
-/*
- * unregistration of synth port:
- * give the registration index returned by seq_oss_synth_register().
- */
-/*int snd_seq_oss_synth_unregister(int index);*/
-
+#define SND_SEQ_DEV_ID_OSS		"seq-oss"
 
 #endif
