@@ -23,7 +23,7 @@
 
 #include "../../../include/driver.h"
 #include "../../../include/seq_oss.h"
-#include "../../../include/midi.h"
+#include "../../../include/rawmidi.h"
 #include "../../../include/seq_kernel.h"
 #include "../../../include/info.h"
 
@@ -38,8 +38,8 @@
 /* version */
 #define SND_SEQ_OSS_MAJOR_VERSION	0
 #define SND_SEQ_OSS_MINOR_VERSION	1
-#define SND_SEQ_OSS_TINY_VERSION	6
-#define SND_SEQ_OSS_VERSION_STR		"0.1.6"
+#define SND_SEQ_OSS_TINY_VERSION	8
+#define SND_SEQ_OSS_VERSION_STR		"0.1.8"
 
 /* device and proc interface name */
 #define SND_SEQ_OSS_DEVNAME	"seq_oss"
@@ -55,6 +55,7 @@ typedef struct seq_oss_writeq_t seq_oss_writeq_t;
 typedef struct seq_oss_readq_t seq_oss_readq_t;
 typedef struct seq_oss_timer_t seq_oss_timer_t;
 typedef struct seq_oss_synthinfo_t seq_oss_synthinfo_t;
+typedef struct seq_oss_synth_sysex_t seq_oss_synth_sysex_t;
 typedef struct seq_oss_chinfo_t seq_oss_chinfo_t;
 typedef unsigned int reltime_t;
 typedef unsigned int abstime_t;
@@ -74,8 +75,11 @@ struct seq_oss_chinfo_t {
 struct seq_oss_synthinfo_t {
 	snd_seq_oss_arg_t arg;
 	seq_oss_chinfo_t *ch;
+	seq_oss_synth_sysex_t *sysex;
 	int nr_voices;
 	int opened;
+	int is_midi;
+	int midi_mapped;
 };
 
 
@@ -122,9 +126,6 @@ struct seq_oss_devinfo_t {
 int snd_seq_oss_create_client(void);
 int snd_seq_oss_delete_client(void);
 
-void snd_seq_oss_start_timer(int ppq, unsigned int tempo);
-void snd_seq_oss_stop_timer(void);
-
 /* device file interface */
 int snd_seq_oss_open(struct file *file, int level);
 void snd_seq_oss_release(seq_oss_devinfo_t *dp);
@@ -153,9 +154,9 @@ void snd_seq_oss_readq_info_read(seq_oss_readq_t *q, snd_info_buffer_t *buf);
 
 /* dispatch event */
 inline static int
-snd_seq_oss_dispatch(seq_oss_devinfo_t *dp, snd_seq_event_t *ev, int atomic)
+snd_seq_oss_dispatch(seq_oss_devinfo_t *dp, snd_seq_event_t *ev, int atomic, int hop)
 {
-	return snd_seq_kernel_client_dispatch(dp->cseq, ev, atomic);
+	return snd_seq_kernel_client_dispatch(dp->cseq, ev, atomic, hop);
 }
 
 /* ioctl */
@@ -170,11 +171,10 @@ inline static void
 snd_seq_oss_fill_addr(seq_oss_devinfo_t *dp, snd_seq_event_t *ev,
 		     int dest_client, int dest_port)
 {
+	ev->queue = dp->queue;
 	ev->source = dp->addr;
 	ev->dest.client = dest_client;
 	ev->dest.port = dest_port;
-	ev->dest.queue = dp->queue;
-	/* ev->dest.channel is not written here */
 }
 
 
