@@ -224,8 +224,10 @@ void snd_gf1_stop_voices(snd_gus_card_t * gus, unsigned short v_min, unsigned sh
 			spin_unlock_irqrestore(&gus->reg_lock, flags);
 		}
 		time = HZ / 20;
-		while (time > 0)
+		while (time > 0 && !signal_pending(current)) {
+			current->state = TASK_INTERRUPTIBLE;
 			time = schedule_timeout(time);
+		}
 	}
 	snd_gf1_clear_voices(gus, v_min, v_max);
 }
@@ -321,9 +323,9 @@ int snd_gf1_start(snd_gus_card_t * gus)
 	unsigned int i;
 
 	snd_gf1_i_write8(gus, SND_GF1_GB_RESET, 0);	/* reset GF1 */
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_i_write8(gus, SND_GF1_GB_RESET, 1);	/* disable IRQ & DAC */
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_i_write8(gus, SND_GF1_GB_JOYSTICK_DAC_LEVEL, gus->joystick_dac);
 
 	snd_gf1_set_default_handlers(gus, SND_GF1_HANDLER_ALL);
@@ -358,9 +360,9 @@ int snd_gf1_start(snd_gus_card_t * gus)
 	snd_gf1_clear_regs(gus);
 	snd_gf1_clear_voices(gus, 0, 31);
 	snd_gf1_look_regs(gus);
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_i_write8(gus, SND_GF1_GB_RESET, 7);	/* Reset Register = IRQ enable, DAC enable */
-	snd_delay(16);
+	udelay(160);
 	snd_gf1_i_write8(gus, SND_GF1_GB_RESET, 7);	/* Reset Register = IRQ enable, DAC enable */
 	if (gus->gf1.enh_mode) {	/* enhanced mode !!!! */
 		snd_gf1_i_write8(gus, SND_GF1_GB_GLOBAL_MODE, snd_gf1_i_look8(gus, SND_GF1_GB_GLOBAL_MODE) | 0x01);
