@@ -25,10 +25,10 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "control.h"
-#include "info.h"
-#include "cs461x.h"
+#include "../../include/driver.h"
+#include "../../include/control.h"
+#include "../../include/info.h"
+#include "../../include/cs461x.h"
 
 /*
  *  constants
@@ -75,7 +75,7 @@ static void snd_cs461x_codec_write(void *private_data,
 	 *  4. Read ACCTL = 460h, DCV should be reset by now and 460h = 07h
 	 *  5. if DCV not cleared, break and return error
 	 */
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, );
 	int count;
 
 	/*
@@ -94,11 +94,11 @@ static void snd_cs461x_codec_write(void *private_data,
 	snd_cs461x_pokeBA0(codec, BA0_ACCDA, val);
 	snd_cs461x_pokeBA0(codec, BA0_ACCTL, ACCTL_DCV | ACCTL_VFRM |
 				             ACCTL_ESYN | ACCTL_RSTN);
-	for (count = 0; count < 100; count++) {
+	for (count = 0; count < 1000; count++) {
 		/*
 		 *  First, we want to wait for a short time.
 		 */
-		snd_delay(1);
+		udelay(10);
 		/*
 		 *  Now, check to see if the write has completed.
 		 *  ACCTL = 460h, DCV should be reset by now and 460h = 07h
@@ -126,7 +126,7 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 	 */
 
 	int count;
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 
 	snd_cs461x_peekBA0(codec, BA0_ACSDA);
 
@@ -157,7 +157,7 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 		/*
 		 *  First, we want to wait for a short time.
 	 	 */
-		snd_delay(1);
+		udelay(10);
 		/*
 		 *  Now, check to see if the read has completed.
 		 *  ACCTL = 460h, DCV should be reset by now and 460h = 17h
@@ -185,7 +185,7 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 		 */
 		if (snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_VSTS)
 			break;
-		snd_delay(1);
+		udelay(10);
 	}
 	
 	/*
@@ -204,9 +204,6 @@ static unsigned short snd_cs461x_codec_read(void *private_data,
 	printk("e) reg = 0x%x, val = 0x%x, BA0_ACCAD = 0x%x\n", reg,
 			snd_cs461x_peekBA0(codec, BA0_ACSDA),
 			snd_cs461x_peekBA0(codec, BA0_ACCAD));
-#endif
-#if 0
-	snd_delay(400);
 #endif
 	return snd_cs461x_peekBA0(codec, BA0_ACSDA);
 }
@@ -327,7 +324,7 @@ static void snd_cs461x_clear_serial_FIFOs(cs461x_t *codec)
 		 *  Make sure the previous FIFO write operation has completed.
 		 */
 		for (loop = 0; loop < 5; loop++) {
-			snd_delay(5);
+			udelay(50);
 			if (!(snd_cs461x_peekBA0(codec, BA0_SERBST) & SERBST_WBSY))
 				break;
 		}
@@ -373,7 +370,7 @@ static void snd_cs461x_powerup_dac(cs461x_t *codec)
 		 *  First, lets wait a short while to let things settle out a
 		 *  bit, and to prevent retrying the read too quickly.
 		 */
-		snd_delay(5);
+		udelay(50);
 
 		/*
 		 *  Read the current state of the power control register.
@@ -410,7 +407,7 @@ static void snd_cs461x_powerup_adc(cs461x_t *codec)
 		 *  First, lets wait a short while to let things settle out a
 		 *  bit, and to prevent retrying the read too quickly.
 		 */
-		snd_delay(5);
+		udelay(50);
 
 		/*
 		 *  Read the current state of the power control register.
@@ -444,7 +441,7 @@ static void snd_cs461x_proc_start(cs461x_t *codec)
 	 *  register.
 	 */
 	for (cnt = 0; cnt < 25; cnt++) {
-		snd_delay(5);
+		udelay(50);
 		if (!(snd_cs461x_peek(codec, BA1_SPCR) & SPCR_RUNFR))
 			break;
 	}
@@ -507,11 +504,11 @@ static void snd_cs461x_set_play_sample_rate(cs461x_t *codec, unsigned int rate)
 	/*
 	 *  Fill in the SampleRateConverter control block.
 	 */
-	snd_spin_lock(codec, reg, &flags);
+	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_cs461x_poke(codec, BA1_PSRC,
 	  ((correctionPerSec << 16) & 0xFFFF0000) | (correctionPerGOF & 0xFFFF));
 	snd_cs461x_poke(codec, BA1_PPI, phiIncr);
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
 static void snd_cs461x_set_capture_sample_rate(cs461x_t *codec, unsigned int rate)
@@ -581,14 +578,14 @@ static void snd_cs461x_set_capture_sample_rate(cs461x_t *codec, unsigned int rat
 	/*
 	 *  Fill in the VariDecimate control block.
 	 */
-	snd_spin_lock(codec, reg, &flags);
+	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_cs461x_poke(codec, BA1_CSRC,
 		((correctionPerSec << 16) & 0xFFFF0000) | (correctionPerGOF & 0xFFFF));
 	snd_cs461x_poke(codec, BA1_CCI, coeffIncr);
 	snd_cs461x_poke(codec, BA1_CD,
 		(((BA1_VARIDEC_BUF_1 + (initialDelay << 2)) << 16) & 0xFFFF0000) | 0x80);
 	snd_cs461x_poke(codec, BA1_CPI, phiIncr);
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 
 	/*
 	 *  Figure out the frame group length for the write back task.  Basically,
@@ -611,53 +608,50 @@ static void snd_cs461x_set_capture_sample_rate(cs461x_t *codec, unsigned int rat
 	/*
 	 * Fill in the WriteBack control block.
 	 */
-	snd_spin_lock(codec, reg, &flags);
+	spin_lock_irqsave(&codec->reg_lock, flags);
 	snd_cs461x_poke(codec, BA1_CFG1, frameGroupLength);
 	snd_cs461x_poke(codec, BA1_CFG2, (0x00800000 | frameGroupLength));
 	snd_cs461x_poke(codec, BA1_CCST, 0x0000FFFF);
 	snd_cs461x_poke(codec, BA1_CSPB, ((65536 * rate) / 24000));
 	snd_cs461x_poke(codec, (BA1_CSPB + 4), 0x0000FFFF);
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
 }
 
 /*
  *  PCM part
  */
 
-static int snd_cs461x_playback_ioctl(snd_pcm1_t * pcm1,
+static int snd_cs461x_playback_ioctl(void *private_data,
+				     snd_pcm_subchn_t * subchn,
 				     unsigned int cmd,
 				     unsigned long *arg)
 {
-	switch (cmd) {
-	case SND_PCM1_IOCTL_RATE:
-		pcm1->playback.real_rate = pcm1->playback.rate;
-		return 0;
-	case SND_PCM1_IOCTL_FRAG:
-		/* fixed size */
-		*(unsigned int *)arg = 2048;
-		return 0;
-	}
-	return -ENXIO;
+	int result;
+
+	result = snd_pcm_lib_ioctl(private_data, subchn, cmd, arg);
+	if (result < 0)
+		return result;		
+	return result;
 }
 
-static int snd_cs461x_record_ioctl(snd_pcm1_t * pcm1,
-				    unsigned int cmd, unsigned long *arg)
+static int snd_cs461x_capture_ioctl(void *private_data,
+				    snd_pcm_subchn_t * subchn,
+				    unsigned int cmd,
+				    unsigned long *arg)
 {
-	switch (cmd) {
-	case SND_PCM1_IOCTL_RATE:
-		pcm1->record.real_rate = pcm1->record.rate;
-		return 0;
-	case SND_PCM1_IOCTL_FRAG:
-		/* fixed size */
-		*(unsigned int *)arg = 2048;
-		return 0;
-	}
-	return -ENXIO;
+	int result;
+
+	result = snd_pcm_lib_ioctl(private_data, subchn, cmd, arg);
+	if (result < 0)
+		return result;		
+	return result;
 }
 
-static void snd_cs461x_copy_playback_buffer(snd_pcm1_t * pcm1, cs461x_t * codec)
+static void snd_cs461x_copy_playback_buffer(snd_pcm_subchn_t * subchn,
+					    cs461x_t * codec)
 {
-	if (pcm1->playback.mode & SND_PCM1_MODE_16) {
+	snd_pcm_runtime_t *runtime = subchn->runtime;
+	if (snd_pcm_format_width(runtime->format.format) == 16) {
 		memcpy(codec->pbuf + (2048 * codec->ppingbuf++),
 		       codec->pbuffer + (2048 * codec->pringbuf++),
 		       2048);
@@ -681,29 +675,37 @@ static void snd_cs461x_copy_playback_buffer(snd_pcm1_t * pcm1, cs461x_t * codec)
 	}
 }
 
-static void snd_cs461x_playback_trigger(snd_pcm1_t * pcm1, int up)
+static int snd_cs461x_playback_trigger(void *private_data,
+				       snd_pcm_subchn_t * subchn,
+				       int cmd)
 {
 	unsigned long flags;
-	cs461x_t *codec = (cs461x_t *)pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	unsigned int tmp;
+	int result = 0;
 
-	snd_spin_lock(codec, reg, &flags);
-	if (up) {
-		snd_cs461x_copy_playback_buffer(pcm1, codec);
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	if (cmd == SND_PCM_TRIGGER_GO) {
+		snd_cs461x_copy_playback_buffer(subchn, codec);
 		tmp = snd_cs461x_peek(codec, BA1_PCTL);
 		tmp &= 0x0000ffff;
 		snd_cs461x_poke(codec, BA1_PCTL, codec->pctl | tmp);
-	} else {
+	} else if (cmd == SND_PCM_TRIGGER_STOP) {
 		tmp = snd_cs461x_peek(codec, BA1_PCTL);
 		tmp &= 0x0000ffff;
 		snd_cs461x_poke(codec, BA1_PCTL, tmp);
+	} else {
+		result = -EINVAL;
 	}
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return result;
 }
 
-static void snd_cs461x_copy_record_buffer(snd_pcm1_t * pcm1, cs461x_t * codec)
+static void snd_cs461x_copy_capture_buffer(snd_pcm_subchn_t * subchn,
+					   cs461x_t * codec)
 {
 	signed int b16;
+	snd_pcm_runtime_t *runtime = subchn->runtime;
 
 	if (codec->cff == 1) {
 		memcpy(codec->cbuffer + (2048 * codec->cringbuf++),
@@ -715,11 +717,11 @@ static void snd_cs461x_copy_record_buffer(snd_pcm1_t * pcm1, cs461x_t * codec)
 	} else if (codec->cff == 2) {
 		unsigned char *dest;
 		unsigned char *src;
-		int count = 1024;
+		int count = 512;
 
 		dest = codec->cbuffer + (1024 * codec->cringbuf++);
 		src = codec->cbuf + (2048 * codec->cpingbuf++);
-		if (pcm1->record.mode & SND_PCM1_MODE_16) {
+		if (snd_pcm_format_width(runtime->format.format) == 16) {
 			while (count-- > 0) {
 				b16 = *((signed short *)src)++;
 				b16 += *((signed short *)src)++;
@@ -754,41 +756,45 @@ static void snd_cs461x_copy_record_buffer(snd_pcm1_t * pcm1, cs461x_t * codec)
 	}
 }
 
-static void snd_cs461x_record_trigger(snd_pcm1_t * pcm1, int up)
+static int snd_cs461x_capture_trigger(void *private_data,	
+				      snd_pcm_subchn_t * subchn,
+				      int cmd)
 {
 	unsigned long flags;
-	cs461x_t *codec = (cs461x_t *)pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	unsigned int tmp;
+	int result = 0;
 
-	snd_spin_lock(codec, reg, &flags);
-	if (up) {
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	if (cmd == SND_PCM_TRIGGER_GO) {
 		tmp = snd_cs461x_peek(codec, BA1_CCTL);
 		tmp &= 0xffff0000;
 		snd_cs461x_poke(codec, BA1_CCTL, codec->cctl | tmp);
-	} else {
+	} else if (cmd == SND_PCM_TRIGGER_STOP) {
 		tmp = snd_cs461x_peek(codec, BA1_CCTL);
 		tmp &= 0xffff0000;
 		snd_cs461x_poke(codec, BA1_CCTL, tmp);
+	} else {
+		result = -EINVAL;
 	}
-	snd_spin_unlock(codec, reg, &flags);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return result;
 }
 
-static void snd_cs461x_playback_prepare(snd_pcm1_t * pcm1,
-					unsigned char *buffer,
-					unsigned int size,
-					unsigned int offset,
-					unsigned int count)
+static int snd_cs461x_playback_prepare(void *private_data,
+				       snd_pcm_subchn_t * subchn)
 {
 	unsigned int tmp, tmp1;
-	cs461x_t *codec = (cs461x_t *)pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	snd_pcm_runtime_t *runtime = subchn->runtime;
 
-	if (count != 2048)
-		snd_printk("cs461x: wrong playback count - %i\n", count);
+	if (snd_pcm_lib_transfer_fragment(subchn) != 2048)
+		snd_printk("cs461x: wrong playback count - %i\n", snd_pcm_lib_transfer_fragment(subchn));
 	/* set the attenuation to 0dB */ 
 	snd_cs461x_poke(codec, BA1_PVOL, 0x80008000);
 	snd_cs461x_poke(codec, BA1_PBA, virt_to_bus(codec->pbuf));
 	tmp1 = 16;
-	if (pcm1->playback.voices < 2)
+	if (runtime->format.voices < 2)
 		tmp1 >>= 1;
 	tmp = snd_cs461x_peek(codec, BA1_PDTC);
 	tmp &= ~0x000003ff;
@@ -796,118 +802,72 @@ static void snd_cs461x_playback_prepare(snd_pcm1_t * pcm1,
 	snd_cs461x_poke(codec, BA1_PDTC, tmp);
 	tmp = snd_cs461x_peek(codec, BA1_PFIE);
 	tmp &= ~0x0000f03f;
-	if (pcm1->playback.mode & SND_PCM1_MODE_U)
+	if (snd_pcm_format_unsigned(runtime->format.format))
 		tmp |= 0x00008000;
-	if (pcm1->playback.mode & SND_PCM1_MODE_BIG)
+	if (snd_pcm_format_big_endian(runtime->format.format))
 		tmp |= 0x00004000;
-	if (pcm1->playback.voices < 2)
+	if (runtime->format.voices < 2)
 		tmp |= 0x00002000;
 	snd_cs461x_poke(codec, BA1_PFIE, tmp);
 	codec->ppingbuf = 0;
-	codec->pbuffer = buffer;
-	codec->psize = size;
+	codec->pbuffer = runtime->dma_area->buf;
+	codec->psize = snd_pcm_lib_transfer_size(subchn);
 	codec->pringbuf = 0;
-	snd_cs461x_set_play_sample_rate(codec, pcm1->playback.rate);
+	snd_cs461x_set_play_sample_rate(codec, runtime->format.rate);
+	return 0;
 }
 
-static void snd_cs461x_record_prepare(snd_pcm1_t * pcm1,
-				      unsigned char *buffer,
-				      unsigned int size,
-				      unsigned int offset,
-				      unsigned int count)
+static int snd_cs461x_capture_prepare(void *private_data,
+				      snd_pcm_subchn_t * subchn)
 {
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	snd_pcm_runtime_t *runtime = subchn->runtime;
 
-	if (count != 2048)
-		snd_printk("cs461x: wrong record count - %i\n", count);
+	if (snd_pcm_lib_transfer_fragment(subchn) != 2048)
+		snd_printk("cs461x: wrong record count - %i\n", snd_pcm_lib_transfer_fragment(subchn));
 	/* set the attenuation to 0dB */
 	snd_cs461x_poke(codec, BA1_CVOL, 0x80008000);
 	snd_cs461x_poke(codec, BA1_CBA, virt_to_bus(codec->cbuf));
 	codec->cpingbuf = 0;
-	codec->cbuffer = buffer;
-	codec->csize = size;
+	codec->cbuffer = runtime->dma_area->buf;
+	codec->csize = snd_pcm_lib_transfer_size(subchn);
 	codec->cringbuf = 0;
-	snd_cs461x_set_capture_sample_rate(codec, pcm1->record.rate);
+	snd_cs461x_set_capture_sample_rate(codec, runtime->format.rate);
 	codec->cff = 1;
 	codec->cffc = 0;
-	if (pcm1->record.voices < 2)
+	if (runtime->format.voices < 2)
 		codec->cff <<= 1;
-	if (!(pcm1->record.mode & SND_PCM1_MODE_16))
+	if (!snd_pcm_format_width(runtime->format.format) == 16)
 		codec->cff <<= 1;
-}
-
-static int snd_cs461x_playback_open(snd_pcm1_t * pcm1)
-{
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
-	int err, pg;
-
-	if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_PLAYBACK, codec->dma1ptr, "CS461x - DAC")) < 0)
-		return err;
-	codec->pbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
-	if (codec->pbuf == NULL) {
-		snd_pcm1_dma_free(pcm1, SND_PCM1_PLAYBACK, codec->dma1ptr);
-		return -ENOMEM;
-	}
 	return 0;
 }
 
-static int snd_cs461x_record_open(snd_pcm1_t * pcm1)
-{
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
-	int err, pg;
-
-	if ((err = snd_pcm1_dma_alloc(pcm1, SND_PCM1_RECORD, codec->dma2ptr, "CS461x - ADC")) < 0)
-		return err;
-	codec->cbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
-	if (codec->cbuf == NULL) {
-		snd_pcm1_dma_free(pcm1, SND_PCM1_RECORD, codec->dma2ptr);
-		return -ENOMEM;
-	}
-	return 0;
-}
-
-static void snd_cs461x_playback_close(snd_pcm1_t * pcm1)
-{
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
-
-	if (codec->pbuf)
-		snd_free_pages(codec->pbuf, PAGE_SIZE);
-	snd_pcm1_dma_free(pcm1, SND_PCM1_PLAYBACK, codec->dma1ptr);
-}
-
-static void snd_cs461x_record_close(snd_pcm1_t * pcm1)
-{
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
-
-	if (codec->cbuf)
-		snd_free_pages(codec->cbuf, PAGE_SIZE);
-	snd_pcm1_dma_free(pcm1, SND_PCM1_RECORD, codec->dma2ptr);
-}
-
-static unsigned int snd_cs461x_playback_pointer(snd_pcm1_t * pcm1,
-						unsigned int used_size)
+static unsigned int snd_cs461x_playback_pointer(void *private_data,
+						snd_pcm_subchn_t * subchn)
 {
 	unsigned int result;
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	snd_pcm_runtime_t *runtime = subchn->runtime;
 
 	result = snd_cs461x_peek(codec, BA1_PBA) - virt_to_bus(codec->pbuf);
 	if (result > 2048)
 		result -= 2048;
-	if (!(pcm1->playback.mode & SND_PCM1_MODE_16))
+	if (!snd_pcm_format_width(runtime->format.format) == 16)
 		result >>= 1;
 	return result;
 }
 
-static unsigned int snd_cs461x_record_pointer(snd_pcm1_t * pcm1,
-					      unsigned int used_size)
+static unsigned int snd_cs461x_capture_pointer(void *private_data,
+					       snd_pcm_subchn_t * subchn)
 {
 	unsigned int result;
-	cs461x_t *codec = (cs461x_t *) pcm1->private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	snd_pcm_runtime_t *runtime = subchn->runtime;
 
 	result = snd_cs461x_peek(codec, BA1_CBA) - virt_to_bus(codec->cbuf);
 	if (result > 2048)
 		result -= 2048;
-	if (!(pcm1->record.mode & SND_PCM1_MODE_16))
+	if (!snd_pcm_format_width(runtime->format.format) == 16)
 		result >>= 1;
 	return result;
 }
@@ -926,132 +886,375 @@ void snd_cs461x_interrupt(cs461x_t *codec)
 	}
 
 	if ((status & HISR_VC0) && codec->pcm) {
-		snd_pcm1_t *pcm1 = (snd_pcm1_t *)codec->pcm->private_data;
-		if (pcm1 && (pcm1->playback.flags & SND_PCM1_FLG_TRIGGER)) {
-			snd_cs461x_copy_playback_buffer(pcm1, codec);
-			if (((pcm1->playback.mode & SND_PCM1_MODE_16) ||
-			     codec->ppingbuf == 1) &&
-			    pcm1->playback.ack)
-				pcm1->playback.ack(pcm1);
+		if (codec->playback_subchn) {
+			snd_cs461x_copy_playback_buffer(codec->playback_subchn, codec);
+			if (snd_pcm_format_width(codec->playback_subchn->runtime->format.format) == 16 ||
+			    codec->ppingbuf == 1)
+				snd_pcm_transfer_done(codec->playback_subchn);
 		}
 	}
 	if ((status & HISR_VC1) && codec->pcm) {
-		snd_pcm1_t *pcm1 = (snd_pcm1_t *)codec->pcm->private_data;
-		if (pcm1 && (pcm1->record.flags & SND_PCM1_FLG_TRIGGER)) {
-			snd_cs461x_copy_record_buffer(pcm1, codec);
+		if (codec->capture_subchn) {
+			snd_cs461x_copy_capture_buffer(codec->capture_subchn, codec);
 			codec->cffc++;
 			if (codec->cffc >= codec->cff) {
 				codec->cffc = 0;
-				if (pcm1->record.ack)
-					pcm1->record.ack(pcm1);
+				snd_pcm_transfer_done(codec->capture_subchn);
 			}
 		}
 	}
+	if ((status & HISR_MIDI) && codec->rmidi) {
+		unsigned char c;
+		while ((snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_RBE) == 0) {
+			c = snd_cs461x_peekBA0(codec, BA0_MIDRP);
+			if ((codec->midcr & MIDCR_RIE) == 0)
+				continue;
+			snd_rawmidi_receive(codec->rmidi, &c, 1);
+		}
+		while ((snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_TBF) == 0) {
+			if ((codec->midcr & MIDCR_TIE) == 0)
+				break;
+			if (snd_rawmidi_transmit(codec->rmidi, &c, 1) != 1)
+				break;
+			snd_cs461x_pokeBA0(codec, BA0_MIDWP, c);
+		}
+	}
 	/*
 	 *  EOI to the PCI part....reenables interrupts
 	 */
 	snd_cs461x_pokeBA0(codec, BA0_HICR, HICR_CHGM | HICR_IEV);
 }
 
-static struct snd_stru_pcm1_hardware snd_cs461x_playback =
+static snd_pcm_hardware_t snd_cs461x_playback =
 {
-	NULL,			/* private data */
-	NULL,			/* private_free */
-	SND_PCM1_HW_AUTODMA | SND_PCM1_HW_BLOCKPTR,	/* flags */
-	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* formats */
+	SND_PCM_CHNINFO_MMAP |
+	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
+	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	3,			/* align value */
-	6,			/* minimal fragment */
+	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
+	1,			/* min. voices */
 	2,			/* max. voices */
-	snd_cs461x_playback_open,
-	snd_cs461x_playback_close,
+	2048,			/* min. fragment size */
+	2048,			/* max. fragment size */
+	3,			/* fragment align */
+	0,			/* FIFO size (unknown) */
+	4,			/* transfer block size */
 	snd_cs461x_playback_ioctl,
 	snd_cs461x_playback_prepare,
 	snd_cs461x_playback_trigger,
-	snd_cs461x_playback_pointer,
-	snd_pcm1_playback_dma_ulaw_loud,
-	snd_pcm1_dma_move,
-	snd_pcm1_playback_dma_neutral
+	snd_pcm_playback_write,
+	snd_cs461x_playback_pointer
 };
 
-static struct snd_stru_pcm1_hardware snd_cs461x_record =
+static snd_pcm_hardware_t snd_cs461x_capture =
 {
-	NULL,			/* private data */
-	NULL,			/* private_free */
-	SND_PCM1_HW_AUTODMA | SND_PCM1_HW_BLOCKPTR,	/* flags */
-	SND_PCM_FMT_MU_LAW | SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* formats */
+	SND_PCM_CHNINFO_MMAP |
+	SND_PCM_CHNINFO_BLOCK | SND_PCM_CHNINFO_INTERLEAVE |
+	SND_PCM_CHNINFO_BLOCK_TRANSFER,		/* flags */
 	SND_PCM_FMT_U8 | SND_PCM_FMT_S16_LE,	/* hardware formats */
-	3,			/* align value */
-	6,			/* minimal fragment */
+	SND_PCM_RATE_PLL | SND_PCM_RATE_8000_48000,
 	5500,			/* min. rate */
 	48000,			/* max. rate */
+	1,			/* min. voices */
 	2,			/* max. voices */
-	snd_cs461x_record_open,
-	snd_cs461x_record_close,
-	snd_cs461x_record_ioctl,
-	snd_cs461x_record_prepare,
-	snd_cs461x_record_trigger,
-	snd_cs461x_record_pointer,
-	snd_pcm1_record_dma_ulaw,
-	snd_pcm1_dma_move,
-	NULL
+	2048,			/* min. fragment size */
+	2048,			/* max. fragment size */
+	3,			/* fragment align */
+	0,			/* FIFO size (unknown) */
+	4,			/* transfer block size */
+	snd_cs461x_capture_ioctl,
+	snd_cs461x_capture_prepare,
+	snd_cs461x_capture_trigger,
+	snd_pcm_capture_read,
+	snd_cs461x_capture_pointer
 };
 
+static int snd_cs461x_playback_open(void *private_data,
+				    snd_pcm_subchn_t * subchn)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	int err, pg;
+
+	if ((err = snd_pcm_dma_alloc(subchn, codec->dma1ptr, "CS461x - DAC")) < 0)
+		return err;
+	codec->pbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
+	if (codec->pbuf == NULL) {
+		snd_pcm_dma_free(subchn);
+		return -ENOMEM;
+	}
+	codec->playback_subchn = subchn;
+	subchn->runtime->hw = &snd_cs461x_playback;
+	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_playback);
+	return 0;
+}
+
+static int snd_cs461x_capture_open(void *private_data,
+				   snd_pcm_subchn_t * subchn)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+	int err, pg;
+
+	if ((err = snd_pcm_dma_alloc(subchn, codec->dma2ptr, "CS461x - ADC")) < 0)
+		return err;
+	codec->cbuf = snd_malloc_pages(PAGE_SIZE, &pg, 0);
+	if (codec->cbuf == NULL) {
+		snd_pcm_dma_free(subchn);
+		return -ENOMEM;
+	}
+	codec->capture_subchn = subchn;
+	subchn->runtime->hw = &snd_cs461x_capture;
+	snd_pcm_set_mixer(subchn, codec->mixer->device, codec->ac97->me_capture);
+	return 0;
+}
+
+static int snd_cs461x_playback_close(void *private_data,
+				     snd_pcm_subchn_t * subchn)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+
+	codec->playback_subchn = NULL;
+	if (codec->pbuf)
+		snd_free_pages(codec->pbuf, PAGE_SIZE);
+	snd_pcm_dma_free(subchn);
+	return 0;
+}
+
+static int snd_cs461x_capture_close(void *private_data,
+				    snd_pcm_subchn_t * subchn)
+{
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
+
+	codec->capture_subchn = NULL;
+	if (codec->cbuf)
+		snd_free_pages(codec->cbuf, PAGE_SIZE);
+	snd_pcm_dma_free(subchn);
+	return 0;
+}
+
 static void snd_cs461x_pcm_free(void *private_data)
 {
-	cs461x_t *codec = (cs461x_t *) private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, );
 	codec->pcm = NULL;
 }
 
-snd_pcm_t *snd_cs461x_pcm(cs461x_t * codec)
+int snd_cs461x_pcm(cs461x_t * codec, int device, snd_pcm_t ** rpcm)
 {
 	snd_pcm_t *pcm;
-	snd_pcm1_t *pcm1;
+	int err;
 
-	pcm = snd_pcm1_new_device(codec->card, "CS461x");
-	if (!pcm)
-		return NULL;
-	pcm1 = (snd_pcm1_t *) pcm->private_data;
-	memcpy(&pcm1->playback.hw, &snd_cs461x_playback, sizeof(snd_cs461x_playback));
-	memcpy(&pcm1->record.hw, &snd_cs461x_record, sizeof(snd_cs461x_record));
-	pcm1->private_data = codec;
-	pcm1->private_free = snd_cs461x_pcm_free;
-	pcm->info_flags = SND_PCM_INFO_CODEC | SND_PCM_INFO_MMAP |
-	    SND_PCM_INFO_PLAYBACK | SND_PCM_INFO_RECORD | SND_PCM_INFO_DUPLEX;
+	*rpcm = NULL;
+	if ((err = snd_pcm_new(codec->card, "CS461x", device, 1, 1, &pcm)) < 0)
+		return err;
+	pcm->private_data = codec;
+	pcm->private_free = snd_cs461x_pcm_free;
+
+	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].private_data = codec;
+	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].open = snd_cs461x_playback_open;
+	pcm->chn[SND_PCM_CHANNEL_PLAYBACK].close = snd_cs461x_playback_close;
+	pcm->chn[SND_PCM_CHANNEL_CAPTURE].private_data = codec;
+	pcm->chn[SND_PCM_CHANNEL_CAPTURE].open = snd_cs461x_capture_open;
+	pcm->chn[SND_PCM_CHANNEL_CAPTURE].close = snd_cs461x_capture_close;
+
+	/* global setup */
+	pcm->info_flags = SND_PCM_INFO_PLAYBACK | SND_PCM_INFO_CAPTURE |
+			  SND_PCM_INFO_DUPLEX;
 	strcpy(pcm->name, "CS461x");
-	return codec->pcm = pcm;
+	*rpcm = codec->pcm = pcm;
+	return 0;
 }
 
 /*
  *  Mixer routines
  */
 
-static void snd_cs461x_mixer_free_ac97(ac97_t * ac97)
+static void snd_cs461x_mixer_free_ac97(void *private_data)
 {
-	((cs461x_t *) ac97->private_data)->mixer = NULL;
-	snd_free(ac97, sizeof(ac97_t));
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, );
+	codec->mixer = NULL;
 }
 
-snd_kmixer_t *snd_cs461x_mixer(cs461x_t * codec, int pcm_dev)
+int snd_cs461x_mixer(cs461x_t * codec, int device, snd_pcm_t * pcm, snd_kmixer_t ** rmixer)
 {
-	ac97_t *ac97;
+	ac97_t ac97;
 	snd_kmixer_t *mixer;
+	int err;
 
-	ac97 = snd_calloc(sizeof(ac97_t));
-	if (!ac97)
-		return NULL;
-	ac97->write = snd_cs461x_codec_write;
-	ac97->read = snd_cs461x_codec_read;
-	ac97->private_data = codec;
-	ac97->private_free = snd_cs461x_mixer_free_ac97;
-	mixer = snd_ac97_mixer(codec->card, ac97, 1, &pcm_dev);
-	if (!mixer) {
-		snd_free(ac97, sizeof(ac97));
-		return NULL;
+	*rmixer = NULL;
+	memset(&ac97, 0, sizeof(ac97));
+	ac97.write = snd_cs461x_codec_write;
+	ac97.read = snd_cs461x_codec_read;
+	ac97.private_data = codec;
+	ac97.private_free = snd_cs461x_mixer_free_ac97;
+	if ((err = snd_ac97_mixer(codec->card, device, &ac97, 1, &pcm->device, &mixer)) < 0)
+		return err;
+	codec->ac97 = snd_magic_cast(ac97_t, mixer->private_data, -ENXIO);
+	*rmixer = codec->mixer = mixer;
+	return 0;
+}
+
+/*
+ *  RawMIDI interface
+ */
+
+static void snd_cs461x_midi_reset(cs461x_t *codec)
+{
+	snd_cs461x_pokeBA0(codec, BA0_MIDCR, MIDCR_MRST);
+	udelay(100);
+	snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+}
+
+static int snd_cs461x_midi_input_open(snd_rawmidi_t * rmidi)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	codec->uartm |= CS461X_MODE_INPUT;
+	codec->midcr |= MIDCR_RXE;
+	if (!(codec->uartm & CS461X_MODE_OUTPUT)) {
+		snd_cs461x_midi_reset(codec);
+	} else {
+		snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
 	}
-	return codec->mixer = mixer;
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return 0;
+}
+
+static int snd_cs461x_midi_input_close(snd_rawmidi_t * rmidi)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	codec->midcr &= ~(MIDCR_RXE | MIDCR_RIE);
+	if (!(codec->uartm & CS461X_MODE_OUTPUT)) {
+		snd_cs461x_midi_reset(codec);
+	} else {
+		snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	}
+	codec->uartm &= ~CS461X_MODE_INPUT;
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return 0;
+}
+
+static int snd_cs461x_midi_output_open(snd_rawmidi_t * rmidi)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	codec->uartm |= CS461X_MODE_OUTPUT;
+	codec->midcr |= MIDCR_TXE;
+	if (!(codec->uartm & CS461X_MODE_INPUT)) {
+		snd_cs461x_midi_reset(codec);
+	} else {
+		snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	}
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return 0;
+}
+
+static int snd_cs461x_midi_output_close(snd_rawmidi_t * rmidi)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, -ENXIO);
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	codec->midcr &= ~(MIDCR_TXE | MIDCR_TIE);
+	if (!(codec->uartm & CS461X_MODE_INPUT)) {
+		snd_cs461x_midi_reset(codec);
+	} else {
+		snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	}
+	codec->uartm &= ~CS461X_MODE_OUTPUT;
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+	return 0;
+}
+
+static void snd_cs461x_midi_input_trigger(snd_rawmidi_t * rmidi, int up)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, );
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	if (up)
+		codec->midcr |= MIDCR_RIE;
+	else
+		codec->midcr &= ~MIDCR_RIE;
+	snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+}
+
+static void snd_cs461x_midi_output_trigger(snd_rawmidi_t * rmidi, int up)
+{
+	unsigned long flags;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, rmidi->private_data, );
+
+	spin_lock_irqsave(&codec->reg_lock, flags);
+	if (up) {
+		codec->midcr |= MIDCR_TIE;
+		/* fill UART FIFO buffer at first, and turn Tx interrupts only if necessary */
+		while ((codec->midcr & MIDCR_TIE) &&
+		       (snd_cs461x_peekBA0(codec, BA0_MIDSR) & MIDSR_TBF) == 0) {
+			unsigned char byte;
+			if (snd_rawmidi_transmit(rmidi, &byte, 1) != 1) {
+				codec->midcr &= ~MIDCR_TIE;
+			} else {
+				snd_cs461x_pokeBA0(codec, BA0_MIDWP, byte);
+			}
+		}
+	} else {
+		codec->midcr &= ~MIDCR_TIE;
+	}
+	snd_cs461x_pokeBA0(codec, BA0_MIDCR, codec->midcr);
+	spin_unlock_irqrestore(&codec->reg_lock, flags);
+}
+
+static struct snd_stru_rawmidi_channel_hw snd_cs461x_midi_output =
+{
+	0,			/* flags */
+	NULL,			/* private_data */
+	NULL,			/* private_free */
+	snd_cs461x_midi_output_open,	/* open */
+	snd_cs461x_midi_output_close,	/* close */
+	snd_cs461x_midi_output_trigger,/* trigger */
+	{NULL},			/* io.write */
+	NULL,			/* abort */
+};
+
+static struct snd_stru_rawmidi_channel_hw snd_cs461x_midi_input =
+{
+	0,			/* flags */
+	NULL,			/* private_data */
+	NULL,			/* private_free */
+	snd_cs461x_midi_input_open,	/* open */
+	snd_cs461x_midi_input_close,	/* close */
+	snd_cs461x_midi_input_trigger,	/* trigger */
+	{NULL},			/* io.read */
+	NULL,			/* abort */
+};
+
+int snd_cs461x_midi(cs461x_t * codec, int device, snd_rawmidi_t **rrawmidi)
+{
+	snd_rawmidi_t *rmidi;
+	int err;
+
+	*rrawmidi = NULL;
+	if ((err = snd_rawmidi_new(codec->card, "ES1370/1", device, &rmidi)) < 0)
+		return err;
+#ifdef CHIP1370
+	strcpy(rmidi->name, "ES1370");
+#else
+	strcpy(rmidi->name, "ES1371");
+#endif
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_OUTPUT].hw, &snd_cs461x_midi_output, sizeof(snd_cs461x_midi_output));
+	memcpy(&rmidi->chn[SND_RAWMIDI_CHANNEL_INPUT].hw, &snd_cs461x_midi_input, sizeof(snd_cs461x_midi_input));
+	rmidi->info_flags |= SND_RAWMIDI_INFO_OUTPUT | SND_RAWMIDI_INFO_INPUT | SND_RAWMIDI_INFO_DUPLEX;
+	rmidi->private_data = codec;
+	*rrawmidi = codec->rmidi = rmidi;
+	return 0;
 }
 
 /*
@@ -1062,15 +1265,14 @@ static long snd_cs461x_BA0_read(void *private_data, void *file_private_data,
 				struct file *file, char *buf, long count)
 {
 	long size;
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	
 	size = count;
 	if (file->f_pos + size > CS461X_BA0_SIZE)
 		size = (long)CS461X_BA0_SIZE - file->f_pos;
 	if (size > 0) {
-		if (verify_area(VERIFY_WRITE, buf, size))
+		if (copy_to_user(buf, codec->ba0 + file->f_pos, size))
 			return -EFAULT;
-		copy_to_user(buf, codec->ba0 + file->f_pos, size);
 		file->f_pos += size;
 		return size;
 	}
@@ -1081,15 +1283,14 @@ static long snd_cs461x_BA1_data0_read(void *private_data, void *file_private_dat
 				      struct file *file, char *buf, long count)
 {
 	long size;
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	
 	size = count;
 	if (file->f_pos + size > CS461X_BA1_DATA0_SIZE)
 		size = (long)CS461X_BA1_DATA0_SIZE - file->f_pos;
 	if (size > 0) {
-		if (verify_area(VERIFY_WRITE, buf, size))
+		if (copy_to_user(buf, codec->ba1.name.data0 + file->f_pos, size))
 			return -EFAULT;
-		copy_to_user(buf, codec->ba1.name.data0 + file->f_pos, size);
 		file->f_pos += size;
 		return size;
 	}
@@ -1100,15 +1301,14 @@ static long snd_cs461x_BA1_data1_read(void *private_data, void *file_private_dat
 				      struct file *file, char *buf, long count)
 {
 	long size;
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	
 	size = count;
 	if (file->f_pos + size > CS461X_BA1_DATA1_SIZE)
 		size = (long)CS461X_BA1_DATA1_SIZE - file->f_pos;
 	if (size > 0) {
-		if (verify_area(VERIFY_WRITE, buf, size))
+		if (copy_to_user(buf, codec->ba1.name.data1 + file->f_pos, size))
 			return -EFAULT;
-		copy_to_user(buf, codec->ba1.name.data1 + file->f_pos, size);
 		file->f_pos += size;
 		return size;
 	}
@@ -1119,15 +1319,14 @@ static long snd_cs461x_BA1_prg_read(void *private_data, void *file_private_data,
 				    struct file *file, char *buf, long count)
 {
 	long size;
-	cs461x_t *codec = (cs461x_t *)private_data;
+	cs461x_t *codec = snd_magic_cast(cs461x_t, private_data, -ENXIO);
 	
 	size = count;
 	if (file->f_pos + size > CS461X_BA1_PRG_SIZE)
 		size = (long)CS461X_BA1_PRG_SIZE - file->f_pos;
 	if (size > 0) {
-		if (verify_area(VERIFY_WRITE, buf, size))
+		if (copy_to_user(buf, codec->ba1.name.pmem + file->f_pos, size))
 			return -EFAULT;
-		copy_to_user(buf, codec->ba1.name.pmem + file->f_pos, size);
 		file->f_pos += size;
 		return size;
 	}
@@ -1206,35 +1405,46 @@ static int snd_cs461x_proc_done(cs461x_t * codec)
  *  initialization routines
  */
 
-cs461x_t *snd_cs461x_create(snd_card_t * card,
-			    struct snd_pci_dev * pci,
-			    snd_dma_t * dma1ptr,
-			    snd_dma_t * dma2ptr,
-			    snd_irq_t * irqptr)
+int snd_cs461x_create(snd_card_t * card,
+		      struct pci_dev * pci,
+		      snd_dma_t * dma1ptr,
+		      snd_dma_t * dma2ptr,
+		      snd_irq_t * irqptr,
+		      cs461x_t ** rcodec)
 {
 	cs461x_t *codec;
 	unsigned short cmdw;
 	unsigned char cmdb;
 	unsigned int tmp;
-	int idx;
+	int idx, err;
+	static snd_device_ops_t ops = {
+		(snd_dev_free_t *)snd_cs461x_free,
+		NULL,
+		NULL
+	};
 	
-	codec = (cs461x_t *) snd_calloc(sizeof(*codec));
-	if (!codec)
-		return NULL;
-	snd_spin_prepare(codec, reg);
-	snd_sleep_prepare(codec, codec);
+	*rcodec = NULL;
+	codec = snd_magic_kcalloc(cs461x_t, 0, GFP_KERNEL);
+	if (codec == NULL)
+		return -ENOMEM;
+	codec->reg_lock = SPIN_LOCK_UNLOCKED;
 	codec->card = card;
 	codec->pci = pci;
 	codec->dma1ptr = dma1ptr;
 	codec->dma2ptr = dma2ptr;
 	codec->irqptr = irqptr;
+#ifdef NEW_PCI
+	codec->ba0_addr = pci->resource[0].start;
+	codec->ba1_addr = pci->resource[1].start;
+#else
 	codec->ba0_addr = pci->base_address[0] & ~PCI_BASE_ADDRESS_SPACE;
 	codec->ba1_addr = pci->base_address[1] & ~PCI_BASE_ADDRESS_SPACE;
+#endif
 	if (codec->ba0_addr == 0 || codec->ba0_addr == ~0 ||
 	    codec->ba1_addr == 0 || codec->ba1_addr == ~0) {
 	    	snd_printk("cs461x: wrong address(es) - ba0 = 0x%lx, ba1 = 0x%lx\n", codec->ba0_addr, codec->ba1_addr);
 	    	snd_cs461x_free(codec);
-	    	return NULL;
+	    	return -ENOMEM;
 	}
 	codec->ba0 = ioremap(codec->ba0_addr, CS461X_BA0_SIZE);
 	codec->ba1.name.data0 = ioremap(codec->ba1_addr + BA1_SP_DMEM0, CS461X_BA1_DATA0_SIZE);
@@ -1245,17 +1455,19 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	    codec->ba1.name.data1 == NULL || codec->ba1.name.pmem == NULL ||
 	    codec->ba1.name.reg == NULL) {
 		snd_cs461x_free(codec);
-		return NULL;
+		return -ENOMEM;
 	}
-	snd_pci_read_config_word(pci, PCI_COMMAND, &cmdw);
-	if ((cmdw & (PCI_COMMAND_MEMORY|PCI_COMMAND_MASTER)) != (PCI_COMMAND_MEMORY|PCI_COMMAND_MASTER)) {
-		cmdw |= PCI_COMMAND_MEMORY|PCI_COMMAND_MASTER;
-		snd_pci_write_config_word(pci, PCI_COMMAND, cmdw);
+	pci_set_master(pci);
+	pci_read_config_word(pci, PCI_COMMAND, &cmdw);
+	if ((cmdw & PCI_COMMAND_MEMORY) != PCI_COMMAND_MEMORY) {
+		cmdw |= PCI_COMMAND_MEMORY;
+		pci_write_config_word(pci, PCI_COMMAND, cmdw);
 	}
-	snd_pci_read_config_byte(pci, PCI_LATENCY_TIMER, &cmdb);
+	pci_set_master(pci);
+	pci_read_config_byte(pci, PCI_LATENCY_TIMER, &cmdb);
 	if (cmdb < 32)
 		cmdb = 32;
-	snd_pci_write_config_byte(pci, PCI_LATENCY_TIMER, cmdb);
+	pci_write_config_byte(pci, PCI_LATENCY_TIMER, cmdb);
 
 	/* 
 	 *  First, blast the clock control register to zero so that the PLL starts
@@ -1279,7 +1491,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
          *  for a reset.
          */
         snd_cs461x_pokeBA0(codec, BA0_ACCTL, 0);
-        snd_delay(5);
+        udelay(50);
         snd_cs461x_pokeBA0(codec, BA0_ACCTL, ACCTL_RSTN);
 
 	/*
@@ -1294,7 +1506,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	 *  generating bit clock (so we don't try to start the PLL without an
 	 *  input clock).
 	 */
-	snd_delay(1000);
+	mdelay(1);
 
 	/*
 	 *  Set the serial port timing configuration, so that
@@ -1319,7 +1531,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	/*
          *  Wait until the PLL has stabilized.
 	 */
-	snd_delay(1000);
+	mdelay(1);
 
 	/*
 	 *  Turn on clocking of the core so that we can setup the serial ports.
@@ -1353,7 +1565,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 		 *  First, lets wait a short while to let things settle out a bit,
 		 *  and to prevent retrying the read too quickly.
 		 */
-		snd_delay(5);
+		udelay(50);
 
 		/*
 		 *  Read the AC97 status register to see if we've seen a CODEC READY
@@ -1369,7 +1581,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	if (!(snd_cs461x_peekBA0(codec, BA0_ACSTS) & ACSTS_CRDY)) {
 		snd_printk("cs461x: create - never read codec ready from AC'97\n");
 		snd_cs461x_free(codec);
-		return NULL;
+		return -EIO;
 	}
 
 	/*
@@ -1387,7 +1599,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 		 *  First, lets wait a short while to let things settle out a bit,
 		 *  and to prevent retrying the read too quickly.
 		 */
-		snd_delay(50);
+		udelay(500);
 		/*
 		 *  Read the input slot valid register and see if input slots 3 and
 		 *  4 are valid yet.
@@ -1403,7 +1615,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	if ((snd_cs461x_peekBA0(codec, BA0_ACISV) & (ACISV_ISV3 | ACISV_ISV4)) != (ACISV_ISV3 | ACISV_ISV4)) {
 		snd_printk("cs461x: create - never read ISV3 & ISV4 from AC'97\n");
 		snd_cs461x_free(codec);
-		return NULL;
+		return -EIO;
 	}
 
 	/*
@@ -1436,7 +1648,7 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 	if (snd_cs461x_download_image(codec) < 0) {
 		snd_printk("cs461x: image download error\n");
 		snd_cs461x_free(codec);
-		return NULL;
+		return -EIO;
 	}
 
 	/*
@@ -1477,13 +1689,21 @@ cs461x_t *snd_cs461x_create(snd_card_t * card,
 
 	snd_cs461x_proc_init(card, codec);
 
-	return codec;
+	if ((err = snd_device_new(card, SND_DEV_LOWLEVEL, codec, 0, &ops, NULL)) < 0) {
+		snd_cs461x_free(codec);
+		return err;
+	}
+
+	*rcodec = codec;
+	return 0;
 }
 
-void snd_cs461x_free(cs461x_t * codec)
+int snd_cs461x_free(cs461x_t * codec)
 {
 	unsigned int tmp;
 
+	snd_debug_check(codec == NULL, -EINVAL);
+
 	snd_cs461x_proc_done(codec);
 
 	tmp = snd_cs461x_peek(codec, BA1_PFIE);
@@ -1533,8 +1753,6 @@ void snd_cs461x_free(cs461x_t * codec)
 	tmp = snd_cs461x_peekBA0(codec, BA0_CLKCR1) & ~CLKCR1_SWCE;
 	snd_cs461x_pokeBA0(codec, BA0_CLKCR1, tmp);
 
-	if (codec == NULL)
-		return;
 	if (codec->ba0)
 		iounmap(codec->ba0);
 	if (codec->ba1.name.data0)
@@ -1546,26 +1764,29 @@ void snd_cs461x_free(cs461x_t * codec)
 	if (codec->ba1.name.reg)
 		iounmap(codec->ba1.name.reg);
 
-	snd_free(codec, sizeof(*codec));
+	snd_magic_kfree(codec);
+	return 0;
 }
 
+EXPORT_SYMBOL(snd_cs461x_create);
+EXPORT_SYMBOL(snd_cs461x_interrupt);
+EXPORT_SYMBOL(snd_cs461x_pcm);
+EXPORT_SYMBOL(snd_cs461x_mixer);
+EXPORT_SYMBOL(snd_cs461x_midi);
+
 /*
  *  INIT part
  */
 
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_cs461x_export;
-#endif
+#ifdef MODULE
 
-int init_module(void)
+int __init init_module(void)
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_cs461x_export) < 0)
-		return -ENOMEM;
-#endif
 	return 0;
 }
 
-void cleanup_module(void)
+void __exit cleanup_module(void)
 {
 }
+
+#endif
