@@ -23,13 +23,15 @@
  */
 
 #define SND_MAIN_OBJECT_FILE
-#include "driver.h"
-#include "emu8000.h"
-#include "emu8000_reg.h"
-#include "soundfont.h"
-#include "info.h"
+#include "../../include/driver.h"
+#include "../../include/emu8000.h"
+#include "../../include/emu8000_reg.h"
+#include "../../include/soundfont.h"
+#include "../../include/info.h"
 #include "emu8000_port.h"
 #include "emu8000_voice.h"
+#include "emu8000_equalizer.h"
+#include "../../include/seq_device.h"
 
 static int snd_emu8000_detect(emu8000_t * emu);
 static void snd_emu8000_free(void *private_data);
@@ -120,92 +122,93 @@ static void snd_emu8000_free(void *private_data)
 {
 	emu8000_t *emu;
 
-	emu = (emu8000_t *)private_data;
+	emu = snd_magic_cast(emu8000_t, private_data, );
+
+	snd_emu8000_free_mixer(emu);
 
 #ifdef CONFIG_SND_OSSEMUL
 	snd_emu8000_free_seq_oss(emu);
 #endif
 	snd_emu8000_detach_seq(emu);
 
-	/* It would be nice if info_free_entry would do this for us */
-	if (emu->ram_proc_entry) {
-		snd_emu8000_proc_free(emu->ram_proc_entry); /* TEMP */
-		snd_info_free_entry(emu->ram_proc_entry);
-	}
-	if (emu->rom_proc_entry) {
-		snd_emu8000_proc_free(emu->rom_proc_entry); /* TEMP */
-		snd_info_free_entry(emu->rom_proc_entry);
-	}
-	if (emu->patch_proc_entry) {
-		snd_emu8000_proc_free(emu->patch_proc_entry); /* TEMP */
-		snd_info_free_entry(emu->patch_proc_entry);
-	}
+	if (emu->ram_proc_entry)
+		snd_emu8000_proc_free(emu->ram_proc_entry);
+
+	if (emu->rom_proc_entry)
+		snd_emu8000_proc_free(emu->rom_proc_entry);
+
+	if (emu->patch_proc_entry)
+		snd_emu8000_proc_free(emu->patch_proc_entry);
 
 	if (emu->sflist) {
 		snd_sf_free(emu->sflist);
 	}
 
 	if (emu->voices) {
-		snd_free(emu->voices, sizeof(emu8000_voice_t) * emu->max_channels);
+		snd_kfree(emu->voices);
 	}
 
-	snd_free(private_data, sizeof(emu8000_t));
+	snd_magic_kfree(emu);
 }
 
 /*
- * create a new synth device for Emu8000
+ * create a new hardware dependent device for Emu8000
  */
-snd_synth_t *snd_emu8000_new_device(snd_card_t * card, unsigned short base)
+int snd_emu8000_new_device(snd_seq_device_t *dev)
 {
-	snd_synth_t *synth;
+	snd_hwdep_t *hw;
 	emu8000_t *emu;
+	emu8000_arg_t *arg;
 	snd_sf_callback_t sf_cb;
+	int err;
 
-	if ((synth = snd_synth_new_device(card, "EMU8000")) == NULL)
-		return NULL;
-	strcpy(synth->name, "EMU-8000");
+	if ((err = snd_hwdep_new(dev->card, dev->id, dev->device, &hw)) < 0)
+		return err;
+	strcpy(hw->name, dev->name);
+	hw->type = SND_HWDEP_TYPE_EMU8000;
 
-	emu = snd_calloc(sizeof(emu8000_t));
-	if (!emu) {
-		snd_synth_free(synth);
-		return NULL;
+	emu = snd_magic_kcalloc(emu8000_t, 0, GFP_KERNEL);
+	if (emu == NULL) {
+		snd_device_free(dev->card, hw);
+		return -ENOMEM;
 	}
 
 	emu->magic = EMU8000_MAGIC;
 
-	emu->port1 = base;
-	emu->port2 = base + 0x400;
-	emu->port3 = base + 0x800;
+	arg = SND_SEQ_DEVICE_ARGPTR(dev);
+	emu->port1 = arg->port;
+	emu->port2 = arg->port + 0x400;
+	emu->port3 = arg->port + 0x800;
 
-	emu->card = card;
+	emu->card = dev->card;
 
 	emu->mem_size = -1;
 	emu->max_channels = EMU8000_CHANNELS;
 	emu->max_voices   = EMU8000_CHANNELS;
 
-	snd_spin_prepare(emu, voice);
-	snd_spin_prepare(emu, reg);
-	snd_sleep_prepare(emu, wait);
-	snd_mutex_prepare(emu, patch);
-	snd_mutex_prepare(emu, register);
+	emu->voice_lock = SPIN_LOCK_UNLOCKED;
+	emu->reg_lock = SPIN_LOCK_UNLOCKED;
+	init_waitqueue_head(&emu->wait);
+	init_MUTEX(&emu->patch_mutex);
+	init_MUTEX(&emu->register_mutex);
 
 	emu->client = -1;
 #ifdef CONFIG_SND_OSSEMUL
-	emu->oss_synth = -1;
+	emu->oss_synth = NULL;
 #endif
 
-	synth->private_data = emu;
-	synth->private_free = snd_emu8000_free;
+	hw->private_data = emu;
+	hw->private_free = snd_emu8000_free;
 
-	emu->voices = snd_calloc(sizeof(emu8000_voice_t) * emu->max_channels);
-	if (! emu->voices) {
-		snd_synth_free(synth);
-		return NULL;
+	emu->voices = snd_kcalloc(sizeof(emu8000_voice_t) * emu->max_channels, GFP_KERNEL);
+	if (emu->voices == NULL) {
+		snd_device_free(dev->card, hw);
+		return -ENOMEM;
 	}
 
 	if (snd_emu8000_detect(emu) < 0) {
-		snd_synth_free(synth);
-		return NULL;
+		snd_device_free(dev->card, hw);
+		return -ENXIO;
 	}
 
 	memset(&sf_cb, 0, sizeof(sf_cb));
@@ -213,9 +216,9 @@ snd_synth_t *snd_emu8000_new_device(snd_card_t * card, unsigned short base)
 	sf_cb.sample_write = snd_emu8000_sample_write;
 	sf_cb.sample_reset = (snd_sf_sample_reset_t)snd_emu8000_terminate_all;
 	emu->sflist = snd_sf_new(&sf_cb);
-	if (! emu->sflist) {
-		snd_synth_free(synth);
-		return NULL;
+	if (emu->sflist == NULL) {
+		snd_device_free(dev->card, hw);
+		return -ENOMEM;
 	}
 
 	emu->bass_level = 5;
@@ -230,10 +233,20 @@ snd_synth_t *snd_emu8000_new_device(snd_card_t * card, unsigned short base)
 #ifdef CONFIG_SND_OSSEMUL
 	snd_emu8000_init_seq_oss(emu);
 #endif
+	snd_emu8000_create_mixer(emu, arg->mixer, arg->mixer_dest, arg->mixer_index);
 
-	return synth;
+	dev->driver_data = hw;
+	snd_device_register(hw->card, hw);
+
+	return 0;
 }
 
+int snd_emu8000_delete_device(snd_seq_device_t *dev)
+{
+	return snd_device_unregister(dev->card, dev->driver_data);
+}
+
+
 /*
  * Set up a channel to be used for DMA.
  */
@@ -356,26 +369,32 @@ void snd_emu8000_wait(emu8000_t *emu, unsigned short delay)
 
 	waittime = (HZ * (unsigned long)delay + 44099) / 44100;
 	//printk("wait %ld\n", waittime);
-	snd_sleep(emu, wait, waittime);
+	interruptible_sleep_on_timeout(&emu->wait, waittime);
 }
 
 /*
  *  INIT part
  */
-#ifndef LINUX_2_1
-extern struct symbol_table snd_symbol_table_emu8000_export;
-#endif
 
-int init_module(void)
+#ifdef MODULE
+int __init init_module(void)
+#else
+int __init alsa_emu8000_init(void)
+#endif
 {
-#ifndef LINUX_2_1
-	if (register_symtab(&snd_symbol_table_emu8000_export) < 0)
-		return -ENOMEM;
-#endif
-
-	return 0;
+	
+	static snd_seq_dev_ops_t ops = {
+		snd_emu8000_new_device,
+		snd_emu8000_delete_device,
+	};
+	return snd_seq_device_register_driver(SND_SEQ_DEV_ID_EMU8000, &ops, sizeof(emu8000_arg_t));
 }
 
-void cleanup_module(void)
+#ifdef MODULE
+
+void __exit cleanup_module(void)
 {
+	snd_seq_device_unregister_driver(SND_SEQ_DEV_ID_EMU8000);
 }
+
+#endif
