@@ -20,9 +20,10 @@
  *
  */
 
-#include "driver.h"
-#include "emu8000.h"
-#include "emu8000_reg.h"
+#define __SND_OSS_COMPAT__
+#include "../../include/driver.h"
+#include "../../include/emu8000.h"
+#include "../../include/emu8000_reg.h"
 #include "emu8000_equalizer.h"
 #include "emu8000_port.h"
 
@@ -40,35 +41,6 @@ MODULE_PARM(use_mixer, "i");
 MODULE_PARM_DESC(use_mixer, "enable bass/treble change");
 #endif
 
-
-/*
- * copy from/to user-space
- */
-#ifndef LINUX_2_1
-
-#undef copy_from_user
-#undef copy_to_user
-
-inline static int
-copy_from_user(void *dst, const void *src, int size)
-{
-	if (verify_area(VERIFY_READ, src, size))
-		return -EFAULT;
-	memcpy_fromfs(dst, src, size);
-	return 0;
-}
-
-inline static int
-copy_to_user(void *dst, const void *src, int size)
-{
-	if (verify_area(VERIFY_WRITE, dst, size))
-		return -EFAULT;
-	memcpy_tofs(dst, src, size);
-	return 0;
-}
-
-#endif
-
 static unsigned short bass_parm[12][3] = {
 	{0xD26A, 0xD36A, 0x0000}, /* -12 dB */
 	{0xD25B, 0xD35B, 0x0000}, /*  -8 */
@@ -325,3 +297,173 @@ snd_emu8000_update_reverb_mode(emu8000_t *emu)
 {
 	snd_emu8000_set_reverb_mode(emu, emu->reverb_mode);
 }
+
+/*----------------------------------------------------------------
+ * mixer interface
+ *----------------------------------------------------------------*/
+
+/*
+ * update bass/treble values
+ */
+static int mixer_tone_control(snd_kmixer_element_t *element, int w_flag,
+			      struct snd_mixer_element_tone_control1 *tc1)
+{
+	emu8000_t *emu = snd_magic_cast(emu8000_t, element->private_data, -ENXIO);
+	int change = 0;
+
+	if (w_flag) {
+		if ((tc1->tc & SND_MIXER_TC1_BASS) &&
+		    tc1->bass != emu->bass_level) {
+			emu->bass_level = tc1->bass;
+			change = 1;
+		}
+		if ((tc1->tc & SND_MIXER_TC1_TREBLE) &&
+		    tc1->treble != emu->treble_level) {
+			emu->treble_level = tc1->treble;
+			change = 1;
+		}
+		if (change)
+			snd_emu8000_update_equalizer(emu);
+	} else {
+		tc1->tc = SND_MIXER_TC1_BASS | SND_MIXER_TC1_TREBLE;
+		tc1->bass = emu->bass_level;
+		tc1->treble = emu->treble_level;
+	}
+
+	return change;
+}
+
+/*
+ * bass group callback
+ */
+static int mixer_group_bass(snd_kmixer_group_t *group,
+			    snd_kmixer_file_t *file, int w_flag,
+			    snd_mixer_group_t *ugroup)
+{
+	emu8000_t *emu = snd_magic_cast(emu8000_t, group->private_data, -ENXIO);
+	struct snd_mixer_element_tone_control1 tc;
+	int change = 0;
+
+	if (w_flag) {
+		tc.tc = SND_MIXER_TC1_BASS;
+		tc.bass = ugroup->volume.names.front_left;
+		if (mixer_tone_control(emu->mixer.me_tone, 1, &tc) > 0) {
+			snd_mixer_element_value_change(file, emu->mixer.me_tone, 0);
+			change = 1;
+		}
+	} else {
+		ugroup->caps = SND_MIXER_GRPCAP_VOLUME;
+		ugroup->channels = SND_MIXER_CHN_MASK_MONO;
+		mixer_tone_control(emu->mixer.me_tone, 0, &tc);
+		ugroup->volume.names.front_left = tc.bass;
+		ugroup->min = 0;
+		ugroup->max = 11;
+	}
+
+	return change;
+}
+
+/*
+ * treble group callback
+ */
+static int mixer_group_treble(snd_kmixer_group_t *group,
+			      snd_kmixer_file_t *file, int w_flag,
+			      snd_mixer_group_t *ugroup)
+{
+	emu8000_t *emu = snd_magic_cast(emu8000_t, group->private_data, -ENXIO);
+	struct snd_mixer_element_tone_control1 tc;
+	int change = 0;
+
+	if (w_flag) {
+		tc.tc = SND_MIXER_TC1_TREBLE;
+		tc.treble = ugroup->volume.names.front_left;
+		if (mixer_tone_control(emu->mixer.me_tone, 1, &tc) > 0) {
+			snd_mixer_element_value_change(file, emu->mixer.me_tone, 0);
+			change = 1;
+		}
+	} else {
+		ugroup->caps = SND_MIXER_GRPCAP_VOLUME;
+		ugroup->channels = SND_MIXER_CHN_MASK_MONO;
+		mixer_tone_control(emu->mixer.me_tone, 0, &tc);
+		ugroup->volume.names.front_left = tc.treble;
+		ugroup->min = 0;
+		ugroup->max = 11;
+	}
+
+	return change;
+}
+
+/*
+ * create and attach mixer elements for WaveTable treble/bass controls
+ */
+int snd_emu8000_create_mixer(emu8000_t *emu, snd_kmixer_t *mixer, snd_kmixer_element_t *me_dest, int index)
+{
+	emu8000_mixer_t *smix;
+	static struct snd_mixer_element_tone_control1_info tone_control = {
+		SND_MIXER_TC1_BASS | SND_MIXER_TC1_TREBLE,
+		0, 11, -1000, 1200,
+		0, 11, -1000, 1200
+	};
+
+	snd_debug_check(mixer == NULL || emu == NULL, -EINVAL);
+
+	smix = &emu->mixer;
+	memset(smix, 0, sizeof(*smix)); /* clear table */
+	smix->mixer = mixer;
+
+	snd_mixer_lock(mixer, 0);
+
+	smix->me_tone = snd_mixer_lib_tone_control1(mixer, "Synth Tone", 0,
+						    &tone_control,
+						    mixer_tone_control, emu);
+	if (smix->me_tone == NULL)
+		goto __error;
+	smix->me_bass = snd_mixer_lib_group_ctrl(mixer, "Synth Bass", 0,
+						 SND_MIXER_OSS_UNKNOWN,
+						 mixer_group_bass, emu);
+	if (smix->me_bass == NULL)
+		goto __error;
+	smix->me_treble = snd_mixer_lib_group_ctrl(mixer, "Synth Treble", 0,
+						   SND_MIXER_OSS_UNKNOWN,
+						   mixer_group_treble, emu);
+	if (smix->me_treble == NULL)
+		goto __error;
+	if (snd_mixer_group_element_add(mixer, smix->me_bass, smix->me_tone) < 0 ||
+	    snd_mixer_group_element_add(mixer, smix->me_treble, smix->me_tone) < 0 ||
+	    snd_mixer_element_route_add(mixer, smix->me_tone, me_dest) < 0)
+		goto __error;
+	
+	snd_mixer_lock(mixer, 1);
+	return 0;
+
+__error:
+	snd_mixer_lock(mixer, 1);
+	snd_emu8000_free_mixer(emu);
+
+	return -EINVAL;
+}
+
+/*
+ * release mixer elements
+ */
+void snd_emu8000_free_mixer(emu8000_t *emu)
+{
+	emu8000_mixer_t *smix;
+	snd_kmixer_t *mixer;
+
+	if (emu == NULL)
+		return;
+
+	smix = &emu->mixer;
+	mixer = smix->mixer;
+
+	snd_mixer_lock(mixer, 0);
+	if (smix->me_bass)
+		snd_mixer_group_remove(mixer, smix->me_bass);
+	if (smix->me_treble)
+		snd_mixer_group_remove(mixer, smix->me_treble);
+	if (smix->me_tone)
+		snd_mixer_element_remove(mixer, smix->me_tone);
+	memset(smix, 0, sizeof(*smix));
+	snd_mixer_lock(mixer, 1);
+}
