@@ -19,7 +19,40 @@
  *
  */
 
-static void *isapnp_alloc(long size);
+#ifdef ALSA_BUILD
+#define MODULE
+#define CONFIG_ISAPNP
+#endif
+
+#define __NO_VERSION__
+
+#include <linux/config.h>
+#include <linux/module.h>
+
+#define LinuxVersionCode(v, p, s) (((v)<<16)|((p)<<8)|(s))
+
+#if LinuxVersionCode(2, 3, 29) <= LINUX_VERSION_CODE
+#define NEW_PROC
+#endif
+
+#ifdef ALSA_BUILD
+#if defined(CONFIG_MODVERSIONS) && !defined(__GENKSYMS__) && !defined(__DEPEND__)
+#define MODVERSIONS
+#include <linux/modversions.h>
+#include "include/sndversions.h"
+#endif
+#endif
+
+#include <linux/kernel.h>
+#include <linux/init.h>
+#include <linux/proc_fs.h>
+#include <linux/poll.h>
+#include <linux/vmalloc.h>
+#include <asm/uaccess.h>
+#include "isapnp.h"
+
+void *isapnp_alloc(long size);
+struct isapnp_card *isapnp_cards;
 struct isapnp_dev *isapnp_devices;
 
 struct isapnp_info_buffer {
@@ -34,7 +67,10 @@ struct isapnp_info_buffer {
 typedef struct isapnp_info_buffer isapnp_info_buffer_t;
 
 static struct proc_dir_entry *isapnp_proc_entry = NULL;
+static struct proc_dir_entry *isapnp_proc_bus_dir = NULL;
+static struct proc_dir_entry *isapnp_proc_devices_entry = NULL;
 
+static void isapnp_proc_read_devices(isapnp_info_buffer_t *buffer);
 static void isapnp_info_read(isapnp_info_buffer_t *buffer);
 static void isapnp_info_write(isapnp_info_buffer_t *buffer);
 
@@ -59,6 +95,18 @@ int isapnp_printf(isapnp_info_buffer_t * buffer, char *fmt,...)
 	return res;
 }
 
+static void isapnp_devid(char *str, unsigned short vendor, unsigned short device)
+{
+	sprintf(str, "%c%c%c%x%x%x%x",
+			'A' + ((vendor >> 2) & 0x3f) - 1,
+			'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
+			'A' + ((vendor >> 8) & 0x1f) - 1,
+			(device >> 4) & 0x0f,
+			device & 0x0f,
+			(device >> 12) & 0x0f,
+			(device >> 8) & 0x0f);
+}
+
 static loff_t isapnp_info_entry_lseek(struct file *file, loff_t offset, int orig)
 {
 	switch (orig) {
@@ -86,7 +134,7 @@ static ssize_t isapnp_info_entry_read(struct file *file, char *buffer,
 	if (mode != O_RDONLY)
 		return -EINVAL;
 	buf = (isapnp_info_buffer_t *) file->private_data;
-	if (!buf)
+	if (buf == NULL)
 		return -EIO;
 	if (file->f_pos >= buf->size)
 		return 0;
@@ -111,7 +159,7 @@ static ssize_t isapnp_info_entry_write(struct file *file, const char *buffer,
 	if (mode != O_WRONLY)
 		return -EINVAL;
 	buf = (isapnp_info_buffer_t *) file->private_data;
-	if (!buf)
+	if (buf == NULL)
 		return -EIO;
 	if (file->f_pos < 0)
 		return -EINVAL;
@@ -131,6 +179,7 @@ static ssize_t isapnp_info_entry_write(struct file *file, const char *buffer,
 
 static int isapnp_info_entry_open(struct inode *inode, struct file *file)
 {
+	struct proc_dir_entry *dp = inode->u.generic_ip;
 	isapnp_info_buffer_t *buffer;
 	int mode;
 
@@ -139,19 +188,24 @@ static int isapnp_info_entry_open(struct inode *inode, struct file *file)
 		return -EINVAL;
 	buffer = (isapnp_info_buffer_t *)
 				isapnp_alloc(sizeof(isapnp_info_buffer_t));
-	if (!buffer)
+	if (buffer == NULL)
 		return -ENOMEM;
 	buffer->len = 4 * PAGE_SIZE;
 	buffer->buffer = vmalloc(buffer->len);
-	if (!buffer->buffer) {
+	if (buffer->buffer == NULL) {
 		kfree(buffer);
 		return -ENOMEM;
 	}
 	buffer->curr = buffer->buffer;
 	file->private_data = buffer;
 	MOD_INC_USE_COUNT;
-	if (mode == O_RDONLY)
-		isapnp_info_read(buffer);
+	if (mode == O_RDONLY) {
+		if (!strcmp(dp->name, "isapnp")) {
+			isapnp_info_read(buffer);
+		} else {
+			isapnp_proc_read_devices(buffer);
+		}
+	}
 	return 0;
 }
 
@@ -160,10 +214,8 @@ static int isapnp_info_entry_release(struct inode *inode, struct file *file)
 	isapnp_info_buffer_t *buffer;
 	int mode;
 
-	if ((buffer = (isapnp_info_buffer_t *) file->private_data) == NULL) {
-		MOD_DEC_USE_COUNT;
+	if ((buffer = (isapnp_info_buffer_t *) file->private_data) == NULL)
 		return -EINVAL;
-	}
 	mode = file->f_flags & O_ACCMODE;
 	if (mode == O_WRONLY)
 		isapnp_info_write(buffer);
@@ -175,7 +227,7 @@ static int isapnp_info_entry_release(struct inode *inode, struct file *file)
 
 static unsigned int isapnp_info_entry_poll(struct file *file, poll_table * wait)
 {
-	if (!file->private_data)
+	if (file->private_data == NULL)
 		return 0;
 	return POLLIN | POLLRDNORM;
 }
@@ -231,24 +283,199 @@ static struct inode_operations isapnp_info_entry_inode_operations =
 	NULL			/* permission */
 };
 
-__initfunc(static int isapnp_proc_init(void))
+static loff_t isapnp_proc_bus_lseek(struct file *file, loff_t off, int whence)
 {
-	struct proc_dir_entry *p;
+	loff_t new;
+	
+	switch (whence) {
+	case 0:
+		new = off;
+		break;
+	case 1:
+		new = file->f_pos + off;
+		break;
+	case 2:
+		new = 256 + off;
+		break;
+	default:
+		return -EINVAL;
+	}
+	if (new < 0 || new > 256)
+		return -EINVAL;
+	return (file->f_pos = new);
+}
 
-	isapnp_proc_entry = NULL;
-	p = create_proc_entry("isapnp", S_IFREG | S_IRUGO | S_IWUSR, &proc_root);
-	if (!p)
+static ssize_t isapnp_proc_bus_read(struct file *file, char *buf, size_t nbytes, loff_t *ppos)
+{
+	struct inode *ino = file->f_dentry->d_inode;
+	struct proc_dir_entry *dp = ino->u.generic_ip;
+	struct isapnp_dev *dev = dp->data;
+	int pos = *ppos;
+	int cnt, size = 256;
+
+	if (pos >= size)
+		return 0;
+	if (nbytes >= size)
+		nbytes = size;
+	if (pos + nbytes > size)
+		nbytes = size - pos;
+	cnt = nbytes;
+
+	if (!access_ok(VERIFY_WRITE, buf, cnt))
+		return -EINVAL;
+		
+	isapnp_cfg_begin(dev->bus->number, dev->devfn);
+	for ( ; pos < 256 && cnt > 0; pos++, buf++, cnt--) {
+		unsigned char val;
+		val = isapnp_read_byte(pos);
+		__put_user(val, buf);
+	}
+	isapnp_cfg_end();
+	
+	*ppos = pos;
+	return nbytes;
+}
+
+static struct file_operations isapnp_proc_bus_file_operations =
+{
+	isapnp_proc_bus_lseek,		/* lseek */
+	isapnp_proc_bus_read,		/* read */
+	NULL,				/* write */
+	NULL,				/* readdir */
+	NULL,				/* poll */
+	NULL,				/* ioctl - default */
+	NULL,				/* mmap */
+	NULL,				/* open */
+	NULL,				/* flush */
+	NULL,				/* release */
+	NULL,				/* can't fsync */
+	NULL,				/* fasync */
+	NULL,				/* lock */
+};
+
+static struct inode_operations isapnp_proc_bus_inode_operations =
+{
+	&isapnp_proc_bus_file_operations,
+};
+
+static int isapnp_proc_attach_device(struct isapnp_dev *dev)
+{
+	struct isapnp_card *bus = dev->bus;
+	struct proc_dir_entry *de, *e;
+	char name[16];
+
+	if (!(de = bus->procdir)) {
+		sprintf(name, "%02x", bus->number);
+		de = bus->procdir = create_proc_entry(name, S_IFDIR | S_IRUGO | S_IXUGO, isapnp_proc_bus_dir);
+		if (!de)
+			return -ENOMEM;
+	}
+	sprintf(name, "%02x", dev->devfn);
+	e = dev->procent = create_proc_entry(name, S_IFREG | S_IRUGO, de);
+	if (!e)
 		return -ENOMEM;
-	p->ops = &isapnp_info_entry_inode_operations;
-	isapnp_proc_entry = p;
+	e->ops = &isapnp_proc_bus_inode_operations;
+	e->data = dev;
+	e->size = 256;
 	return 0;
 }
 
 #ifdef MODULE
-static int isapnp_proc_done(void)
+static int isapnp_proc_detach_device(struct isapnp_dev *dev)
 {
+	struct isapnp_card *bus = dev->bus;
+	struct proc_dir_entry *de;
+	char name[16];
+
+	if (!(de = bus->procdir))
+		return -EINVAL;
+	sprintf(name, "%02x", dev->devfn);
+	remove_proc_entry(name, de);
+	return 0;
+}
+
+static int isapnp_proc_detach_bus(struct isapnp_card *bus)
+{
+	struct proc_dir_entry *de;
+	char name[16];
+
+	if (!(de = bus->procdir))
+		return -EINVAL;
+	sprintf(name, "%02x", bus->number);
+	remove_proc_entry(name, isapnp_proc_bus_dir);
+	return 0;
+}
+#endif
+
+static void isapnp_proc_read_devices(isapnp_info_buffer_t *buffer)
+{
+	struct isapnp_dev *dev;
+	int i;
+
+	for (dev = isapnp_devices; dev != NULL; dev = dev->next) {
+		char bus_id[8], device_id[8];
+	
+		isapnp_devid(bus_id, dev->bus->vendor, dev->bus->device);
+		isapnp_devid(device_id, dev->vendor, dev->device);
+		isapnp_printf(buffer, "%02x%02x\t%s%s\t",
+			      dev->bus->number,
+			      dev->devfn,
+			      bus_id,
+			      device_id);
+		isapnp_cfg_begin(dev->bus->number, dev->devfn);
+		isapnp_printf(buffer, "%02x", isapnp_read_byte(ISAPNP_CFG_ACTIVATE));
+		for (i = 0; i < 8; i++)
+			isapnp_printf(buffer, "%04x", isapnp_read_word(ISAPNP_CFG_PORT + (i << 1)));
+		for (i = 0; i < 2; i++)
+			isapnp_printf(buffer, "%04x", isapnp_read_word(ISAPNP_CFG_IRQ + (i << 1)));
+		for (i = 0; i < 2; i++)
+			isapnp_printf(buffer, "%04x", isapnp_read_word(ISAPNP_CFG_DMA + i));
+		for (i = 0; i < 4; i++)
+			isapnp_printf(buffer, "%08x", isapnp_read_dword(ISAPNP_CFG_MEM + (i << 3)));
+		isapnp_cfg_end();
+		isapnp_printf(buffer, "\n");
+	}
+}
+
+int isapnp_proc_init(void)
+{
+	struct proc_dir_entry *p;
+	struct isapnp_dev *dev;
+
+	isapnp_proc_entry = NULL;
+	p = create_proc_entry("isapnp", S_IFREG | S_IRUGO | S_IWUSR, &proc_root);
+	if (p == NULL)
+		return -ENOMEM;
+	p->ops = &isapnp_info_entry_inode_operations;
+	isapnp_proc_entry = p;
+	isapnp_proc_bus_dir = create_proc_entry("isapnp", S_IFDIR | S_IRUGO | S_IXUGO, proc_bus);
+	isapnp_proc_devices_entry = create_proc_entry("devices", 0, isapnp_proc_bus_dir);
+	if (isapnp_proc_devices_entry)
+		isapnp_proc_devices_entry->ops = &isapnp_info_entry_inode_operations;
+	for (dev = isapnp_devices; dev != NULL; dev = dev->next) {
+		isapnp_proc_attach_device(dev);
+	}
+	return 0;
+}
+
+#ifdef MODULE
+int isapnp_proc_done(void)
+{
+	struct isapnp_dev *dev;
+	struct isapnp_card *card;
+
+	for (dev = isapnp_devices; dev != NULL; dev = dev->next) {
+		isapnp_proc_detach_device(dev);
+	}
+	for (card = isapnp_cards; card != NULL; card = card->next) {
+		isapnp_proc_detach_bus(card);
+	}
+	if (isapnp_proc_devices_entry)
+		remove_proc_entry("devices", isapnp_proc_devices_entry);
+	if (isapnp_proc_bus_dir)
+		remove_proc_entry("isapnp", proc_bus);
 	if (isapnp_proc_entry)
-		proc_unregister(&proc_root, isapnp_proc_entry->low_ino);
+		remove_proc_entry("isapnp", &proc_root);
 	return 0;
 }
 #endif /* MODULE */
@@ -261,24 +488,22 @@ static void isapnp_print_devid(isapnp_info_buffer_t *buffer, unsigned short vend
 {
 	char tmp[8];
 	
-	sprintf(tmp, "%c%c%c%x%x%x%x",
-			'A' + ((vendor >> 2) & 0x3f) - 1,
-			'A' + (((vendor & 3) << 3) | ((vendor >> 13) & 7)) - 1,
-			'A' + ((vendor >> 8) & 0x1f) - 1,
-			(device >> 4) & 0x0f,
-			device & 0x0f,
-			(device >> 12) & 0x0f,
-			(device >> 8) & 0x0f);
+	isapnp_devid(tmp, vendor, device);
 	isapnp_printf(buffer, tmp);
 }
 
-static void isapnp_print_compatible(isapnp_info_buffer_t *buffer, struct isapnp_compatid *compat)
+static void isapnp_print_compatible(isapnp_info_buffer_t *buffer, struct isapnp_dev *dev)
 {
-	while (compat) {
+	int idx;
+
+	for (idx = 0; idx < DEVICE_COUNT_COMPATIBLE; idx++) {
+		if (dev->vendor_compatible[idx] == 0)
+			continue;
 		isapnp_printf(buffer, "    Compatible device ");
-		isapnp_print_devid(buffer, compat->vendor, compat->function);
+		isapnp_print_devid(buffer,
+				   dev->vendor_compatible[idx],
+				   dev->device_compatible[idx]);
 		isapnp_printf(buffer, "\n");
-		compat = compat->next;
 	}
 }
 
@@ -308,13 +533,13 @@ static void isapnp_print_irq(isapnp_info_buffer_t *buffer, char *space, struct i
 		}
 	if (!irq->map)
 		isapnp_printf(buffer, "<none>");
-	if (irq->flags & ISAPNP_IRQ_FLAG_HIGHEDGE)
+	if (irq->flags & IORESOURCE_IRQ_HIGHEDGE)
 		isapnp_printf(buffer, " High-Edge");
-	if (irq->flags & ISAPNP_IRQ_FLAG_LOWEDGE)
+	if (irq->flags & IORESOURCE_IRQ_LOWEDGE)
 		isapnp_printf(buffer, " Low-Edge");
-	if (irq->flags & ISAPNP_IRQ_FLAG_HIGHLEVEL)
+	if (irq->flags & IORESOURCE_IRQ_HIGHLEVEL)
 		isapnp_printf(buffer, " High-Level");
-	if (irq->flags & ISAPNP_IRQ_FLAG_LOWLEVEL)
+	if (irq->flags & IORESOURCE_IRQ_LOWLEVEL)
 		isapnp_printf(buffer, " Low-Level");
 	isapnp_printf(buffer, "\n");
 }
@@ -336,31 +561,31 @@ static void isapnp_print_dma(isapnp_info_buffer_t *buffer, char *space, struct i
 		}
 	if (!dma->map)
 		isapnp_printf(buffer, "<none>");
-	switch (dma->type) {
-	case ISAPNP_DMA_TYPE_8BIT:
+	switch (dma->flags & IORESOURCE_DMA_TYPE_MASK) {
+	case IORESOURCE_DMA_8BIT:
 		s = "8-bit";
 		break;
-	case ISAPNP_DMA_TYPE_8AND16BIT:
+	case IORESOURCE_DMA_8AND16BIT:
 		s = "8-bit&16-bit";
 		break;
 	default:
 		s = "16-bit";
 	}
 	isapnp_printf(buffer, " %s", s);
-	if (dma->flags & ISAPNP_DMA_FLAG_MASTER)
+	if (dma->flags & IORESOURCE_DMA_MASTER)
 		isapnp_printf(buffer, " master");
-	if (dma->flags & ISAPNP_DMA_FLAG_BYTE)
+	if (dma->flags & IORESOURCE_DMA_BYTE)
 		isapnp_printf(buffer, " byte-count");
-	if (dma->flags & ISAPNP_DMA_FLAG_WORD)
+	if (dma->flags & IORESOURCE_DMA_WORD)
 		isapnp_printf(buffer, " word-count");
-	switch (dma->speed) {
-	case ISAPNP_DMA_SPEED_TYPEA:
+	switch (dma->flags & IORESOURCE_DMA_SPEED_MASK) {
+	case IORESOURCE_DMA_TYPEA:
 		s = "type-A";
 		break;
-	case ISAPNP_DMA_SPEED_TYPEB:
+	case IORESOURCE_DMA_TYPEB:
 		s = "type-B";
 		break;
-	case ISAPNP_DMA_SPEED_TYPEF:
+	case IORESOURCE_DMA_TYPEF:
 		s = "type-F";
 		break;
 	default:
@@ -376,21 +601,21 @@ static void isapnp_print_mem(isapnp_info_buffer_t *buffer, char *space, struct i
 
 	isapnp_printf(buffer, "%sMemory 0x%x-0x%x, align 0x%x, size 0x%x",
 			space, mem->min, mem->max, mem->align, mem->size);
-	if (mem->flags & ISAPNP_MEM_FLAG_WRITEABLE)
+	if (mem->flags & IORESOURCE_MEM_WRITEABLE)
 		isapnp_printf(buffer, ", writeable");
-	if (mem->flags & ISAPNP_MEM_FLAG_CACHEABLE)
+	if (mem->flags & IORESOURCE_MEM_CACHEABLE)
 		isapnp_printf(buffer, ", cacheable");
-	if (mem->flags & ISAPNP_MEM_FLAG_RANGELENGTH)
+	if (mem->flags & IORESOURCE_MEM_RANGELENGTH)
 		isapnp_printf(buffer, ", range-length");
-	if (mem->flags & ISAPNP_MEM_FLAG_SHADOWABLE)
+	if (mem->flags & IORESOURCE_MEM_SHADOWABLE)
 		isapnp_printf(buffer, ", shadowable");
-	if (mem->flags & ISAPNP_MEM_FLAG_EXPANSIONROM)
+	if (mem->flags & IORESOURCE_MEM_EXPANSIONROM)
 		isapnp_printf(buffer, ", expansion ROM");
-	switch (mem->type) {
-	case ISAPNP_MEM_TYPE_8BIT:
+	switch (mem->flags & IORESOURCE_MEM_TYPE_MASK) {
+	case IORESOURCE_MEM_8BIT:
 		s = "8-bit";
 		break;
-	case ISAPNP_MEM_TYPE_8AND16BIT:
+	case IORESOURCE_MEM_8AND16BIT:
 		s = "8-bit&16-bit";
 		break;
 	default:
@@ -449,16 +674,16 @@ static void isapnp_print_resources(isapnp_info_buffer_t *buffer, char *space, st
 		isapnp_print_mem32(buffer, space, mem32);
 }
 
-static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isapnp_logdev *logdev)
+static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isapnp_dev *dev)
 {
 	int i, tmp, next;
 	char *space = "    ";
 
-	isapnp_cfg_begin(logdev->dev->csn, logdev->number);
+	isapnp_cfg_begin(dev->bus->number, dev->devfn);
 	isapnp_printf(buffer, "%sDevice is %sactive\n",
-			space, isapnp_cfg_get_byte(ISAPNP_CFG_ACTIVATE)?"":"not ");
+			space, isapnp_read_byte(ISAPNP_CFG_ACTIVATE)?"":"not ");
 	for (i = next = 0; i < 8; i++) {
-		tmp = isapnp_cfg_get_word(ISAPNP_CFG_PORT + (i << 1));
+		tmp = isapnp_read_word(ISAPNP_CFG_PORT + (i << 1));
 		if (!tmp)
 			continue;
 		if (!next) {
@@ -470,7 +695,7 @@ static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isap
 	if (next)
 		isapnp_printf(buffer, "\n");
 	for (i = next = 0; i < 2; i++) {
-		tmp = isapnp_cfg_get_word(ISAPNP_CFG_IRQ + (i << 1));
+		tmp = isapnp_read_word(ISAPNP_CFG_IRQ + (i << 1));
 		if (!(tmp >> 8))
 			continue;
 		if (!next) {
@@ -484,7 +709,7 @@ static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isap
 	if (next)
 		isapnp_printf(buffer, "\n");
 	for (i = next = 0; i < 2; i++) {
-		tmp = isapnp_cfg_get_byte(ISAPNP_CFG_DMA + i);
+		tmp = isapnp_read_byte(ISAPNP_CFG_DMA + i);
 		if (tmp == 4)
 			continue;
 		if (!next) {
@@ -496,7 +721,7 @@ static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isap
 	if (next)
 		isapnp_printf(buffer, "\n");
 	for (i = next = 0; i < 4; i++) {
-		tmp = isapnp_cfg_get_dword(ISAPNP_CFG_MEM + (i << 3));
+		tmp = isapnp_read_dword(ISAPNP_CFG_MEM + (i << 3));
 		if (!tmp)
 			continue;
 		if (!next) {
@@ -510,32 +735,32 @@ static void isapnp_print_configuration(isapnp_info_buffer_t *buffer, struct isap
 	isapnp_cfg_end();
 }
 
-static void isapnp_print_logdev(isapnp_info_buffer_t *buffer, struct isapnp_logdev *logdev)
+static void isapnp_print_device(isapnp_info_buffer_t *buffer, struct isapnp_dev *dev)
 {
 	int block, block1;
 	char *space = "    ";
 	struct isapnp_resources *res, *resa;
 
-	if (!logdev)
+	if (dev == NULL)
 		return;
-	isapnp_printf(buffer, "  Logical device %i '", logdev->number);
-	isapnp_print_devid(buffer, logdev->vendor, logdev->function);
-	isapnp_printf(buffer, ":%s'", logdev->identifier?logdev->identifier:"Unknown");
+	isapnp_printf(buffer, "  Logical device %i '", dev->devfn);
+	isapnp_print_devid(buffer, dev->vendor, dev->device);
+	isapnp_printf(buffer, ":%s'", dev->name[0]?dev->name:"Unknown");
 	isapnp_printf(buffer, "\n");
 #if 0
-	isapnp_cfg_begin(logdev->dev->csn, logdev->number);
+	isapnp_cfg_begin(dev->bus->number, dev->devfn);
 	for (block = 0; block < 128; block++)
 		if ((block % 16) == 15)
-			isapnp_printf(buffer, "%02x\n", isapnp_cfg_get_byte(block));
+			isapnp_printf(buffer, "%02x\n", isapnp_read_byte(block));
 		else
-			isapnp_printf(buffer, "%02x:", isapnp_cfg_get_byte(block));
+			isapnp_printf(buffer, "%02x:", isapnp_read_byte(block));
 	isapnp_cfg_end();
 #endif
-	if (logdev->regs)
-		isapnp_printf(buffer, "%sSupported registers 0x%x\n", space, logdev->regs);
-	isapnp_print_compatible(buffer, logdev->compat);
-	isapnp_print_configuration(buffer, logdev);
-	for (res = logdev->res, block = 0; res; res = res->next, block++) {
+	if (dev->regs)
+		isapnp_printf(buffer, "%sSupported registers 0x%x\n", space, dev->regs);
+	isapnp_print_compatible(buffer, dev);
+	isapnp_print_configuration(buffer, dev);
+	for (res = (struct isapnp_resources *)dev->sysdata, block = 0; res; res = res->next, block++) {
 		isapnp_printf(buffer, "%sResources %i\n", space, block);
 		isapnp_print_resources(buffer, "      ", res);
 		for (resa = res->alt, block1 = 1; resa; resa = resa->alt, block1++) {
@@ -551,20 +776,20 @@ static void isapnp_print_logdev(isapnp_info_buffer_t *buffer, struct isapnp_logd
  
 static void isapnp_info_read(isapnp_info_buffer_t *buffer)
 {
+	struct isapnp_card *card;
 	struct isapnp_dev *dev;
-	struct isapnp_logdev *logdev;
 	
-	for (dev = isapnp_devices; dev; dev = dev->next) {
-		isapnp_printf(buffer, "Device %i '", dev->csn);
-		isapnp_print_devid(buffer, dev->vendor, dev->device);
-		isapnp_printf(buffer, ":%s'", dev->identifier?dev->identifier:"Unknown");
-		if (dev->pnpver)
-			isapnp_printf(buffer, " PnP version %x.%x", dev->pnpver >> 4, dev->pnpver & 0x0f);
-		if (dev->productver)
-			isapnp_printf(buffer, " Product version %x.%x", dev->productver >> 4, dev->productver & 0x0f);
+	for (card = isapnp_cards; card; card = card->next) {
+		isapnp_printf(buffer, "Card %i '", card->number);
+		isapnp_print_devid(buffer, card->vendor, card->device);
+		isapnp_printf(buffer, ":%s'", card->name[0]?card->name:"Unknown");
+		if (card->pnpver)
+			isapnp_printf(buffer, " PnP version %x.%x", card->pnpver >> 4, card->pnpver & 0x0f);
+		if (card->productver)
+			isapnp_printf(buffer, " Product version %x.%x", card->productver >> 4, card->productver & 0x0f);
 		isapnp_printf(buffer,"\n");
-		for (logdev = dev->logdev; logdev; logdev = logdev->next)
-			isapnp_print_logdev(buffer, logdev);
+		for (dev = card->devices; dev; dev = dev->sibling)
+			isapnp_print_device(buffer, dev);
 	}
 }
 
@@ -572,8 +797,8 @@ static void isapnp_info_read(isapnp_info_buffer_t *buffer)
  *
  */
 
-static struct isapnp_dev *isapnp_info_dev;
-static struct isapnp_logdev *isapnp_info_logdev;
+static struct isapnp_card *isapnp_info_card;
+static struct isapnp_dev *isapnp_info_device;
 
 static char *isapnp_get_str(char *dest, char *src, int len)
 {
@@ -601,11 +826,11 @@ static char *isapnp_get_str(char *dest, char *src, int len)
 
 static unsigned char isapnp_get_hex(unsigned char c)
 {
-	if (c >= '0' || c <= '9')
+	if (c >= '0' && c <= '9')
 		return c - '0';
-	if (c >= 'a' || c <= 'f')
+	if (c >= 'a' && c <= 'f')
 		return (c - 'a') + 10;
-	if (c >= 'A' || c <= 'F')
+	if (c >= 'A' && c <= 'F')
 		return (c - 'A') + 10;
 	return 0;
 }
@@ -623,25 +848,27 @@ static unsigned int isapnp_parse_id(const char *id)
 			(isapnp_get_hex(id[6])<<8);
 }
 
-static int isapnp_set_device(char *line)
+static int isapnp_set_card(char *line)
 {
-	int idx;
+	int idx, idx1;
 	unsigned int id;
 	char index[16], value[32];
 
-	isapnp_info_logdev = NULL;
+	isapnp_info_card = NULL;
 	line = isapnp_get_str(index, line, sizeof(index));
 	isapnp_get_str(value, line, sizeof(value));
-	idx = simple_strtoul(index, NULL, 0);
+	idx = idx1 = simple_strtoul(index, NULL, 0);
 	id = isapnp_parse_id(value);
-	isapnp_info_dev = isapnp_find_device(id >> 16, id & 0xffff, idx);
-	if (!isapnp_info_dev) {
-		printk("isapnp: device '%s' order %i not found\n", value, idx);
+	isapnp_info_card = isapnp_find_card(id >> 16, id & 0xffff, NULL);
+	while (isapnp_info_card && idx1-- > 0)
+		isapnp_info_card = isapnp_find_card(id >> 16, id & 0xffff, isapnp_info_card);
+	if (isapnp_info_card == NULL) {
+		printk("isapnp: card '%s' order %i not found\n", value, idx);
 		return 1;
 	}
-	if (isapnp_cfg_begin(isapnp_info_dev->csn, -1)<0) {
+	if (isapnp_cfg_begin(isapnp_info_card->number, -1)<0) {
 		printk("isapnp: configuration start sequence for device '%s' failed\n", value);
-		isapnp_info_dev = NULL;
+		isapnp_info_card = NULL;
 		return 1;
 	}
 	return 0;
@@ -652,53 +879,59 @@ static int isapnp_select_csn(char *line)
 	int csn;
 	char index[16], value[32];
 
-	isapnp_info_logdev = NULL;
+	isapnp_info_device = NULL;
 	isapnp_get_str(index, line, sizeof(index));
 	csn = simple_strtoul(index, NULL, 0);
-	for (isapnp_info_dev = isapnp_devices; isapnp_info_dev; isapnp_info_dev = isapnp_info_dev->next)
-		if (isapnp_info_dev->csn == csn)
+	for (isapnp_info_card = isapnp_cards; isapnp_info_card; isapnp_info_card = isapnp_info_card->next)
+		if (isapnp_info_card->number == csn)
 			break;
-	if (!isapnp_info_dev) {
+	if (isapnp_info_card == NULL) {
 		printk("isapnp: cannot find CSN %i\n", csn);
 		return 1;
 	}
-	if (isapnp_cfg_begin(isapnp_info_dev->csn, -1)<0) {
+	if (isapnp_cfg_begin(isapnp_info_card->number, -1)<0) {
 		printk("isapnp: configuration start sequence for device '%s' failed\n", value);
-		isapnp_info_dev = NULL;
+		isapnp_info_card = NULL;
 		return 1;
 	}
 	return 0;
 }
 
-static int isapnp_set_logdev(char *line)
+static int isapnp_set_device(char *line)
 {
-	int idx;
+	int idx, idx1;
 	unsigned int id;
 	char index[16], value[32];
 
 	line = isapnp_get_str(index, line, sizeof(index));
 	isapnp_get_str(value, line, sizeof(value));
-	idx = simple_strtoul(index, NULL, 0);
+	idx = idx1 = simple_strtoul(index, NULL, 0);
 	id = isapnp_parse_id(value);
-	isapnp_info_logdev = isapnp_find_logdev(isapnp_info_dev, id >> 16, id & 0xffff, idx);
-	if (!isapnp_info_logdev) {
-		printk("isapnp: logical device '%s' order %i not found\n", value, idx);
+	isapnp_info_device = isapnp_find_dev(isapnp_info_card, id >> 16, id & 0xffff, NULL);
+	while (isapnp_info_device && idx-- > 0)
+		isapnp_info_device = isapnp_find_dev(isapnp_info_card, id >> 16, id & 0xffff, isapnp_info_device);
+	if (isapnp_info_device == NULL) {
+		printk("isapnp: device '%s' order %i not found\n", value, idx);
 		return 1;
 	}
-	isapnp_logdev(isapnp_info_logdev->number);
+	isapnp_device(isapnp_info_device->devfn);
 	return 0;
 }
 
 static int isapnp_autoconfigure(void)
 {
-	struct isapnp_config cfg;
-
-	if (isapnp_config_init(&cfg, isapnp_info_logdev)<0) {
-		printk("isapnp: auto-configuration initialization failed, skipping\n");
+	if (isapnp_info_device == NULL) {
+		printk("isapnp: device is not set\n");
 		return 0;
 	}
-	if (isapnp_configure(&cfg)<0) {
-		printk("isapnp: auto-configuration failed (out of resources?), skipping\n");
+	if (isapnp_info_device->active)
+		isapnp_info_device->deactivate(isapnp_info_device);
+	if (isapnp_info_device->prepare(isapnp_info_device) < 0) {
+		printk("isapnp: cannot prepare device for the activation");
+		return 0;
+	}
+	if (isapnp_info_device->activate(isapnp_info_device) < 0) {
+		printk("isapnp: cannot activate device");
 		return 0;
 	}
 	return 0;
@@ -721,9 +954,26 @@ static int isapnp_set_port(char *line)
 		printk("isapnp: wrong port value 0x%x\n", port);
 		return 1;
 	}
-	isapnp_cfg_set_word(ISAPNP_CFG_PORT + (idx << 1), port);
+	isapnp_write_word(ISAPNP_CFG_PORT + (idx << 1), port);
+	if (!isapnp_info_device->resource[idx].flags)
+		return 0;
+	if (isapnp_info_device->resource[idx].flags & IORESOURCE_AUTO) {
+		isapnp_info_device->resource[idx].start = port;
+		isapnp_info_device->resource[idx].end += port - 1;
+		isapnp_info_device->resource[idx].flags &= ~IORESOURCE_AUTO;
+	} else {
+		isapnp_info_device->resource[idx].end -= isapnp_info_device->resource[idx].start;
+		isapnp_info_device->resource[idx].start = port;
+		isapnp_info_device->resource[idx].end += port;
+	}
 	return 0;
 }
+
+static void isapnp_set_irqresource(struct resource *res, int irq)
+{
+	res->start = res->end = irq;
+	res->flags = IORESOURCE_IRQ;
+}
  
 static int isapnp_set_irq(char *line)
 {
@@ -744,10 +994,17 @@ static int isapnp_set_irq(char *line)
 		printk("isapnp: wrong IRQ value %i\n", irq);
 		return 1;
 	}
-	isapnp_cfg_set_byte(ISAPNP_CFG_IRQ + (idx << 1), irq);
+	isapnp_write_byte(ISAPNP_CFG_IRQ + (idx << 1), irq);
+	isapnp_set_irqresource(isapnp_info_device->irq_resource + idx, irq);
 	return 0;
 }
  
+static void isapnp_set_dmaresource(struct resource *res, int dma)
+{
+	res->start = res->end = dma;
+	res->flags = IORESOURCE_DMA;
+}
+ 
 static int isapnp_set_dma(char *line)
 {
 	int idx, dma;
@@ -765,7 +1022,8 @@ static int isapnp_set_dma(char *line)
 		printk("isapnp: wrong DMA value %i\n", dma);
 		return 1;
 	}
-	isapnp_cfg_set_byte(ISAPNP_CFG_DMA + idx, dma);
+	isapnp_write_byte(ISAPNP_CFG_DMA + idx, dma);
+	isapnp_set_dmaresource(isapnp_info_device->dma_resource + idx, dma);
 	return 0;
 }
  
@@ -784,7 +1042,18 @@ static int isapnp_set_mem(char *line)
 		return 1;
 	}
 	mem >>= 8;
-	isapnp_cfg_set_word(ISAPNP_CFG_MEM + (idx<<2), mem & 0xffff);
+	isapnp_write_word(ISAPNP_CFG_MEM + (idx<<2), mem & 0xffff);
+	if (!isapnp_info_device->resource[idx + 8].flags)
+		return 0;
+	if (isapnp_info_device->resource[idx + 8].flags & IORESOURCE_AUTO) {
+		isapnp_info_device->resource[idx + 8].start = mem & ~0x00ffff00;
+		isapnp_info_device->resource[idx + 8].end += (mem & ~0x00ffff00) - 1;
+		isapnp_info_device->resource[idx + 8].flags &= ~IORESOURCE_AUTO;
+	} else {
+		isapnp_info_device->resource[idx + 8].end -= isapnp_info_device->resource[idx + 8].start;
+		isapnp_info_device->resource[idx + 8].start = mem & ~0x00ffff00;
+		isapnp_info_device->resource[idx + 8].end += mem & ~0x00ffff00;
+	}
 	return 0;
 }
  
@@ -804,13 +1073,13 @@ static int isapnp_poke(char *line, int what)
 	}
 	switch (what) {
 	case 1:
-		isapnp_cfg_set_word(reg, val);
+		isapnp_write_word(reg, val);
 		break;
 	case 2:
-		isapnp_cfg_set_dword(reg, val);
+		isapnp_write_dword(reg, val);
 		break;
 	default:
-		isapnp_cfg_set_byte(reg, val);
+		isapnp_write_byte(reg, val);
 		break;
 	}
 	return 0;
@@ -821,28 +1090,30 @@ static int isapnp_decode_line(char *line)
 	char cmd[32];
 
 	line = isapnp_get_str(cmd, line, sizeof(cmd));
-	if (!strcmp(cmd, "dev") || !strcmp(cmd, "device"))
-		return isapnp_set_device(line);
+	if (!strcmp(cmd, "card"))
+		return isapnp_set_card(line);
 	if (!strcmp(cmd, "csn"))
 		return isapnp_select_csn(line);
-	if (!isapnp_info_dev) {
-		printk("isapnp: device isn't selected\n");
+	if (isapnp_info_card == NULL) {
+		printk("isapnp: card is not selected\n");
 		return 1;
 	}
-	if (!strcmp(cmd, "logdev") || !strcmp(cmd, "logical"))
-		return isapnp_set_logdev(line);
-	if (!isapnp_info_logdev) {
-		printk("isapnp: logical device isn't selected\n");
+	if (!strncmp(cmd, "dev", 3))
+		return isapnp_set_device(line);
+	if (isapnp_info_device == NULL) {
+		printk("isapnp: device is not selected\n");
 		return 1;
 	}
-	if (!strcmp(cmd, "auto") || !strcmp(cmd, "autoconfigure"))
+	if (!strncmp(cmd, "auto", 4))
 		return isapnp_autoconfigure();
-	if (!strcmp(cmd, "act") || !strcmp(cmd, "activate")) {
-		isapnp_activate(isapnp_info_logdev->number);
+	if (!strncmp(cmd, "act", 3)) {
+		isapnp_activate(isapnp_info_device->devfn);
+		isapnp_info_device->active = 1;
 		return 0;
 	}
-	if (!strcmp(cmd, "deact") || !strcmp(cmd, "deactivate")) {
-		isapnp_deactivate(isapnp_info_logdev->number);
+	if (!strncmp(cmd, "deact", 5)) {
+		isapnp_deactivate(isapnp_info_device->devfn);
+		isapnp_info_device->active = 0;
 		return 0;
 	}
 	if (!strcmp(cmd, "port"))
@@ -851,7 +1122,7 @@ static int isapnp_decode_line(char *line)
 		return isapnp_set_irq(line);
 	if (!strcmp(cmd, "dma"))
 		return isapnp_set_dma(line);
-	if (!strcmp(cmd, "mem") || !strcmp(cmd, "memory"))
+	if (!strncmp(cmd, "mem", 3))
 		return isapnp_set_mem(line);
 	if (!strcmp(cmd, "poke"))
 		return isapnp_poke(line, 0);
@@ -874,8 +1145,8 @@ static void isapnp_info_write(isapnp_info_buffer_t *buffer)
 
 	if (buffer->size <= 0)
 		return;
-	isapnp_info_dev = NULL;
-	isapnp_info_logdev = NULL;
+	isapnp_info_card = NULL;
+	isapnp_info_device = NULL;
 	for (idx = 0; idx < buffer->size; idx++) {
 		c = buffer->buffer[idx];
 		if (c == '\n') {
@@ -894,6 +1165,6 @@ static void isapnp_info_write(isapnp_info_buffer_t *buffer)
 		line[idx1++] = c;
 	}
       __end:
-	if (isapnp_info_dev)
+	if (isapnp_info_card)
 		isapnp_cfg_end();
 }
